/*
##############################################################################
#
# MODULE:       r.avaflow.main.c
#
# AUTHORS:      Martin Mergili and Shiva P. Pudasaini
# CONTRIBUTORS: Massimiliano Alvioli, Wolfgang Fellin, Jan-Thomas Fischer,
#               Markus Metz, Alexander Ostermann, Matthias Rauter
#
# PURPOSE:      The simulation model for avalanche and debris flows
#               Flow routing script
#
# COPYRIGHT:    (c) 2008 - 2016 by the authors
#               (c) 2013 - 2016 by the BOKU University, Vienna
#               (c) 2015 - 2016 by the University of Vienna
#               (c) 2014 - 2016 by the University of Bonn
#               (c) 2000 - 2016 by the GRASS Development Team
#
# VERSION:      20161104 (4 November 2016)
#
#               This program is free software under the GNU General Public
#               License (>=v2). Read the file COPYING that comes with GRASS
#               for details.
#
##############################################################################
*/


// Use of GRASS GIS


#define WITHGRASS


// Libraries and structures

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>


#ifdef WITHGRASS


    #include <grass/gmath.h>
    #include <grass/gis.h>
    #include <grass/gprojects.h>
    #include <grass/glocale.h>
    #include <grass/raster.h>
    #include <grass/segment.h>


#endif


struct ico { // input constants and control variables

    char *MAINMAPSET; int MULT; int TOPO; int MODEL; int M; int N; int IMAX; int ENTR; int ENTR2; int CENTR; int RELM; int RELM2; int DELTA;
    double PI; double UNDEF; double HFLOWMIN; double CSZ; double BDWEST; double BDNORTH; double BDSOUTH; double GRAVITY; double CFL[2];
    double IMPTHR[3]; int CORRHEIGHT; int DIFFCTRL; int CONSVOL; int NOOSC; int ENTRAINMENT; int STOPPING;


#ifdef WITHGRASS


    double NSEGRC; double NSEGS;


#endif


};

struct flow { // flow parameters

    double RHO; double PHI; double MY; double EPS; double RPROD; double RDEC; double R0;
    double RHOS; double RHOF; double VM; double DELTA0; double DELTA; double CHI; double XI; double VTERM; double P;
    double REP; double JE; double NR; double NRA; double GAMMA; double AMBDRAG;
};

struct base { // basal surface parameters

    double CENTR0; double CENTR; double CHEZY; double RHOS; double RHOF; double MY;
};


// Functions

int **alloc_imatrix(int mrows, int mcols) { // function for allocating 2D integer arrays
    int **mm;
    int mi;
    mm = (int **) calloc(mrows, sizeof(int *));
    mm[0] = (int *) calloc(mrows * mcols, sizeof(int));
    for (mi = 1; mi < mrows; mi++)
    mm[mi] = mm[mi - 1] + mcols;
    return mm;
}

double **alloc_dmatrix(int mrows, int mcols) { // function for allocating 2D double arrays
    double **mm;
    int mi;
    mm = (double **) calloc(mrows, sizeof(double *));
    mm[0] = (double *) calloc(mrows * mcols, sizeof(double));
    for (mi = 1; mi < mrows; mi++)
    mm[mi] = mm[mi - 1] + mcols;
    return mm;
}

char **alloc_cmatrix(int mrows, int mcols) { // function for allocating 2D char arrays
    char **mm;
    int mi;
    mm = (char **) calloc(mrows, sizeof(char *));
    mm[0] = (char *) calloc(mrows * mcols, sizeof(char));
    for (mi = 1; mi < mrows; mi++)
    mm[mi] = mm[mi - 1] + mcols;
    return mm;
}

double ***alloc_dmatrix3 (int mrows, int mcols, int mdepths) { // function for allocating 3D double arrays
    double ***mm;
    int mi, mj;
    mm = (double***)calloc(mrows, sizeof(double**));

    for (mi=0; mi<mrows; mi++) {
        mm[mi] = (double**)calloc(mcols, sizeof(double*));

        for (mj=0; mj<mcols; mj++) {
            mm[mi][mj] = (double*)calloc(mdepths, sizeof(double));
        }
    }
    return mm;
}
void free_dmatrix3(double ***mm,int mi,int mj)
{
    int i,j;

    for(i=0;i<mi;i++)
    {
        for(j=0;j<mj;j++)
        {
            free(mm[i][j]);
        }
        free(mm[i]);
    }
    free(mm);
}

int fiparam ( FILE *gfparam ) { // function for reading integer parameters

    int gparam;
    char gsparam[25];
    char **strtodhlp=NULL;
    fgets(gsparam, 25, gfparam);
    gparam=strtod(gsparam, strtodhlp);
    return gparam;
}

double fdparam ( FILE *gfparam ) { // function for reading double parameters

    double gparam;
    char gsparam[25];
    char **strtodhlp=NULL;
    fgets(gsparam, 25, gfparam);
    gparam=strtod(gsparam, strtodhlp);
    return gparam;
}

char *fcparam ( FILE *gfparam ) { // function for reading character parameters

    char *gsparam = (char*) calloc ( 1000, sizeof(char));
    fgets(gsparam, 1000, gfparam);
    gsparam[strnlen(gsparam, 1010) - 1] = '\0';
    return gsparam;
}

char *flparam ( FILE *gfparam ) { // function for reading ascii raster line

    int asclinemax = 100000;
    char *gsparam2 = (char*) calloc (( asclinemax + 20 ), sizeof(char));
    fgets(gsparam2, asclinemax, gfparam);
    gsparam2[strnlen(gsparam2, asclinemax+10) - 1] = '\0';
    return gsparam2;
}

int *fin ( int gi, int *gpx, int *gpy, double *gpelev, struct ico sico ) { // function for identifying pixel environment

    int *gin, gj;

    gin = (int*) calloc( 9, sizeof(int));

    gin[0] = gi;
    if ( sico.TOPO == 1 ) { // for slope mode (rectangular raster over all pixels):
        if ( gi%sico.N == 0 || (gi+1)%sico.N == 0 || gi < sico.N || gi > sico.IMAX - sico.N || gpelev[gi] == sico.UNDEF ) {
            // edge pixels (to be excluded from computations):
            gin[1] = sico.IMAX;
            gin[2] = sico.IMAX;
            gin[3] = sico.IMAX;
            gin[4] = -1;
            gin[5] = -1;
            gin[6] = -1;
            gin[7] = sico.IMAX;
            gin[8] = -1;
        }
        else { // pixels to be included in computation:
            gin[1] = gi + 1;
            gin[2] = gi + sico.N;
            gin[3] = gi + sico.N + 1;
            gin[4] = gi - 1;
            gin[5] = gi - sico.N;
            gin[6] = gi - sico.N - 1;
            gin[7] = gi + sico.N - 1;
            gin[8] = gi - sico.N + 1;
        }
    }
    else { // for channel mode (irregularly shaped raster):

        gj = gi;
        while ( ( gpx[gj] != gpx[gi] || gpy[gj] != gpy[gi]+1 ) && gpx[gj] < sico.IMAX && gpx[gj] < gpx[gi]+2 ) gj += 1; gin[1] = gj;
        if ( gpx[gj] != gpx[gi] || gpy[gj] != gpy[gi]+1 || gpelev[gi] == sico.UNDEF ) gin[1] = sico.IMAX; // x, y+1

        gj = gi;
        while ( ( gpx[gj] != gpx[gi]+1 || gpy[gj] != gpy[gi] ) && gj < sico.IMAX && gpx[gj] < gpx[gi]+2 ) gj += 1; gin[2] = gj;
        if ( gpx[gj] != gpx[gi]+1 || gpy[gj] != gpy[gi] || gpelev[gi] == sico.UNDEF ) gin[2] = sico.IMAX; // x+1, y

        gj = gi;
        while ( ( gpx[gj] != gpx[gi]+1 || gpy[gj] != gpy[gi]+1 ) && gj < sico.IMAX && gpx[gj] < gpx[gi]+2 ) gj += 1; gin[3] = gj;
        if ( gpx[gj] != gpx[gi]+1 || gpy[gj] != gpy[gi]+1 || gpelev[gi] == sico.UNDEF ) gin[3] = sico.IMAX; // x+1, y+1

        gj = gi;
        while ( ( gpx[gj] != gpx[gi] || gpy[gj] != gpy[gi]-1 ) && gj >= 0 && gpx[gj] > gpx[gi]-2 ) gj -= 1; gin[4] = gj;
        if ( gpx[gj] != gpx[gi] || gpy[gj] != gpy[gi]-1 || gpelev[gi] == sico.UNDEF ) gin[4] = -1; // x, y-1

        gj = gi;
        while ( ( gpx[gj] != gpx[gi]-1 || gpy[gj] != gpy[gi] ) && gj >= 0 && gpx[gj] > gpx[gi]-2 ) gj -= 1; gin[5] = gj;
        if ( gpx[gj] != gpx[gi]-1 || gpy[gj] != gpy[gi] || gpelev[gi] == sico.UNDEF ) gin[5] = -1; // x-1, y

        gj = gi;
        while ( ( gpx[gj] != gpx[gi]-1 || gpy[gj] != gpy[gi]-1 ) && gj >= 0 && gpx[gj] > gpx[gi]-2 ) gj -= 1; gin[6] = gj;
        if ( gpx[gj] != gpx[gi]-1 || gpy[gj] != gpy[gi]-1 || gpelev[gi] == sico.UNDEF ) gin[6] = -1; // x-1, y-1

        gj = gi;
        while ( ( gpx[gj] != gpx[gi]+1 || gpy[gj] != gpy[gi]-1 ) && gj < sico.IMAX && gpx[gj] > gpx[gi]-2 ) gj += 1; gin[7] = gj;
        if ( gpx[gj] != gpx[gi]+1 || gpy[gj] != gpy[gi]-1 || gpelev[gi] == sico.UNDEF ) gin[7] = sico.IMAX; // x+1, y-1

        gj = gi;
        while ( ( gpx[gj] != gpx[gi]-1 || gpy[gj] != gpy[gi]+1 ) && gj >= 0 && gpx[gj] > gpx[gi]-2 ) gj -= 1; gin[8] = gj;
        if ( gpx[gj] != gpx[gi]-1 || gpy[gj] != gpy[gi]+1 || gpelev[gi] == sico.UNDEF ) gin[8] = -1; // x-1, y+1
    }

    return gin;
}

double fbeta ( double gelevm, double gelevp, struct ico sico ) { // function for topographic slopes in x and y direction

    double gbeta;
    gbeta = atan (( gelevm - gelevp ) / ( 2 * sico.CSZ ));

    return gbeta;
}

double fbetaxy ( double gbetax, double gbetay ) { // function for topographic slope

    double gbetaxy;
    if ( gbetax != 0 || gbetay != 0 ) gbetaxy = atan ( pow ( pow ( tan ( gbetax) , 2 ) + pow ( tan ( gbetay ), 2 ), 0.5 ));
    else gbetaxy = 0;
    return gbetaxy;
}

double falpha ( double gbetax, double gbetay, struct ico sico ) { // function for topographic aspect

    double galpha;

    if ( gbetax == 0 && gbetay == 0 ) galpha = 0;
    else if ( gbetax == 0 && gbetay > 0 ) galpha = sico.PI / 2;
    else if ( gbetax == 0 && gbetay < 0 ) galpha = 3 * sico.PI / 2;
    else if ( gbetax >= 0 ) galpha = atan( tan( gbetay ) / tan( gbetax ));
    else if ( gbetax < 0 ) galpha = atan( tan( gbetay ) / tan( gbetax )) + sico.PI;

    if ( galpha < 0 ) galpha += 2 * sico.PI;

    return galpha;
}

double falphav ( double gmomx, double gmomy, struct ico sico ) { // function for direction of movement

    double galphav;

    if ( gmomx == 0 && gmomy == 0 ) galphav = 0;
    else if ( gmomx == 0 && gmomy > 0 ) galphav = sico.PI / 2;
    else if ( gmomx == 0 && gmomy < 0 ) galphav = 3 * sico.PI / 2;
    else if ( gmomx >= 0 ) galphav = atan( gmomy / gmomx );
    else if ( gmomx < 0 ) galphav = atan( gmomy / gmomx ) + sico.PI;

    if ( galphav < 0 ) galphav += 2 * sico.PI;

    return galphav;
}

double fgz ( double gbetaxy, struct ico sico ) { // function for z component of gravity

    double ggz;
    ggz = -sico.GRAVITY * cos ( gbetaxy );
    return ggz;
}

double fgd ( double gbetax, double gbetay, double gbetad, int gfunc, struct ico sico ) { // function for x and y components of gravity

    double gbetaxy, galpha, ggd;
    gbetaxy = fbetaxy ( gbetax, gbetay );
    ggd = sico.GRAVITY * sin ( gbetaxy );
    if ( gbetaxy == 0 ) ggd = 0;
    else {
        galpha = falpha ( gbetax, gbetay, sico );
        if ( gfunc == 1 ) ggd = ggd * cos ( galpha ); else ggd = ggd * sin ( galpha );
        if ( gbetad > 0 ) ggd = pow( pow (ggd, 2 ), 0.5); else ggd = pow( pow (ggd, 2 ), 0.5) * -1;
    }
    return ggd;
}

double **finhyd ( char *gname, int ghydtmax, int ghydtmaxx ) { // function for input of hydrograph

    FILE *gfhyd;
    char gpath[200], *gtesth, *gtesth2;
    int gi, gj;
    char *tok;
    char gdelim[] = " \t\n";
    char **strtodhlp=NULL;

    double **ghyd = alloc_dmatrix( ghydtmaxx+1, 5 ); // allocating memory to array of input hydrograph data

    sprintf(gpath, "%s", gname );
    gfhyd=fopen(gpath, "r");

    gtesth = fcparam ( gfhyd );
    free( gtesth );
    for ( gi=0; gi<=ghydtmax; gi++ ) {
        gtesth = fcparam ( gfhyd );
        tok=strtok(gtesth, gdelim);
        gj = 0;
        while ( tok != NULL ) {
            gtesth2=tok;
            tok=strtok(NULL, gdelim);
            ghyd[gi][gj] = strtod (gtesth2, strtodhlp);
            gj += 1;
        }
        free(gtesth);
    }

    fclose(gfhyd);
    return ghyd;
}

int *fhydp ( int ghydx, int ghydy, double ghydl, int *gpx, int *gpy, double galpha,
    double ghydlmax, struct ico sico ) { // function for input hydrograph profile

    int gdistx, gdisty, gi, gj, gx0, gy0, gdprex, gdprey, gctrl, *gx, *gy;
    double gd, gdxtest, gdytest;

    int *ghydp = (int*) calloc( (int)(2*ghydlmax/sico.CSZ+2), sizeof(int) );
        // allocating memory to variable for internal coordinates of hydrograph profile

    gdistx = (int)( ghydl * fabs( cos( galpha )) / ( 2 * sico.CSZ ) + 0.5 );
    gdisty = (int)( ghydl * fabs( sin( galpha )) / ( 2 * sico.CSZ ) + 0.5 );

    gdxtest = fabs((float)( gdistx ) / cos( galpha ));
    gdytest = fabs((float)( gdisty ) / sin( galpha ));

    gx = (int*) calloc( 4 * (( gdistx + 1 ) * ( gdisty + 1 )), sizeof(int));
    gy = (int*) calloc( 4 * (( gdistx + 1 ) * ( gdisty + 1 )), sizeof(int));

    gctrl = 1;

    if ( galpha == 0 || galpha == sico.PI / 2 || gdytest == 0 ) {

        gx0 = ghydx - gdistx;
        gy0 = ghydy;

        gx[gctrl] = gx0;
        gy[gctrl] = gy0;

        while ( gx[gctrl] <= ghydx + gdistx ) {

            gctrl += 1;
            gx[gctrl] = gx0 + gctrl;
            gy[gctrl] = gy0;
        }

    }
    else if ( galpha == sico.PI || galpha == 3 * sico.PI / 2 || gdxtest == 0 ) {

        gx0 = ghydx;
        gy0 = ghydy - gdisty;

        gx[gctrl] = gx0;
        gy[gctrl] = gy0;

        while ( gy[gctrl] <= ghydy + gdisty ) {

            gctrl += 1;
            gx[gctrl] = gx0;
            gy[gctrl] = gy0 + gctrl;
        }

    }
    else {

        if ( gdxtest > gdytest ) {

            gx0 = ghydx - gdistx;

            if (( galpha > 0 && galpha < sico.PI / 2 ) || ( galpha > sico.PI && galpha < 3 * sico.PI / 2 ))
                gy0 = ghydy - gdisty;// (int)( gdxtest * fabs(sin( galpha )) + 0.5 );
            else gy0 = ghydy + gdisty;// (int)( gdxtest * fabs(sin( galpha )) + 0.5 );

            gdprex = 1;
            if (( galpha > 0 && galpha < sico.PI / 2 ) || ( galpha > sico.PI && galpha < 3 * sico.PI / 2 )) gdprey = 1;
            else gdprey = -1;
        }
        else {

            if (( galpha > 0 && galpha < sico.PI / 2 ) || ( galpha > sico.PI && galpha < 3 * sico.PI / 2 ))
                gx0 = ghydx - gdistx;// (int)( gdytest * fabs(cos( galpha )) + 0.5 );
            else gx0 = ghydx + gdistx;//(int)( gdytest * fabs(cos( galpha )) + 0.5 );
            gy0 = ghydy - gdisty;
            if (( galpha > 0 && galpha < sico.PI / 2 ) || ( galpha > sico.PI && galpha < 3 * sico.PI / 2 )) gdprex = 1;
            else gdprex = -1;
            gdprey = 1;
        }
        if ( fabs( 1 / cos( galpha )) < fabs(1 / sin ( galpha ))) gd = fabs( 1 / cos( galpha )); else gd = fabs(1 / sin ( galpha ));

        gx[gctrl] = gx0;
        gy[gctrl] = gy0;

        while ( gx[gctrl] <= ghydx + gdistx && gy[gctrl] <= ghydy + gdisty
            && gx[gctrl] >= ghydx - gdistx && gy[gctrl] >= ghydy - gdisty ) {

            gctrl += 1;
            gx[gctrl] = gx0 + gdprex * (int)( gctrl * gd * fabs(cos( galpha )) + 0.5 );
            gy[gctrl] = gy0 + gdprey * (int)( gctrl * gd * fabs(sin( galpha )) + 0.5 );
        }
    }
    gctrl -= 1;
    ghydp[0] = gctrl;

    for ( gi=1; gi<sico.IMAX; gi++ ) {
        for ( gj=1; gj<=gctrl; gj++ ) {
            if ( gpx[gi] == gx[gj] && gpy[gi] == gy[gj] ) {
                ghydp[gj] = gi;
            }
        }
    }

    free(gx); free(gy);

    return ghydp;
}


// F1 -- START -- Conversion of heights to depths ------------------------------------------------------------------

double fconvin ( double *ghflow, double *ghflow2, double ghflowmax, int gmaterial, int *gpx, int *gpy, double *gpelev, double *gbetax,
    double *gbetay, double *gbetaxy, int gi, struct ico sico ) { // function for converting heights into depths

    int *gin, gj, gl, gctrlv;
    double gdflowi, ghflowi, ghflowj, grflowi, grflowj, gdxtest, gdytest, gdxytest, gdxytestmin, gdz;

    if ( sico.MODEL == 1 ) { // depth at target pixel:
        ghflowi = ghflow[gi];
        grflowi = 1;
        if ( ghflowi != 0 ) gdflowi = ghflowi * cos ( gbetaxy[gi] );
        else gdflowi = 0;
    }
    else if ( sico.MODEL == 2 ) {

        ghflowi = ghflow[gi] + ghflow2[gi];

        if ( ghflowi != 0 ) {
            if ( gmaterial == 1 ) grflowi = ghflow[gi] / ghflowi;
            else if ( gmaterial == 2 ) grflowi = ghflow2[gi] / ghflowi;
            gdflowi = ghflowi * grflowi * cos ( gbetaxy[gi] );
        }
        else gdflowi = 0;
    }

    gin = fin( gi, gpx, gpy, gpelev, sico ); // identifying pixel neighbourhood
    gctrlv = 1; for ( gl=1; gl<9; gl++ ) if ( gin[gl] < 0 || gin[gl] >= sico.IMAX ) gctrlv = 0;
    free(gin);

    if ( gctrlv == 0 ) gdflowi = 0;
    else if ( sico.CORRHEIGHT == 2 && ghflowi * tan( gbetaxy[gi] ) > 0.5 * sico.CSZ ) { // if advanced conversion is necessary:

        gdxytestmin = sico.UNDEF*-1; // initial value of comparison of horizontal component of depth and shift

        for ( gj=0; gj<sico.IMAX; gj++ ) { // loop over all pixels:

            if ( gpelev[gj] != sico.UNDEF ) {

                if (( sico.MODEL == 1 && ghflow[gj] != 0 ) || ( sico.MODEL == 2 && ghflow[gj] + ghflow2[gj] != 0 )) {
                if ( pow( sico.CSZ * sico.CSZ * (double)( pow( gpx[gj] - gpx[gi], 2 ) + pow( gpy[gj] - gpy[gi], 2 )), 0.5 )
                    <= ghflowmax * sin( gbetaxy[gi] )) {

                    if ( sico.MODEL == 1 ) { // height and ratio at shift location:
                        ghflowj = ghflow[gj];
                        if ( ghflowj != 0 ) grflowj = 1;
                    }
                    else if ( sico.MODEL == 2 ) {
                        ghflowj = ghflow[gj] + ghflow2[gj];
                        if ( ghflowj != 0 && gmaterial == 1 ) grflowj = ghflow[gj] / ghflowj;
                        else if ( ghflowj != 0 && gmaterial == 2 ) grflowj = ghflow2[gj] / ghflowj;
                    }

                    gdz = gpelev[gj] - gpelev[gi]; // elevation difference between shift location and target pixel
                    gdxtest = sico.CSZ * (double)( gpx[gj] - gpx[gi] ) - ( ghflowj + gdz ) * tan( gbetax[gi] );
                    gdytest = sico.CSZ * (double)( gpy[gj] - gpy[gi] ) - ( ghflowj + gdz ) * tan( gbetay[gi] );
                    gdxytest = pow( pow( gdxtest , 2 ) + pow( gdytest , 2 ), 0.5 );
                        // comparison of horizontal component of depth and shift

                    if ( gdxytest < gdxytestmin ) {
                        if ( ghflowj != 0 ) gdflowi = grflowj * ( ghflowj + gdz ) / cos ( gbetaxy[gi] ); // depth
                        gdxytestmin = gdxytest;
                    }
                }
            }}
        }
    }

    if ( sico.CORRHEIGHT == 0 ) gdflowi = ghflowi * grflowi;

    return gdflowi;
}

// F1 -- STOP --- Conversion of heights to depths ------------------------------------------------------------------


double *ff1 ( double ghflow, double gvflowx, double gvflowy, double gdvflow, double grflow, double ggz, struct flow sflow, struct ico sico ) {
    // function for fluxes in x direction (Voellmy-Salm one-phase model)

    double *ggf, gkap;

    ggf = (double*) calloc( 4, sizeof(double)); // allocating memory

    if ( gdvflow >= 0 ) gkap = pow ( tan ( sico.PI / 4 - sflow.PHI / 2 ), 2 ); // active earth pressure coefficient
    else gkap = pow ( tan ( sico.PI / 4 + sflow.PHI / 2 ), 2 ); // passive earth pressure coefficient

    ggf[0] = ghflow * gvflowx; // fluxes in x direction
    ggf[1] = ghflow * pow( gvflowx, 2 ) - ggz * gkap * pow( ghflow, 2 ) / 2;
    ggf[2] = ghflow * gvflowx * gvflowy;
    ggf[3] = ghflow * grflow * gvflowx;

    return ggf;
}

double *fg1 ( double ghflow, double gvflowx, double gvflowy, double gdvflow, double grflow, double ggz, struct flow sflow, struct ico sico ) {
    // function for fluxes in y direction (Voellmy-Salm one-phase model)

    double *ggg, gkap;

    ggg = (double*) calloc( 4, sizeof(double)); // allocating memory

    if ( gdvflow >= 0 ) gkap = pow ( tan ( sico.PI / 4 - sflow.PHI / 2 ), 2 ); // active earth pressure coefficient
    else gkap = pow ( tan ( sico.PI / 4 + sflow.PHI / 2 ), 2 ); // passive earth pressure coefficient

    ggg[0] = ghflow * gvflowy; // fluxes in y direction
    ggg[1] = ghflow * gvflowx * gvflowy;
    ggg[2] = ghflow * pow( gvflowy, 2 ) - ggz * gkap * pow( ghflow, 2 ) / 2;
    ggg[3] = ghflow * grflow * gvflowy;

    return ggg;
}

double *fs1 ( double ghflow, double gvflowx, double gvflowy, double gvflow, double grflow,
    double ggx, double ggy, double ggz, struct flow sflow, struct ico sico, struct base sbase, double gpotentr ) {
    // function for source terms (Voellmy-Salm one-phase model)

    double *ggs, gsgx, gsgy, gmyr, gepsr, gsfx, gsfy, gprflow, gdrflow, gqentr;

    ggs = (double*) calloc( 4, sizeof(double)); // allocating memory to array for source term

    gsgx = ggx * ghflow; // acceleration terms
    gsgy = ggy * ghflow;

    gmyr = sflow.MY * exp( -grflow / sflow.R0 ); // energy-dependent friction coefficients
    gepsr = sflow.EPS * exp( grflow / sflow.R0 );

    if ( gvflow == 0 ) gsfx = 0;
    else gsfx = gvflowx / gvflow * ( gmyr * -ggz * ghflow + sico.GRAVITY * pow ( gvflow, 2 )) / gepsr; // friction terms
    if ( gvflow == 0 ) gsfy = 0;
    else gsfy = gvflowy / gvflow * ( gmyr * -ggz * ghflow + sico.GRAVITY * pow ( gvflow, 2 )) / gepsr;

    gprflow = sflow.RPROD * pow ( pow ( gsfx, 2 ) + pow ( gsfy, 2 ), 0.5 ) * gvflow; // production term of random kinetic energy
    gdrflow = sflow.RDEC * ghflow * grflow; // decay term of random kinetic energy

    ggs[0] = 0; // source terms
    ggs[1] = gsgx - gsfx;
    ggs[2] = gsgy - gsfy;
    ggs[3] = gprflow - gdrflow;

    return ggs;
}

double *fk2 ( double gdvflowx, double gdvflowy, struct flow sflow ) {
    // function for eath pressure coefficients (Pudasaini two-phase model)

    double *ggk, gkxa, gkxp, gkxaya, gkxayp, gkxpya, gkxpyp;

    ggk = (double*) calloc( 2, sizeof(double)); // allocating memory

    gkxa = 2 * ( 1 - pow( 1 - pow( cos ( sflow.PHI ) / cos ( sflow.DELTA ), 2 ), 0.5 )) / pow ( cos( sflow.PHI ), 2 ) - 1;
    gkxp = 2 * ( 1 + pow( 1 - pow( cos ( sflow.PHI ) / cos ( sflow.DELTA ), 2 ), 0.5 )) / pow ( cos( sflow.PHI ), 2 ) - 1;

    gkxaya = 0.5 * ( gkxa + 1 - pow(pow( gkxa - 1 , 2 ) + 4 * pow( tan( sflow.DELTA ), 2 ), 0.5 ));
    gkxayp = 0.5 * ( gkxa + 1 + pow(pow( gkxa - 1 , 2 ) + 4 * pow( tan( sflow.DELTA ), 2 ), 0.5 ));
    gkxpya = 0.5 * ( gkxp + 1 - pow(pow( gkxp - 1 , 2 ) + 4 * pow( tan( sflow.DELTA ), 2 ), 0.5 ));
    gkxpyp = 0.5 * ( gkxp + 1 + pow(pow( gkxp - 1 , 2 ) + 4 * pow( tan( sflow.DELTA ), 2 ), 0.5 ));

    if ( gdvflowx >= 0 ) {

        ggk[0]=gkxa; // earth pressure coefficient in x direction
        if ( gdvflowy >= 0 ) ggk[1] = gkxaya; // earth pressure coefficient in y direction
        else ggk[1] = gkxayp;
    }
    else {

        ggk[0]=gkxp;  // earth pressure coefficient in x direction
        if ( gdvflowy >= 0 ) ggk[1] = gkxpya; // earth pressure coefficient in y direction
        else ggk[1] = gkxpyp;
    }

    return ggk;
}

double *ff2 ( double ghflow, double ghflow2, double gvflowx, double gvflowx2, double gvflowy, double gvflowy2,
    double gkx, double ggz, struct flow sflow, struct ico sico ) { // function for fluxes in x direction  (Pudasaini two-phase model)

    double *ggf, galpha, ggamma;

    ggf = (double*) calloc( 6, sizeof(double)); // allocating memory

    if ( ghflow + ghflow2 > sico.HFLOWMIN ) galpha = ghflow / ( ghflow + ghflow2 ); else galpha = 0;
        // ratio of solid component of flow depth

    if ( ghflow2 <= sico.HFLOWMIN ) ggamma = 0;
    else ggamma = sflow.GAMMA;

    if ( ghflow > sico.HFLOWMIN ) { // x fluxes for solid components:

        ggf[0] = ghflow * gvflowx;
        ggf[1] = ghflow * ( pow ( gvflowx, 2 ) - ggamma * sflow.VM * ( pow( gvflowx2, 2 ) - pow ( gvflowx, 2 ))
            - ggz * ( 1 - ggamma ) * gkx * ( ghflow + ghflow2 ) / 2 );
        ggf[2] = ghflow * ( gvflowx * gvflowy - ggamma * sflow.VM * ( gvflowx2 * gvflowy2 - gvflowx * gvflowy ));
    }
    else { ggf[0] = 0; ggf[1] = 0; ggf[2] = 0; }

    if ( ghflow2 > sico.HFLOWMIN ) { // x fluxes for fluid components:

        //ggf[3] = ghflow2 * gvflowx2;
        //ggf[4] = ghflow2 * ( pow ( gvflowx2, 2 )
            //+ galpha / ( 1 - galpha ) * sflow.VM * ( pow( gvflowx2, 2 ) - pow ( gvflowx, 2 )) - ggz * ( ghflow + ghflow2 ) / 2 );
        //ggf[5] = ghflow2 * ( gvflowx2 * gvflowy2 + galpha / ( 1 - galpha ) * sflow.VM * ( gvflowx2 * gvflowy2 - gvflowx * gvflowy ));

        ggf[3] = ghflow2*gvflowx2;
        if ( ghflow2 > sico.HFLOWMIN ) ggf[4] = ghflow2*gvflowx2*gvflowx2+sflow.VM*(pow(ghflow2*gvflowx,2)*ghflow/(pow(ghflow2,2)));
        else ggf[4] = 0;
        if ( ghflow > sico.HFLOWMIN) ggf[4] -= sflow.VM*pow(ghflow*gvflowx,2)/ghflow;
        ggf[5] = ghflow2*gvflowx2*gvflowy2+sflow.VM*ghflow*(gvflowx2*gvflowy2-gvflowx*gvflowy);

    }
    else { ggf[3] = 0; ggf[4] = 0; ggf[5] = 0; }

    return ggf;
}

double *fg2 ( double ghflow, double ghflow2, double gvflowx, double gvflowx2, double gvflowy, double gvflowy2,
    double gky, double ggz, struct flow sflow, struct ico sico ) { // function for fluxes in y direction  (Pudasaini two-phase model)

    double *ggg, galpha, ggamma;

    ggg = (double*) calloc( 6, sizeof(double)); // allocating memory

    if ( ghflow + ghflow2 > sico.HFLOWMIN ) galpha = ghflow / ( ghflow + ghflow2 ); else galpha = 0;
        // ratio of solid component of flow depth

    if ( ghflow2 <= sico.HFLOWMIN ) ggamma = 0;
    else ggamma = sflow.GAMMA;

    if ( ghflow > sico.HFLOWMIN ) { // y fluxes for solid components:

        ggg[0] = ghflow * gvflowy;
        ggg[1] = ghflow * ( gvflowx * gvflowy - ggamma * sflow.VM * ( gvflowx2 * gvflowy2 - gvflowx * gvflowy ));
        ggg[2] = ghflow * ( pow ( gvflowy, 2 ) - ggamma * sflow.VM * ( pow( gvflowy2, 2 ) - pow ( gvflowy, 2 ))
            - ggz * ( 1 - ggamma ) * gky * ( ghflow + ghflow2 ) / 2 );
    }
    else { ggg[0] = 0; ggg[1] = 0; ggg[2] = 0; }

    if ( ghflow2 > sico.HFLOWMIN ) { // y fluxes for fluid components:

        //ggg[3] = ghflow2 * gvflowy2;
        //ggg[4] = ghflow2 * ( gvflowx2 * gvflowy2 + galpha / ( 1 - galpha ) * sflow.VM * ( gvflowx2 * gvflowy2 - gvflowx * gvflowy ));
        //ggg[5] = ghflow2 * ( pow ( gvflowy2, 2 )
            //+ galpha / ( 1 - galpha ) * sflow.VM * ( pow( gvflowy2, 2 ) - pow ( gvflowy, 2 )) - ggz * ( ghflow + ghflow2 ) / 2 );

        ggg[3] = ghflow2*gvflowy2;
        ggg[4] = ghflow2*gvflowy2*gvflowx2+sflow.VM*ghflow*(gvflowx2*gvflowy2-gvflowx*gvflowy);
        if ( ghflow2 > sico.HFLOWMIN) ggg[5] = ghflow2*gvflowy2*gvflowy2+sflow.VM*(pow(ghflow2*gvflowy2,2)*ghflow/(pow(ghflow2,2)));
        else ggg[5] = 0;
        if ( ghflow > sico.HFLOWMIN) ggg[5] -= sflow.VM*pow(ghflow*gvflowy,2)/ghflow;
    }
    else { ggg[3] = 0; ggg[4] = 0; ggg[5] = 0; }

    return ggg;
}

double *fs2 ( double ghflow, double ghflow2, double gvflowx, double gvflowx2, double gvflowy, double gvflowy2,
    double gghflowx, double gghflowy, double *ggvflowx, double *ggvflowy, double *ggvflowx2, double *ggvflowy2,
    double *ggalpha, double *gelev, double ggx, double ggy, double ggz, struct flow sflow, double *gdx, double *gdy, int gi,
    struct ico sico, double *gbetax, double *gbetay, double *gbetaxy, double gpotentr, double gpotentr2, struct base sbase ) {
        // function for source terms (Pudasaini two-phase model)

    int gctrl_hflow, gctrl_hflow2;
    double *ggs, *gs_s, *gs_f, gkappax, gkappaxy, gkappay, gkappau, gvxw, gvyw, galpha, gs_drag, gs_dragvx, gs_dragvy,
        gvflow, gvflow2, gdragctrl, galphab, glambdas, glambdasb, glambdaf, glambdafb, gny, ghscale, gqentr, gqentr2, ggamma, ggamma2;

    ggs = (double*) calloc( 6, sizeof(double)); // allocating memory to array for source term
    gs_s = (double*) calloc( 6, sizeof(double)); // allocating memory to arrays
    gs_f = (double*) calloc( 6, sizeof(double));

    if ( ghflow2 <= sico.HFLOWMIN ) ggamma = 0;
    else ggamma = sflow.GAMMA;

    if ( ghflow2 <= sico.HFLOWMIN ) ggamma2 = 1;
    else ggamma2 = sflow.GAMMA;


// Curvature

    if ( ghflow > sico.HFLOWMIN && ( gvflowx != 0 || gvflowy != 0 )) {

        gkappax = 8 * sico.CSZ * ( gelev[2] + gelev[5] - 2 * gelev[0] )
            / ( 4 * pow ( sico.CSZ, 2 ) + pow( gelev[2] - gelev[5], 2 ) * pow( 4 * pow ( sico.CSZ, 2 )
            + pow( gelev[2] - gelev[5] , 2 ) + pow( gelev[1] - gelev[4] , 2 ), 0.5 )); // x curvature

        gkappaxy = 2 * sico.CSZ * ( gelev[3] + gelev[6] - gelev[8] - gelev[7] )
            / ( pow( 4 * pow ( sico.CSZ, 2 ) + pow( gelev[2] - gelev[5], 2 ), 0.5 ) * pow( 4 * pow ( sico.CSZ, 2 )
            + pow( gelev[1] - gelev[4] , 2 ), 0.5 ) * pow( 4 * pow( sico.CSZ, 2 ) + pow( gelev[2] - gelev[5] , 2 )
            + pow( gelev[1] - gelev[4] , 2 ), 0.5 )); // xy curvature

        gkappay = 8 * sico.CSZ * ( gelev[1] + gelev[4] - 2 * gelev[0] )
            / ( 4 * pow ( sico.CSZ, 2 ) + pow( gelev[1] - gelev[4], 2 ) * pow( 4 * pow ( sico.CSZ, 2 )
            + pow( gelev[2] - gelev[5] , 2 ) + pow( gelev[1] - gelev[4] , 2 ), 0.5 )); // y curvature

        gkappau = ( pow( gvflowx, 2 ) * gkappax + 2 * gvflowx * gvflowy * gkappaxy + pow( gvflowy, 2 ) * gkappay )
            / ( pow( gvflowx, 2 ) + pow( gvflowy, 2 )); // combination of curvature and velocity
    }
    else gkappau = 0;


// Components of solid flow velocity in x and y directions

    if ( pow( gvflowx, 2) + pow( gvflowy, 2 ) == 0 ) {
        gvxw = 0.0;
        gvyw = 0.0;
    }
    else {
        gvxw = gvflowx / pow( pow( gvflowx, 2) + pow( gvflowy, 2 ), 0.5 ); // x component
        gvyw = gvflowy / pow( pow( gvflowx, 2) + pow( gvflowy, 2 ), 0.5 ); // y component
    }


// Ratio and gradient of ratio of solid flow depth

    if ( ghflow > sico.HFLOWMIN && ghflow2 > sico.HFLOWMIN ) galpha = ghflow / ( ghflow + ghflow2 ); // ratio of solid flow depth
    else if ( ghflow > sico.HFLOWMIN ) galpha = 1;
    else if ( ghflow2 > sico.HFLOWMIN ) galpha = 0;


// Components of source terms

    gctrl_hflow = 0; // control for solid phase:
    if ( ghflow > sico.HFLOWMIN ) gctrl_hflow = 1;

    gctrl_hflow2 = 0; // control for fluid phase:
    if ( ghflow2 > sico.HFLOWMIN ) gctrl_hflow2 = 1;

    if ( gctrl_hflow == 1 ) { // if the solid phase has to be considered:

        gs_s[1] = ggx - gvxw * ( tan( sflow.DELTA ) * ( 1 - ggamma ) * -ggz + ( 1 - ggamma ) * tan( sflow.DELTA ) * gkappau );
        gs_s[2] = ggy - gvyw * ( tan( sflow.DELTA ) * ( 1 - ggamma ) * -ggz + ( 1 - ggamma ) * tan( sflow.DELTA ) * gkappau );

        gs_f[1] = (-ggz*ggamma*ghflow*gghflowx)/(ghflow+ghflow2);
        gs_f[2] = (-ggz*ggamma*ghflow*gghflowy)/(ghflow+ghflow2);
    }
    else { gs_s[1] = 0; gs_s[2] = 0; gs_f[1] = 0; gs_f[2] = 0; }

    if ( gctrl_hflow2 == 1 ) { // if the fluid phase has to be considered:

        gs_s[4] = (sico.GRAVITY*ghflow2*tan(gbetax[gi]))/(ghflow+ghflow2);
        gs_s[5] = (sico.GRAVITY*ghflow2*tan(gbetay[gi]))/(ghflow+ghflow2);

        gs_f[4] = (-ggz*ghflow2*gghflowx+(ghflow+ghflow2)/sflow.NR*((2.*(ggvflowx2[2]-2.*gvflowx2+ggvflowx2[5])/(gdx[gi]*gdx[gi])
            +(ggvflowx2[1]-2.*gvflowx2+ggvflowx2[4])/(gdy[gi]*gdy[gi])+(ggvflowy2[3]-ggvflowy2[7]-ggvflowy2[8]+ggvflowy2[6])
            /(4*gdx[gi]*gdy[gi])) -sflow.CHI*gvflowx2/(pow(ghflow+ghflow2,2)))-(1./sflow.NRA*(ghflow+ghflow2))*(2*((ggalpha[2]-2.*galpha
            +ggalpha[5])/(gdx[gi]*gdx[gi])*(gvflowx2-gvflowx)+(ggalpha[2]-ggalpha[5])/(2.*gdx[gi])*((ggvflowx2[2]-ggvflowx[2])
            -(ggvflowx2[5]-ggvflowx[5]))/(2.*gdx[gi]))+(ggalpha[3]-ggalpha[7]-ggalpha[8]+ggalpha[6])/(4.*gdx[gi]*gdy[gi])*(gvflowy2-gvflowy)
            +(ggalpha[2]-ggalpha[5])/(2.*gdx[gi])*((ggvflowy2[1]-ggvflowy[1])-(ggvflowy2[4]-ggvflowy[4]))/(2.*gdy[gi])+(ggalpha[1]
            -2.*galpha+ggalpha[4])/(gdy[gi]*gdy[gi])*(gvflowx2-gvflowx) +(ggalpha[1]-ggalpha[4])/(2.*gdy[gi])*((ggvflowx2[1]-ggvflowx[1])
            -(ggvflowx2[4]-ggvflowx[4]))/(2.*gdy[gi])-sflow.XI*(gvflowx2-gvflowx)*galpha/(pow(ghflow+ghflow2,2))))/(ghflow+ghflow2);

        gs_f[5] = (-ggz*ghflow2*gghflowy+(ghflow+ghflow2)/sflow.NR*((2.*(ggvflowy2[1]-2.*gvflowy2+ggvflowy2[4])/(gdy[gi]*gdy[gi])
            +(ggvflowy2[2]-2.*gvflowy2+ggvflowy2[5])/(gdx[gi]*gdx[gi])+(ggvflowx2[3]-ggvflowx2[7]-ggvflowx2[8]+ggvflowx2[6])
            /(4*gdx[gi]*gdy[gi]))-sflow.CHI*gvflowy2/(pow(ghflow+ghflow2,2)))-1./sflow.NRA*(ghflow+ghflow2)*(2.*(((ggalpha[1]-2.*galpha
            +ggalpha[4])/(gdy[gi]*gdy[gi]))*(gvflowy2-gvflowy)+(ggalpha[1]-ggalpha[4])/(2.*gdy[gi])*((ggvflowy2[1]-ggvflowy[1])
            -(ggvflowy2[4]-ggvflowy[4]))/(2.*gdy[gi]))+(ggalpha[3]-ggalpha[7]-ggalpha[8]+ggalpha[6])/(4.*gdx[gi]*gdy[gi])*(gvflowx2-gvflowx)
            +(ggalpha[1]-ggalpha[4])/(2.*gdy[gi])*((ggvflowx2[2]-ggvflowx[2])-(ggvflowx2[5]-ggvflowx[5]))/(2.*gdx[gi])+(ggalpha[2]
            -2.*galpha+ggalpha[5])/(gdx[gi]*gdx[gi])*(gvflowy2-ggvflowy[1])+(ggalpha[2]-ggalpha[5])/(2.*gdx[gi])*((ggvflowy2[2]-ggvflowy[2])
            -(ggvflowy2[5]-ggvflowy[5]))/(2.*gdx[gi])-sflow.XI*(gvflowy2-gvflowy)*galpha/(pow(ghflow+ghflow2,2))))/(ghflow+ghflow2);
    }
    else { gs_s[4] = 0; gs_s[5] = 0; gs_f[4] = 0; gs_f[5] = 0; }

    if ( gctrl_hflow == 1 && gctrl_hflow2 == 1 ) { // if both phases have to be considered:

        gs_drag = galpha * ( 1 - galpha ) * ( 1 - ggamma ) / pow( sflow.VTERM * ( sflow.P * sflow.REP * ggamma / 180
            * pow(( 1 - galpha ) / galpha, 3 ) + ( 1 - sflow.P ) * pow( 1 - galpha, 2 )), sflow.JE ); // drag

        gvflow = pow( pow( gvflowx, 2 ) + pow( gvflowy, 2 ), 0.5 );
        gvflow2 = pow( pow( gvflowx2, 2 ) + pow( gvflowy2, 2 ), 0.5 );

        gs_dragvx = ( gvflowx2 - gvflowx ) * pow( fabs( gvflow2 - gvflow ), sflow.JE - 1 );
        gs_dragvy = ( gvflowy2 - gvflowy ) * pow( fabs( gvflow2 - gvflow ), sflow.JE - 1 );

        if ( galpha < 1 - galpha ) gdragctrl = galpha;
        else gdragctrl = 1 - galpha;

        if ( gvflowx2 > gvflowx && gs_dragvx > ( gvflowx2 - gvflowx ) * ( gdragctrl )) gs_dragvx = ( gvflowx2 - gvflowx ) * ( gdragctrl );
        if ( gvflowx2 < gvflowx && gs_dragvx < ( gvflowx2 - gvflowx ) * ( gdragctrl )) gs_dragvx = ( gvflowx2 - gvflowx ) * ( gdragctrl );

        if ( gvflowy2 > gvflowy && gs_dragvy > ( gvflowy2 - gvflowy ) * ( gdragctrl )) gs_dragvy = ( gvflowy2 - gvflowy ) * ( gdragctrl );
        if ( gvflowy2 < gvflowy && gs_dragvy < ( gvflowy2 - gvflowy ) * ( gdragctrl )) gs_dragvy = ( gvflowy2 - gvflowy ) * ( gdragctrl );
            // constraining drag
    }
    else { gs_drag = 0; gs_dragvx = 0; gs_dragvy = 0; }


// Source terms

    ggs[0] = gs_s[0];

    ggs[1] = ( ghflow + ghflow2 ) * ( galpha * gs_s[1] - galpha * gs_f[1] + gs_drag * gs_dragvx
        - sflow.AMBDRAG * galpha * gvflowx * pow(pow( gvflowx, 2) + pow( gvflowy, 2), 0.5 ));

    ggs[2] = ( ghflow + ghflow2 ) * ( galpha * gs_s[2] - galpha * gs_f[2] + gs_drag * gs_dragvy
        - sflow.AMBDRAG * galpha * gvflowy * pow(pow( gvflowx, 2) + pow( gvflowy, 2), 0.5 ));

    ggs[3] = gs_s[3];

    ggs[4] = ( ghflow + ghflow2 ) * (( 1 - galpha ) * ( gs_s[4] - gs_f[4] ) - 1 / ggamma2 * gs_drag * gs_dragvx
     - sflow.AMBDRAG * ( 1 - galpha ) * gvflowy2 * pow(pow( gvflowx2, 2) + pow( gvflowy2, 2), 0.5 ));

    ggs[5] = ( ghflow + ghflow2 ) * (( 1 - galpha ) * ( gs_s[5] - gs_f[5] ) - 1 / ggamma2 * gs_drag * gs_dragvy
     - sflow.AMBDRAG * ( 1 - galpha ) * gvflowx2 * pow(pow( gvflowx2, 2) + pow( gvflowy2, 2), 0.5 ));

    free( gs_s ); free( gs_f ); // releasing memory

    return ggs;
}

double fsigma ( double gv, double gvm, double gvp, int gdir, double *gdx, double *gdy, int gi ) { // function for slope of vector

    double gsigma, gd;

    if ( gdir == 1 ) gd = gdx[gi]; // pixel spacing in appropriate direction
    else if ( gdir == 2 ) gd = gdy[gi];

    if ( gvp == gv ) gsigma = 0;
    else {
        gsigma = ( gv - gvm ) / ( gvp - gv ); // input for minmod limiter
        if ( gsigma < 0 ) gsigma = 0; // minmod limiter
        else if ( gsigma > 1 ) gsigma = 1;
        gsigma *= ( gvp - gv ) / gd; // slope of vector
    }
    return gsigma;
}


// F1 -- START -- Conversion of depths to heights ------------------------------------------------------------------

double fconvout ( int ggi, double **gdflow, double ghflowmax, int gmaterial, int gtype, int *gpx, int *gpy, double *gpelev, double *gbetax,
    double *gbetay, double *gbetaxy, struct ico sico ) {
    // function for converting depths into heights

    int *gin, ggj, gl, gctrlv, gm1, gm2, gsign;
    double ghflowi, gdflowi, gdflowj, grflowi, grflowj, gdxtest, gdytest, gdxytest, gdxytestmin, gdz, gbetaxyi, gbetaxj, gbetayj, gbetaxyj;

    gsign = 1;

    gbetaxyi = gbetaxy[ggi];

    if ( sico.MODEL == 1 ) { // height at target pixel:

        if ( gtype == 1 ) gm1 = 0;
        else if ( gtype == 2 ) gm1 = 4;
        else if ( gtype == 3 ) gm1 = 8;
        gdflowi = gsign * gdflow[ggi][gm1];

        if ( gdflowi != 0 ) ghflowi = gdflowi / cos ( gbetaxyi );
        else ghflowi = 0;

        grflowi = 1;
    }
    else if ( sico.MODEL == 2 ) {

        if ( gtype == 1 ) { gm1 = 0; gm2 = 3; }
        else if ( gtype == 2 ) { gm1 = 6; gm2 = 7; }
        else if ( gtype == 3 ) { gm1 = 18; gm2 = 20; }
        gdflowi = gsign * ( gdflow[ggi][gm1] + gdflow[ggi][gm2] );

        if ( gdflowi != 0 ) {
            if ( gmaterial == 1 ) grflowi = gsign * gdflow[ggi][gm1] / gdflowi;
            else if ( gmaterial == 2 ) grflowi = gsign * gdflow[ggi][gm2] / gdflowi;
            else if ( gmaterial == 3 ) grflowi = 1;
            ghflowi = gdflowi * grflowi / cos ( gbetaxyi );
        }
        else ghflowi = 0;
    }

    gin = fin( ggi, gpx, gpy, gpelev, sico ); // identifying pixel neighbourhood
    gctrlv = 1; for ( gl=1; gl<9; gl++ ) if ( gin[gl] < 0 || gin[gl] >= sico.IMAX ) gctrlv = 0;
    free(gin);

    if ( gctrlv == 0 ) ghflowi = 0;
    else if ( sico.CORRHEIGHT == 2 && gdflowi * sin( gbetaxyi ) > 0.5 * sico.CSZ ) { // if advanced conversion is necessary:

        gdxytestmin = sico.UNDEF*-1; // initial value of comparison of horizontal component of depth and shift

        for ( ggj=0; ggj<sico.IMAX; ggj++) { // loop over all pixels:

            if ( gpelev[ggj] != sico.UNDEF ) {

                if (( sico.MODEL == 1 && gdflow[ggj][gm1] != 0 ) || ( sico.MODEL == 2 && gdflow[ggj][gm1] + gdflow[ggj][gm2] != 0 )) {
                if ( pow( sico.CSZ * sico.CSZ * (double)(pow( gpx[ggj] - gpx[ggi], 2 ) + pow( gpy[ggj] - gpy[ggi], 2 )), 0.5 )
                     <= ghflowmax ) {

                     gin = fin( ggj, gpx, gpy, gpelev, sico ); // identifying pixel neighbourhood
                     gctrlv = 1; for ( gl=1; gl<9; gl++ ) if ( gin[gl] < 0 || gin[gl] >= sico.IMAX ) gctrlv = 0;
                     free(gin);

                     if ( gctrlv == 1 ) {

                        if ( sico.MODEL == 1 ) { // depth and ratio at shift location:
                            gdflowj = gsign * gdflow[ggj][gm1];
                            if ( gdflowj != 0 ) grflowj = 1;
                        }
                        else if ( sico.MODEL == 2 ) {

                            gdflowj = gsign * ( gdflow[ggj][gm1] + gdflow[ggj][gm2] );
                            if ( gdflowj != 0 && gmaterial == 1 ) grflowj = gsign * gdflow[ggj][gm1] / gdflowj;
                            else if ( gdflowj != 0 && gmaterial == 2 ) grflowj = gsign * gdflow[ggj][gm2] / gdflowj;
                            else if ( gmaterial == 3 ) grflowj = 1;
                        }

                        gbetaxj = gbetax[ggj];
                        gbetayj = gbetay[ggj];
                        gbetaxyj = gbetaxy[ggj];

                        gdz = gpelev[ggj] - gpelev[ggi]; // elevation difference between shift location and target pixel
                        gdxtest = sico.CSZ * (double)( gpx[ggj] - gpx[ggi] ) + ( gdflowj * sin( gbetaxj ));
                        gdytest = sico.CSZ * (double)( gpy[ggj] - gpy[ggi] ) + ( gdflowj * sin( gbetayj ));
                        gdxytest = pow( pow( gdxtest , 2 ) + pow( gdytest , 2 ), 0.5 );
                            // comparison of horizontal component of depth and shift

                        if ( gdxytest < gdxytestmin ) {
                            if ( gdflowj != 0 ) ghflowi = grflowj * ( gdflowj * cos ( gbetaxyj ) + gdz ); // height at target pixel
                            gdxytestmin = gdxytest;
                        }
                    }
                }
            }}
        }
    }

    if ( sico.CORRHEIGHT == 0 ) ghflowi = gdflowi * grflowi;

    return ghflowi;
}

// F1 -- STOP --- Conversion of depths to heights ------------------------------------------------------------------


double fcvelr1 ( double ghflow, double gvflowx, double gvflowy, double gdvflow, struct flow sflow, double *gbetax, double *gbetay, int gi,
    struct ico sico ) { // function for determining time step length (Voellmy-Salm one-phase model)

    double gkxy, gcvelx, gcvely, gcvelr;

    if ( gdvflow >= 0 ) gkxy = pow ( tan ( sico.PI / 4 - sflow.PHI / 2 ), 2 ); // active earth pressure coefficient
    else gkxy = pow ( tan ( sico.PI / 4 + sflow.PHI / 2 ), 2 ); // passive earth pressure coefficient

    gcvelx = fabs( gvflowx ) + pow( sico.GRAVITY * fabs( cos( gbetax[gi] )) * gkxy * ghflow, 0.5 );
    gcvely = fabs( gvflowy ) + pow( sico.GRAVITY * fabs( cos( gbetay[gi] )) * gkxy * ghflow, 0.5 );

    if ( gcvelx > gcvely ) gcvelr = gcvelx; else gcvelr = gcvely;
    return gcvelr;
}

double fcvelr2 ( double ghflow, double ghflow2, double gvflowx, double gvflowx2, double gvflowy, double gvflowy2,
    struct flow sflow, double gkx, double gky, double *gbetax, double *gbetay, int gi, struct ico sico ) {
    // function for determining time step length (Pudasaini two-phase model)

    double gcvelx, gcvely, gcvelx2, gcvely2, gcvel, gcvel2, gcvelr;

    gcvelx = fabs( gvflowx ) + pow( sico.GRAVITY * fabs( cos( gbetax[gi] )) * ( 1 - sflow.GAMMA ) * gkx * ( ghflow + 0.5 * ghflow2 ), 0.5 );
    gcvely = fabs( gvflowy ) + pow( sico.GRAVITY * fabs( cos( gbetay[gi] )) * ( 1 - sflow.GAMMA ) * gky * ( ghflow + 0.5 * ghflow2 ), 0.5 );
    gcvelx2 = fabs( gvflowx2 ) + pow( sico.GRAVITY * fabs( cos( gbetax[gi] )) * ( 0.5 * ghflow + ghflow2 ), 0.5 );
    gcvely2 = fabs( gvflowy2 ) + pow( sico.GRAVITY * fabs( cos( gbetay[gi] )) * ( 0.5 * ghflow + ghflow2 ), 0.5 );

    if ( gcvelx > gcvely ) gcvel = gcvelx; else gcvel = gcvely;
    if ( gcvelx2 > gcvely2 ) gcvel2 = gcvelx2; else gcvel2 = gcvely2;
    if ( gcvel > gcvel2 ) gcvelr = gcvel; else gcvelr = gcvel2;

    return gcvelr;
}

void foutasc ( double **gparam, int *gpx, int *gpy, char *goutmaps, char gname[20], double ghflowmax, double *gpelev, double *gbetax,
    double *gbetay, double *gbetaxy, int gk, struct ico sico ) {
    // function for output of ascii raster maps

    FILE *gfascii;
    char gpath[200];
    int gi, gx, gy;
    double gpout;

    sprintf(gpath, "%s%s.asc", goutmaps, gname ); // writing name of ascii file to string
    gfascii=fopen(gpath, "w"); // opening ascii file

    fprintf( gfascii, "ncols %i\n", sico.N ); // writing header to ascii file
    fprintf( gfascii, "nrows %i\n", sico.M );
    fprintf( gfascii, "xllcorner %.2f\n", sico.BDWEST );
    fprintf( gfascii, "yllcorner %.2f\n", sico.BDSOUTH );
    fprintf( gfascii, "cellsize %.2f\n", sico.CSZ );
    fprintf( gfascii, "NODATA_value %.3f\n", sico.UNDEF );

    gi = 0;
    for ( gx=0; gx<sico.M; gx+=1 ) { for ( gy=0; gy<sico.N; gy+=1 ) { // loop over all pixels:

        if ( gpx[gi] == gx && gpy[gi] == gy ) { // converting depths into heights:

            if ( sico.MODEL == 1 && gk == 0 ) gpout = fconvout( gi, gparam, ghflowmax, 1, 1,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 4 && gk == 0 ) gpout = fconvout( gi, gparam, ghflowmax, 1, 2,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 1 && gk == 8 ) gpout = fconvout( gi, gparam, ghflowmax, 1, 3,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 0 ) gpout = fconvout( gi, gparam, ghflowmax, 1, 1,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 3 ) gpout = fconvout( gi, gparam, ghflowmax, 2, 1,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 6 ) gpout = fconvout( gi, gparam, ghflowmax, 1, 2,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 7 ) gpout = fconvout( gi, gparam, ghflowmax, 2, 2,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 10 ) gpout = fconvout( gi, gparam, ghflowmax, 3, 1,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 17 ) gpout = fconvout( gi, gparam, ghflowmax, 3, 2,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 18 ) gpout = fconvout( gi, gparam, ghflowmax, 1, 3,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 20 ) gpout = fconvout( gi, gparam, ghflowmax, 2, 3,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else if ( sico.MODEL == 2 && gk == 22 ) gpout = fconvout( gi, gparam, ghflowmax, 3, 3,
                gpx, gpy, gpelev, gbetax, gbetay, gbetaxy, sico );
            else gpout = gparam[gi][gk];

            fprintf(gfascii, "%.3f ", gpout ); // writing output to ascii file
            gi += 1;
        }
        else fprintf(gfascii, "0.000 "); // writing zero to file if pixel is not in area of interest
    } fprintf(gfascii, "\n"); }

    fclose(gfascii);

    return;
}

void foutdircoord ( FILE *gf_directions, int gintmode, int *gpx, int *gpy, struct ico sico ) {
    // function for coordinates for output flow direction file

    int gi, git, gy0, gx, gy, intv, intv2;
    double gxmetric, gymetric;

    intv = (int)( sico.N / 75 + 1 ); // pixel interval for writing output

    if ( gintmode == 1 ) { intv2 = 1; gy0 = 0; } // defining shift depending on type of parameter
    else if ( gintmode == 2 ) { intv2 = 2; gy0 = 0; }
    else if ( gintmode == 3 ) { intv2 = 2; gy0 = intv; }

    gi = 0;
    for ( gx=0; gx<sico.M; gx+=intv ) { for ( gy=gy0; gy<sico.N; gy+=(intv2*intv)) { // writing x coordinate to file:

        git = gi;
        while (( gpx[git] != gx || gpy[git] != gy ) && git < sico.IMAX ) git+= 1;
        if ( git < sico.IMAX ) {

            gi = git;
            gxmetric = ( float ) gpy[gi] * sico.CSZ + sico.BDWEST;
            fprintf( gf_directions, "%.0f\t", gxmetric );
        }
    } if ( gy0 == intv ) gy0 = 0; else gy0 = intv; }
    fprintf( gf_directions, "\n" );

    gi = 0;
    for ( gx=0; gx<sico.M; gx+=intv ) { for ( gy=gy0; gy<sico.N; gy+=(intv2*intv)) { // writing y coordinate to file:

        git = gi;
        while (( gpx[git] != gx || gpy[git] != gy ) && git < sico.IMAX ) git+= 1;
        if ( git < sico.IMAX ) {

            gi = git;
            gymetric = sico.BDNORTH - ( float ) gpx[gi] * sico.CSZ;
            fprintf( gf_directions, "%.0f\t", gymetric );
        }
    } if ( gy0 == intv ) gy0 = 0; else gy0 = intv; }
    fprintf( gf_directions, "\n" );

    return;
}

void foutdir ( FILE *gf_directions, double **gparam, int gl, int gintmode, int *gpx, int *gpy, struct ico sico ) {
    // function for output flow direction file

    int gi, git, gy0, gx, gy, intv, intv2;
    double ghflowi, gparami;

    intv = (int)( sico.N / 75 + 1 ); // pixel interval for writing output

    if ( gintmode == 1 ) { intv2 = 1; gy0 = 0; } // defining shift depending on type of parameter
    else if ( gintmode == 2 ) { intv2 = 2; gy0 = 0; }
    else if ( gintmode == 3 ) { intv2 = 2; gy0 = intv; }

    gi = 0;
    for ( gx=0; gx<sico.M; gx+=intv ) { for ( gy=gy0; gy<sico.N; gy+=(intv2*intv)) { // writing parameter to file:

        git = gi;
        while (( gpx[git] != gx || gpy[git] != gy ) && git < sico.IMAX ) git+= 1;
        if ( git < sico.IMAX ) {

            gi = git;

            if ( gintmode == 1 ) ghflowi = gparam[gi][0];
            else if ( gintmode == 2 ) ghflowi = gparam[gi][0];
            else if ( gintmode == 3 ) ghflowi = gparam[gi][3];

            if ( ghflowi >= sico.IMPTHR[0] ) {

                if ( gintmode == 1 || gintmode == 2 ) gparami = gparam[gi][gl] / gparam[gi][0];
                else if ( gintmode == 3 ) gparami = gparam[gi][gl] / gparam[gi][3];
                fprintf( gf_directions, "%.2f\t", gparami );
            }
            else fprintf( gf_directions, "0\t" );
        }
    } if ( gy0 == intv ) gy0 = 0; else gy0 = intv; }
    fprintf( gf_directions, "\n" );

    return;
}


#ifdef WITHGRASS


    SEGMENT finrasti ( char gm[1000], struct ico sico ) { // input of integer GRASS raster maps

        int gf, gx, gy, gin;
        CELL *gc;
        SEGMENT gseg_in;

        Segment_open ( &gseg_in, G_tempfile(), sico.M, sico.N, sico.NSEGRC, sico.NSEGRC, sizeof(int), sico.NSEGS );

        gc = Rast_allocate_c_buf();
        gf = Rast_open_old ( gm, sico.MAINMAPSET );

        for ( gx = 0; gx < sico.M; gx++ ) {

            Rast_get_c_row ( gf, gc, gx );

            for ( gy = 0; gy < sico.N; gy++ ) {

                if ( !Rast_is_c_null_value ( gc + gy ) ) gin = ( int ) gc[gy];
                else gin = (int)sico.UNDEF;
                Segment_put(&gseg_in, &gin, gx, gy);
	    }
        }
        Segment_flush(&gseg_in);
        G_free ( gc );
        Rast_close ( gf );

        return gseg_in;
    }

    SEGMENT finrastd ( char gm[1000], struct ico sico ) { // input of double GRASS raster maps

        int gf, gx, gy;
        double gin;
        DCELL *gc;
        SEGMENT gseg_in;

        Segment_open ( &gseg_in, G_tempfile(), sico.M, sico.N, sico.NSEGRC, sico.NSEGRC, sizeof(double), sico.NSEGS );

        gc = Rast_allocate_d_buf();
        gf = Rast_open_old ( gm, sico.MAINMAPSET );

        for ( gx = 0; gx < sico.M; gx++ ) {

            Rast_get_d_row ( gf, gc, gx );

            for ( gy = 0; gy < sico.N; gy++ ) {

                if ( !Rast_is_d_null_value ( gc + gy ) ) gin = ( double ) gc[gy];
                else gin = sico.UNDEF;
                Segment_put(&gseg_in, &gin, gx, gy);
	    }
        }

        Segment_flush(&gseg_in);
        G_free ( gc );
        Rast_close ( gf );

        return gseg_in;
    }

    void foutrast ( char gm[1000], SEGMENT gseg_v, struct ico sico ) { // output of GRASS raster maps

        int gf, gx, gy;
        double gv;
        DCELL *gc;

        if ( gm != NULL ) {
            gc = Rast_allocate_d_buf();
	    gf = Rast_open_new( gm, DCELL_TYPE );
        }

        for ( gx = 0; gx < sico.M; gx++ ) {

            if ( gm != NULL ) {
	        for ( gy = 0; gy < sico.N; gy++ ) {
                    Segment_get(&gseg_v, &gv, gx, gy);
                    gc[gy] = (DCELL) gv;
                }
	        Rast_put_d_row( gf, gc );
            }
        }
        Rast_close( gf );
        G_free( gc );

        return;
    }


#else


    double *finaschdr ( char *gname, struct ico sico ) { // function for input of ascii raster map header

        FILE *gfascii;
        char gpath[200], *tok;
        char delim[] = " \t\n";
        int gi, gx, gy;
        double gtest3, *gparam, gtestthrsh;
        char *gtest, *gtest2;
        char **strtodhlp=NULL;

        gparam = (double*) calloc( 7, sizeof(double));

        sprintf(gpath, "%s", gname ); // writing name of ascii file to string
        gfascii=fopen(gpath, "r"); // opening ascii file

        for ( gi=0; gi<6; gi++ ) { // reading header lines and writing to array:
            gtest = fcparam ( gfascii );
            tok=strtok(gtest, delim);
            while ( tok != NULL ) {
                gtest2=tok;
                tok=strtok(NULL, delim);
            }
            gparam[gi] = strtod (gtest2, strtodhlp);
            free(gtest);
        }

        if ( sico.TOPO == 1 ) gtestthrsh = gparam[5]; // threshold for validity, depending on raster used
        else if ( sico.TOPO == 2 ) gtestthrsh = 1;

        gi = 0;
        for ( gx=0; gx<(int)gparam[1]; gx++ ) { // counting number of pixels of reduced coordinate system:

            gtest = flparam ( gfascii );

            tok = strtok(gtest, delim);
            for ( gy=0; gy<(int)gparam[0]; gy++ ) {
                gtest2 = tok;
                tok = strtok(NULL, delim);
                gtest3 = strtod (gtest2, strtodhlp);
                if ( gtest3 >= gtestthrsh ) gi += 1;
            }
            free(gtest);
        }
        gparam[6] = (double)gi;
        fclose( gfascii );
        return gparam;
    }

    int *finascxy ( char *gname, int gcontrol, struct ico sico ) { // function for input of ascii raster x and y coordinates

        FILE *gfascii;
        char *gpath, *tok;
        char delim[] = " \t\n";
        int gi, gx, gy, *gpxy;
        double gtest3, gtestthrsh;
        char **strtodhlp=NULL;

        char *gtest; // allocating memory to arrays
        char *gtest2;

        gpath = (char*) calloc ( 1000, sizeof(char)); // allocating memory to arrays:
        gpxy = (int*) calloc ( sico.IMAX, sizeof(int));

        if ( sico.TOPO == 1 ) gtestthrsh = sico.UNDEF; // threshold for validity, depending on raster used
        else if ( sico.TOPO == 2 ) gtestthrsh = 1;

        sprintf(gpath, "%s", gname ); // writing name of ascii file to string
        gfascii=fopen(gpath, "r"); // opening ascii file
        for ( gi=0; gi<6; gi++ ) {
            gtest = fcparam ( gfascii ); // reading header lines
            free(gtest);
        }

        gi = 0;
        for ( gx=0; gx<sico.M; gx+=1 ) { // writing internal coordinates of all rows and columns to array:

            gtest = flparam ( gfascii );
            tok = strtok(gtest, delim);
            for ( gy=0; gy<sico.N; gy+=1 ) {

                gtest2 = tok;
                tok = strtok(NULL, delim);
                gtest3 = strtod (gtest2, strtodhlp);

                if ( gtest3 >= gtestthrsh ) {

                    if ( gcontrol == 1 ) gpxy[gi] = gx;
                    else gpxy[gi] = gy;

                    gi += 1;
                }
            }
            free(gtest);
        }
        fclose( gfascii );
        free(gpath);
        return gpxy;
    }

    double *finascval ( char *gname, int *gpx, int *gpy, struct ico sico ) { // function for input of ascii raster map values

        FILE *gfascii;
        char *gpath, *tok;
        char delim[] = " \t\n";
        int gi, gx, gy, gtest3;
        double *gparam;
        char **strtodhlp=NULL;

        char *gtest; // allocating memory to arrays
        char *gtest2;

        gpath = (char*) calloc ( 1000, sizeof(char)); // allocating memory to arrays:
        gparam = (double*) calloc( sico.IMAX, sizeof(double));

        sprintf(gpath, "%s", gname ); // writing name of ascii file to string
        gfascii=fopen(gpath, "r"); // opening ascii file
        for ( gi=0; gi<6; gi++ ) {
            gtest = fcparam ( gfascii ); // reading header lines
            free(gtest);
        }

        gi = 0;
        for ( gx=0; gx<sico.M; gx+=1 ) { // writing raster values of all rows and columns to array:

            gtest = flparam ( gfascii );
            tok = strtok(gtest, delim);
            for ( gy=0; gy<sico.N; gy+=1 ) {

                gtest2 = tok;
                tok = strtok(NULL, delim);
                gtest3 = strtod (gtest2, strtodhlp);

                if ( gx == gpx[gi] && gy == gpy[gi] ) {
                    gparam[gi] = gtest3;
                    gi += 1;
                }
            }
            free(gtest);
        }
        fclose( gfascii );
        free(gpath);
        return gparam;
    }


#endif


// Calling main function

int main ( int argc, char *argv[] ) {


// Variables

    FILE *fparam;

    char path[1000], wkdir[1000], *prefix, *outmaps, *outfiles, *elevname, *aoiname,
        *hreleasename, *hreleasename2, *hentrmaxname, *hentrmaxname2, *centrname, *deltaname, *hydreleasename0, mv[1000], **mv0;
    char *madd = (char*) calloc(1000, sizeof(char));

    int *px, *py, *in, *inn, *innn, in6, hctrl, hydrograph, hydnin, hydnout, hydfx, hydfy, *hydi, hydj, hydk, *hydx, *hydy, hydt, *hydtmax,
        hydtmaxx, *hydp0, **hydp, nvect_all, nvect_red, nvect_min, lmax, xint, coverflow, ccontinue, csuccess, nsum, nout, time_start,
        time_stop, ctrlr, ctrlv, ctrlvv, ctrlvvv, ccfl, fj[4][2], i, j, jj, jjj, k, l, p, prec_hflow, prec_vol, prec_ekin, imax, nalpha,
        csolid, cfluid, vol_npixels, vol_npixels2, ctrl_hydout;

    double *pelev, *pelev0, *gk, *gf, *gg, *gs, *flowpar, *basepar, tout, tmax, vflowx, vflowy, vflow, rflow, dvflowx, dvflowy, dvflow,
        vflowx2, vflowy2, time_elapsed, tlength, tint, tsum, cfl, cflmax, gx, gy, gz, dw_dt, hflow_maxmax,
        hflow_max0, hflow_max, hflow_max02, hflow_max2, vflow_max, vflow_max2, **hydhyd0, ***hydhyd, rflow_max, vol_flow0, vol_flow,
        vol_flow02, vol_flow2, vol_entr, vol_entr2, vol_flow_bef, vol_flow2_bef, vol_entr_bef, vol_entr2_bef, vol_edge, vol_edge2,
        whflowx, whflowy, vcelr0, vcelr, *hydelev, *hydalpha, *hydx_metric, *hydy_metric, *hydl, hhyd0, hhyd, hyde, hyde2, hydr,
        hydfalpha, hydout_xmin_metric, hydout_xmax_metric, hydout_ymin_metric, hydout_ymax_metric, hydh, hydh2, hydv, hydv2, hydq, hydq2,
        hydlmax, ekin_flow, fflux, ffluxn, vol_noflux, vol_noflux2, vol_noflux_bef, vol_noflux2_bef, carea, vflowxp,
        vflowxm, vflowyp, vflowym, hydmalpha, hydnalpha, hydbeta, hydm0, hydmx, hydmy, hydm, hydfcorr, wvflowx[9], wvflowy[9],
        wvflowx2[9], wvflowy2[9], walpha[9], welev[9], potentr, potentr2, corrfact, qentr, qentr2, alpha, alphav, alphavg, betavg,
        momx, momy, mom, mass, vbulkx, vbulky, vol_diff, vol_diff2, vol_flowold, vol_flow2old, rpotentr;

    struct ico sico;
    struct flow sflow;
    struct base sbase;


    #ifdef WITHGRASS


        SEGMENT seg_aoi, seg_elev, seg_hrelease, seg_hrelease2, seg_hentrmax, seg_hentrmax2, seg_centr, seg_delta;
        char *yint = (char*) calloc(1000, sizeof(char));
        int x, y, aoi;
        double hflowi, hflowi2, hentri, hentri2, pout, elev, hrelease, hrelease2, hentrmax, hentrmax2, centr, delta;

        struct GModule *module;
        struct Cell_head cellhd;


    #else


        double *aschdr;


    #endif


// Timer

    time_t timer;
    time_start = clock(); // start time
    printf("Starting model execution.\n\n");
    fflush(stdout); // forcing immediate display


// Preparing environment


    #ifdef WITHGRASS


        sprintf(wkdir, "%s/%s/.tmp/rtemp", G_location_path(),G_mapset()); // defining working directory
        G_init_tempfile(); // initializing generation of temporary files
        sico.NSEGRC = 1; // defining segment parameters
        sico.NSEGS = 1;
        xint=10;
        yint=getenv("XINT");
        xint=atoi(yint);


    #else


        if ( argc > 1 ) sprintf( wkdir, "%s", argv[1] ); // defining working directory
        else {
            printf( "ERROR: Please provide a parameter file.\n" );
            fflush(stdout); // forcing immediate display
            exit( EXIT_SUCCESS );
        }
        xint=1;


    #endif


// Defining constants

    sico.PI = 3.1415926536;
    sico.GRAVITY = 9.81;


// Reading parameters from file and deriving further parameters

    sprintf( path, "%s/param%d.txt", wkdir, xint ); // writing name of parameter file to string
    fparam=fopen( path, "r" ); // opening parameter file

    if( !fparam ) { // testing whether parameter file exists:
        printf( "ERROR: Unable to open parameter file: '%s'\n", path );
        fflush(stdout); // forcing immediate display
        exit( EXIT_SUCCESS );
    }

    sico.MAINMAPSET = fcparam ( fparam ); // name of main mapset
    sico.MULT = fiparam ( fparam ); // identifier for single model run (0) or multiple model runs (1)
    sico.MODEL = fiparam ( fparam ); // model (1 = Voellmy-Salm one-phase model, 2 = Pudasaini two-phase model)
    sico.TOPO = fiparam ( fparam ); // topography (1 = slope, 2 = channel)

    if ( sico.MODEL == 1 ) { // size of arrays

        nvect_all = 13;
        nvect_red = 8;
        nvect_min = 4;
    }
    else if ( sico.MODEL == 2 ) {

        nvect_all = 29;
        nvect_red = 18;
        nvect_min = 6;
    }

    sico.CORRHEIGHT = fdparam ( fparam ); // correction of flow height for slope (0=none, 1=basic, 2=advanced)
    sico.DIFFCTRL = fdparam ( fparam ); // diffusion control (0=no, 1=yes)
    sico.CONSVOL = fdparam ( fparam ); // conservation of volume (0=no, 1=yes)
    sico.NOOSC = fdparam ( fparam ); // avoidance of oscillations (0=no, 1=yes)
    sico.ENTRAINMENT = fdparam ( fparam ); // entrainment and deposition (0=no, 1-3=type of approach)
    sico.STOPPING = fdparam ( fparam ); // stopping of flow (0=no, 1=yes)

    sico.RELM = fiparam ( fparam ); // identifier for use of (solid) release height map as input (1) or not (0)
    if ( sico.MODEL == 2 ) sico.RELM2 = fiparam ( fparam ); // identifier for use of fluid release height map as input (1) or not (0)
    sico.ENTR = fiparam ( fparam ); // identifier for use of maximum height of (solid) entrainment map as input (1) or not (0)
    if ( sico.MODEL == 2 ) sico.ENTR2 = fiparam ( fparam );
        // identifier for use of max. height of fluid entrainment map as input (1) or not (0)
    sico.CENTR = fiparam ( fparam ); // identifier for use of entrainment coefficient map as input (1) or not (0)
    sico.DELTA = fiparam ( fparam ); // identifier for use of basal friction angle map as input (1) or not (0)
    hydrograph = fiparam ( fparam ); // identifier for use of hydrograph(s) for input and/or output (1) or not (0)

    if ( sico.TOPO == 2 ) aoiname = fcparam ( fparam ); // name of area of interest map
    elevname = fcparam ( fparam ); // name of elevation map


// Setting spatial extent and pixel size


    #ifdef WITHGRASS


    // If GRASS rasters are used:

        G_gisinit( argv[0] );
        module = G_define_module();
        G_get_set_window( &cellhd );

        sico.CSZ=cellhd.ew_res; // pixel size
        sico.BDWEST=cellhd.west; // western boundary
        sico.BDNORTH=cellhd.north; // northern boundary
        sico.BDSOUTH=cellhd.south; // southern boundary
        sico.N=cellhd.cols; // number of pixels in y direction (W-E)
        sico.M=cellhd.rows; // number of pixels in x direction (S-N)
        sico.UNDEF = -9999; // no data value


    #else


    // If Ascii rasters are used:

        if ( sico.TOPO == 1 ) aschdr = finaschdr( elevname, sico ); // reading header of Ascii raster
        else if ( sico.TOPO == 2 ) aschdr = finaschdr( aoiname, sico );

        sico.N = (int)aschdr[0]; // number of pixels in y direction (W-E)
        sico.M = (int)aschdr[1]; // number of pixels in x direction (S-N)
        sico.CSZ = aschdr[4]; // pixel size
        sico.BDWEST = aschdr[2]; // western boundary
        sico.BDSOUTH = aschdr[3]; // southern boundary
        sico.BDNORTH = aschdr[3] + sico.M * sico.CSZ; // northern boundary
        sico.UNDEF = aschdr[5]; // no data value
        sico.IMAX = (int)aschdr[6]; // number of pixels in reduced coordinate system

        free(aschdr);


    #endif


// Continuing reading parameters from file and deriving further parameters

    if ( sico.RELM == 1 ) hreleasename = fcparam ( fparam ); // name of (solid) release height map
    if ( sico.MODEL == 2 && sico.RELM2 == 1 ) hreleasename2 = fcparam ( fparam ); // name of fluid release height map
    if ( sico.ENTR == 1 ) hentrmaxname = fcparam ( fparam ); // name of maximum height of (solid ) entrainment map
    if ( sico.MODEL == 2  && sico.ENTR2 == 1 ) hentrmaxname2 = fcparam ( fparam ); // name of maximum height of fluid entrainment map
    if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) centrname = fcparam ( fparam ); // name of entrainment coefficient map
    if ( sico.DELTA == 1 ) deltaname = fcparam ( fparam ); // name of basal friction angle map

    if ( hydrograph == 1 ) {

        hydnin = fiparam ( fparam ); // number of input hydrographs
        hydnout = fiparam ( fparam ); // number of output hydrographs

        hydtmax = (int*) calloc( hydnin, sizeof(int)); // allocating memory to variable for number(s) of hydrograph time steps
        hydi = (int*) calloc(( hydnin + hydnout ), sizeof(int));
            // allocating memory to variable for internal coordinate of hydrograph point
        hydelev = (double*) calloc(( hydnin + hydnout ), sizeof(double));
            // allocating memory to variable for elevation of hydrograph point
        hydalpha = (double*) calloc(( hydnin + hydnout ), sizeof(double));
            // allocating memory to variable for aspect of hydrograph point
        hydx_metric = (double*) calloc(( hydnin + hydnout ), sizeof(double));
            // allocating memory to variable for x coordinate of hydrograph
        hydy_metric = (double*) calloc(( hydnin + hydnout ), sizeof(double));
            // allocating memory to variable for y coordinate of hydrograph
        hydl = (double*) calloc(( hydnin + hydnout ), sizeof(double)); // allocating memory to variable for length of hydrograph profile
        hydx = (int*) calloc(( hydnin + hydnout ), sizeof(int)); // allocating memory to variable for internal x coordinate of hydrograph
        hydy = (int*) calloc(( hydnin + hydnout ), sizeof(int)); // allocating memory to variable for internal y coordinate of hydrograph

        hydtmaxx = 0; // initializing maximum number of hydrograph time steps
    }
    else hydnin = 1;

    char *hydreleasename[hydnin]; // declaring pointers to input hydrograph files

    if ( hydrograph == 1 ) {

        for ( hydj = 0; hydj < hydnin; hydj++ ) { // loop over all input hydrographs:

            hydreleasename[hydj] = (char*) calloc( 1000 * hydnin, sizeof(char)); // allocating memory to pointers to input hydrograph files

            hydreleasename0 = fcparam ( fparam ); // name of hydrograph file
            strcpy( hydreleasename[hydj], hydreleasename0 );
            free( hydreleasename0 );
            hydtmax[hydj] = fiparam ( fparam ); // number of hydrograph time steps
            if ( hydtmax[hydj] > hydtmaxx ) hydtmaxx = hydtmax[hydj]; // updating maximum number of hydrograph time steps
        }

        for ( hydj = 0; hydj < hydnin + hydnout; hydj++ ) { // loop over all hydrographs:

            hydx_metric[hydj] = fdparam ( fparam ); // x coordinate of hydrograph
            hydy_metric[hydj] = fdparam ( fparam ); // y coordinate of hydrograph
            hydl[hydj] = fdparam ( fparam ); // length of hydrograph profile
            hydalpha[hydj] = fdparam ( fparam ); // aspect of hydrograph profile
        }

        hydhyd = alloc_dmatrix3( hydtmaxx+1, 5, hydnin ); // allocating memory to array of input hydrograph data
    }
    else { hydnin = 1; hydnout = 1; }

    lmax = fiparam ( fparam ); // number of flow parameters (depending on model)
    flowpar = (double*) calloc( lmax, sizeof(double)); // allocating memory for storage of flow parameter values
    for ( l=0; l<lmax; l++ ) flowpar[l] = fdparam ( fparam ); // flow parameters

    if ( sico.MODEL == 1 ) { // passing flow parameters to variables, one-phase model:

        sflow.RHO = flowpar[0]; // material density
        sflow.PHI = flowpar[1] * sico.PI / 180; // internal friction angle
        sflow.DELTA0 = flowpar[2] * sico.PI / 180; // basal friction angle
        sflow.EPS = flowpar[3]; // turbulent friction
        sflow.RPROD = flowpar[4]; // production of random kinetic energy
        sflow.RDEC = flowpar[5]; // decay of random kinetic energy
        sflow.R0 = flowpar[6]; // value of random kinetic energy for adaptation of friction coefficients
    }
    else if ( sico.MODEL == 2 ) { // passing flow parameters to variables, two-phase model:

        sflow.RHOS = flowpar[0]; // solid material density
        sflow.RHOF = flowpar[1]; // fluid material density
        sflow.VM = flowpar[2]; // virtual mass
        sflow.PHI = flowpar[3] * sico.PI / 180; // internal friction angle
        sflow.DELTA0 = flowpar[4] * sico.PI / 180; // basal friction angle
        sflow.CHI = flowpar[5]; // viscous shearing coefficient for fluid
        sflow.XI = flowpar[6]; // solid concentration distribution with depth
        sflow.VTERM = flowpar[7]; // terminal velocity
        sflow.P = flowpar[8]; // parameter for combination of solid- and fluid-like contributions to drag resistance
        sflow.REP = pow(10, flowpar[9]); // particle Reynolds number
        sflow.JE = flowpar[10]; // exponent for drag (1=linear, 2=quadratic)
        sflow.NR  = pow(10, flowpar[11]); // quasi Reynolds number
        sflow.NRA = pow(10, flowpar[12]); // mobility number
        sflow.AMBDRAG = flowpar[13]; // coefficient for ambient drag

        if ( sflow.RHOF == sico.UNDEF || sflow.RHOS == sico.UNDEF ) sflow.GAMMA = 0;
        else sflow.GAMMA = sflow.RHOF / sflow.RHOS; // ratio between fluid and solid densities
    }
    sflow.MY = tan( sflow.DELTA0 ); // basal friction coefficient

    if ( sico.ENTRAINMENT > 0 ) {
        // entrainment/deposition, passing entrainment coefficient to variables:

        sbase.CENTR0 = fdparam ( fparam ); // entrainment/deposition coefficient
        if ( sbase.CENTR0 == sico.UNDEF ) sbase.CENTR0 = 0.0; // default values (no entrainment or deposition)
        else sbase.CENTR0 = pow(10, sbase.CENTR0 ); // entrainment/deposition coefficient
    }

    /*else if ( sico.MODEL == 2 && sico.ENTRAINMENT == 3 ) {
        // two-phase model, entrainment/deposition approach #3, passing basal surface parameters to variables:

        basepar = (double*) calloc( 4, sizeof(double)); // allocating memory for storage of basal surface parameter values
        for ( l=0; l<4; l++ ) basepar[l] = fdparam ( fparam ); // basal surface parameters

        sbase.CHEZY = basepar[0]; // Chezy coefficient
        sbase.RHOS = basepar[1]; // solid density
        sbase.RHOF = basepar[2]; // fluid density
        sbase.MY = basepar[3]; // friction coefficient

        if ( sbase.CHEZY == sico.UNDEF ) sbase.CHEZY = 40; // default values (no entrainment or deposition)
        if ( sbase.RHOS == sico.UNDEF ) sbase.RHOS = sflow.RHOS;
        if ( sbase.RHOF == sico.UNDEF ) sbase.RHOS = sflow.RHOF;
        if ( sbase.MY == sico.UNDEF ) sbase.MY = sflow.MY;
    }*/

    sico.HFLOWMIN = fdparam ( fparam ); // minimum flow depth to be considered
    sico.IMPTHR[0] = fdparam ( fparam ); // threshold for display of flow height
    sico.IMPTHR[1] = fdparam ( fparam ); // threshold for display of flow kinetic energy
    sico.IMPTHR[2] = fdparam ( fparam ); // threshold for display of flow pressure

    tout = fdparam ( fparam ); // time interval for writing output
    tmax = fdparam ( fparam ); // time at which to stop simulation
    sico.CFL[0] = fdparam ( fparam ); // CFL criterion
    sico.CFL[1] = fdparam ( fparam ); // maximum time step length

    prefix = fcparam ( fparam ); // prefix for output
    outmaps = fcparam ( fparam ); // path and prefix for storing output maps
    outfiles = fcparam ( fparam ); // path and prefix for storing output files

    fclose(fparam); // closing parameter file


// Input hydrograph data and internal coordinates of hydrographs

    if ( hydrograph == 1 ) { // if at least one input hydrograph is provided:

        hydlmax = 0;
        for ( hydj = 0; hydj < hydnin + hydnout; hydj++ ) if ( hydl[hydj] > hydlmax ) hydlmax = hydl[hydj];

        for ( hydj = 0; hydj < hydnin; hydj++ ) {
            hydhyd0 = finhyd ( hydreleasename[hydj], hydtmax[hydj], hydtmaxx ); // reading data from all input hydrographs
            for ( i=0; i<hydtmaxx+1; i++ ) { for ( j=0; j<5; j++ ) hydhyd[i][j][hydj] = hydhyd0[i][j]; }
            free( hydhyd0[0] ); free( hydhyd0 );
        }

        for ( hydj = 0; hydj < hydnin + hydnout; hydj++ ) { // internal x and y coordinates of all hydrographs:

            hydy[hydj] = (int)(( hydx_metric[hydj] - sico.BDWEST ) / sico.CSZ + 0.5 );
            hydx[hydj] = (int)(( sico.BDNORTH - hydy_metric[hydj] ) / sico.CSZ + 0.5 );
        }
    }


// Initializing release heights and associated volumes

    hflow_max0 = 0; // initializing maximum (solid) release height
    if ( sico.MODEL == 2 ) hflow_max02 = 0; // initializing maximum fluid release height
    vol_flow0 = 0; // initializing (solid) release volume
    if ( sico.MODEL == 2 ) vol_flow02 = 0; // initializing fluid release volume


    #ifdef WITHGRASS


    // Reading input raster maps

        if ( sico.TOPO == 2 ) seg_aoi = finrasti( aoiname, sico ); // area of interest
        seg_elev = finrastd( elevname, sico ); // elevation
        if ( sico.RELM == 1 ) seg_hrelease = finrastd( hreleasename, sico ); // (solid) release height
        if ( sico.MODEL == 2 && sico.RELM2 == 1 ) seg_hrelease2 = finrastd( hreleasename2, sico ); // fluid release height
        if ( sico.ENTR == 1 ) seg_hentrmax = finrastd( hentrmaxname, sico ); // maximum height of (solid) entrainment
        if ( sico.MODEL == 2 && sico.ENTR2 == 1 ) seg_hentrmax2 = finrastd( hentrmaxname2, sico ); // maximum height of fluid entrainment
        if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) seg_centr = finrastd( centrname, sico );
            // entrainment coefficient
        if ( sico.DELTA == 1 ) seg_delta = finrastd( deltaname, sico ); // basal friction angle


    // Computing number of pixels in aoi

        sico.IMAX = 0;
        for ( x = 0; x < sico.M; x++ ) {
            for ( y = 0; y < sico.N; y++ ) {

                if ( sico.TOPO == 1 ) sico.IMAX += 1; // increasing number of pixels in area of interest by 1 for each pixel for topo == 1
                else if ( sico.TOPO == 2 ) {

                    Segment_get(&seg_aoi, &aoi, x, y);
                    if ( aoi < 1 ) {

                        aoi = sico.UNDEF; // excluding all pixels with aoi < 1
                        Segment_put(&seg_aoi, &aoi, x, y);
                        Segment_flush(&seg_aoi);
                    }
                    else sico.IMAX += 1; // increasing number of pixels in area of interest by 1 for each pixel in aoi for topo == 2
                }
	    }
        }


    #endif


// Vectors

    int **cedge, **cedge2, *cedge0, *cedge02, *cneighbours, *cneighbours2;
    double *v, *phrelease, *phrelease2, *phentrmax, *phentrmax2, *pcentr, *pdelta, *dentr, *dentr2, **cready, **cready2, *fom;

    int *cdomain = (int*) calloc( sico.IMAX, sizeof(int));
    int *cdomain2 = (int*) calloc( sico.IMAX, sizeof(int));
    int *cflux = (int*) calloc( sico.IMAX, sizeof(int));
    int *cstopped = (int*) calloc( sico.IMAX, sizeof(int));

    double *betax = (double*) calloc( sico.IMAX, sizeof(double));
    double *betay = (double*) calloc( sico.IMAX, sizeof(double));
    double *betaxy = (double*) calloc( sico.IMAX, sizeof(double));

    pelev0 = (double*) calloc( sico.IMAX, sizeof(double));


    #ifdef WITHGRASS


        v = (double*) calloc( nvect_all, sizeof(double));
        px = (int*) calloc( sico.IMAX, sizeof(int));
        py = (int*) calloc( sico.IMAX, sizeof(int));
        pelev = (double*) calloc( sico.IMAX, sizeof(double));
        phrelease = (double*) calloc( sico.IMAX, sizeof(double));
        phentrmax = (double*) calloc( sico.IMAX, sizeof(double));

        if ( sico.MODEL == 2 ) {

            phrelease2 = (double*) calloc( sico.IMAX, sizeof(double));
            phentrmax2 = (double*) calloc( sico.IMAX, sizeof(double));
        }
        if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) pcentr = (double*) calloc( sico.IMAX, sizeof(double));
        if ( sico.DELTA == 1 ) pdelta = (double*) calloc( sico.IMAX, sizeof(double));


    #endif


    dentr = (double*) calloc( sico.IMAX, sizeof(double));
    if ( sico.MODEL == 2 ) dentr2 = (double*) calloc( sico.IMAX, sizeof(double));

    if ( sico.STOPPING != 0 ) fom = (double*) calloc( sico.IMAX, sizeof(double));

    double **aw = alloc_dmatrix( sico.IMAX, nvect_all );
    double **awt = alloc_dmatrix( sico.IMAX, 8 );
    double **af = alloc_dmatrix( sico.IMAX, nvect_min );
    double **ag = alloc_dmatrix( sico.IMAX, nvect_min );
    double **as = alloc_dmatrix( sico.IMAX, nvect_min );

    double **asigma_x = alloc_dmatrix( sico.IMAX, nvect_min );
    double **asigma_y = alloc_dmatrix( sico.IMAX, nvect_min );
    double **asigma_f = alloc_dmatrix( sico.IMAX, nvect_min );
    double **asigma_g = alloc_dmatrix( sico.IMAX, nvect_min );

    double **wintd = alloc_dmatrix( sico.IMAX, nvect_min );
    double **winta = alloc_dmatrix( 4, nvect_min );
    double **wintb = alloc_dmatrix( 6, nvect_min );
    double ***wintc = alloc_dmatrix3( sico.IMAX, 4, nvect_min );

    double **f = alloc_dmatrix( 4, nvect_min );
    double **g = alloc_dmatrix( 4, nvect_min );
    double **s = alloc_dmatrix( 4, nvect_min );

    double *dx = (double*) calloc( sico.IMAX, sizeof(double));
    double *dy = (double*) calloc( sico.IMAX, sizeof(double));

    if ( sico.DIFFCTRL == 1 ) { // if diffusion control is activated:

        cedge = alloc_imatrix( sico.IMAX, 9 );
        cready = alloc_dmatrix( sico.IMAX, 9 );
        cedge2 = alloc_imatrix( sico.IMAX, 9 );
        cready2 = alloc_dmatrix( sico.IMAX, 9 );
        cedge0 = (int*) calloc( sico.IMAX, sizeof(int));
        cedge02 = (int*) calloc( sico.IMAX, sizeof(int));
        cneighbours = (int*) calloc( sico.IMAX, sizeof(int));
        cneighbours2 = (int*) calloc( sico.IMAX, sizeof(int));
    }


// Variables for names of output raster maps

    mv0 = (char**) calloc( 1000, sizeof(char*));

    if ( sico.MODEL == 1 ) {

        mv0[0] = "hflow"; mv0[1] = "vflowx"; mv0[2] = "vflowy"; mv0[3] = "rflow"; mv0[4] = "basechange"; mv0[5] = "vflow";
        mv0[6] = "tflow"; mv0[7] = "pflow"; mv0[8] = "hflow_max"; mv0[9] = "vflow_max"; mv0[10] = "rflow_max";
        mv0[11] = "tflow_max"; mv0[12] = "pflow_max";
    }
    else if ( sico.MODEL == 2 ) {

        mv0[0] = "hflows"; mv0[1] = "vflowxs"; mv0[2] = "vflowys"; mv0[3] = "hflowf"; mv0[4] = "vflowxf"; mv0[5] = "vflowyf";
        mv0[6] = "basechanges"; mv0[7] = "basechangef"; mv0[8] = "vflows"; mv0[9] = "vflowf"; mv0[10] = "hflow"; mv0[11] = "tflows";
        mv0[12] = "tflowf"; mv0[13] = "tflow"; mv0[14] = "pflows"; mv0[15] = "pflowf"; mv0[16] = "pflow"; mv0[17] = "basechange";
        mv0[18] = "hflows_max"; mv0[19] = "vflows_max"; mv0[20] = "hflowf_max"; mv0[21] = "vflowf_max"; mv0[22] = "hflow_max";
        mv0[23] = "tflows_max"; mv0[24] = "pflows_max"; mv0[25] = "tflowf_max"; mv0[26] = "pflowf_max"; mv0[27] = "tflow_max";
        mv0[28] = "pflow_max";
    }


// Defining, opening and preparing output files

    FILE *f_summary, *f_directions, *f_directions2, *f_nout, *f_hydout, *f_hydinfo[hydnin+hydnout];

    if ( sico.MULT == 0 ) sprintf(path, "%s%ssummary.txt", outfiles, prefix); // summary file
    else sprintf(path, "%s%ssummary%d.txt", outfiles, prefix, xint);
    f_summary=fopen(path, "w");

    if ( hydrograph == 1 ) {

        sprintf(path, "%s%shydprofiles.txt", outfiles, prefix); // hydrograph profiles file
        f_hydout=fopen(path, "w");

        fprintf(f_hydout, "ID\tx1\txC\tx2\ty1\tyC\ty2\n"); // printing header of hydrograph profiles file

        if ( sico.MULT == 0 ) {

            for ( i = 0; i < hydnin + hydnout; i++ ) { // loop over all hydrographs:

                sprintf(path, "%s%shydinfo%i.txt", outfiles, prefix, i+1); // hydrograph info file
                f_hydinfo[i]=fopen(path, "w");

                if ( i >= hydnin ) fprintf(f_hydinfo[i],
                    "T\tHs\tVs\tR\tEs\tQs\tHf\tVf\tEf\tQf\n0.0\t0.00\t0.00\t0.00\t0.00\t0\t0.00\t0.00\t0.00\t0\n");
                else fprintf(f_hydinfo[i], "T\tHs\tVs\tR\tEs\tQs\tHf\tVf\tEf\tQf\n"); // printing header of hydrograph info file
            }

        }
    }

    if ( sico.MULT == 0 ) {
        sprintf(path, "%s%sdirections.txt", outfiles, prefix); // (solid) flow direction file (for display as arrows)
        f_directions=fopen(path, "w");

        if ( sico.MODEL == 2 ) {
            sprintf(path, "%s%sdirectionsf.txt", outfiles, prefix); // fluid flow direction file (for display as arrows)
            f_directions2=fopen(path, "w");
        }
    }


    #ifdef WITHGRASS


    // Preparing segment files for processing and output (if GRASS is used)

        SEGMENT *seg_v = (SEGMENT*) calloc( nvect_all, sizeof(SEGMENT));

        for ( k=0; k<nvect_all; k++ ) // segment files for all vector components
            Segment_open (&seg_v[k], G_tempfile(), sico.M, sico.N, sico.NSEGRC, sico.NSEGRC, sizeof(double), sico.NSEGS);


    // Initializing and preparing working and output variables (if GRASS is used)

        i = 0;

        for ( x = 0; x < sico.M; x++ ) { for ( y = 0; y < sico.N; y++ ) { // loop over all pixels:

            if ( sico.TOPO == 2 ) Segment_get(&seg_aoi, &aoi, x, y); // area of interest
            else aoi = 1;
            Segment_get(&seg_elev, &elev, x, y); // elevation
            if ( sico.RELM == 1 ) Segment_get(&seg_hrelease, &hrelease, x, y); // (solid) release height
            if ( sico.MODEL == 2 && sico.RELM2 == 1 ) Segment_get(&seg_hrelease2, &hrelease2, x, y); //fluid release height
            if ( sico.ENTR == 1 ) Segment_get(&seg_hentrmax, &hentrmax, x, y); // maximum height of (solid) entrainment
            if ( sico.MODEL == 2 && sico.ENTR2 == 1 ) Segment_get(&seg_hentrmax2, &hentrmax2, x, y); // maximum height of fluid entrainment
            if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) Segment_get(&seg_centr, &centr, x, y); // entrainment coefficient
            if ( sico.DELTA == 1 ) Segment_get(&seg_delta, &delta, x, y); // basal friction angle

            if ( aoi >= 1 ) { // if pixel is in area of interest:

                px[i] = x; // internal x coordinate of pixel
                py[i] = y; // internal y coordinate of pixel
                pelev[i] = elev; pelev0[i] = elev;

                if ( sico.RELM == 1 && hrelease != sico.UNDEF ) phrelease[i] = hrelease; // writing input to arrays
                else phrelease[i] = 0;
                if ( sico.MODEL == 2 && sico.RELM2 == 1  && hrelease2 != sico.UNDEF ) phrelease2[i] = hrelease2;
                else if ( sico.MODEL == 2 ) phrelease2[i] = 0;
                if ( sico.ENTR == 1 ) phentrmax[i] = hentrmax;
                else phentrmax[i] = 9999;
                if ( sico.MODEL == 2 && sico.ENTR2 == 1 && hentrmax2 != sico.UNDEF ) phentrmax2[i] = hentrmax2;
                else if ( sico.MODEL == 2 ) phentrmax2[i] = 9999;
                if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 && centr != sico.UNDEF ) pcentr[i] = pow( 10, centr );
                else if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 )  pcentr[i] = sico.UNDEF;
                if ( sico.DELTA == 1 && delta != sico.UNDEF ) pdelta[i] = delta * sico.PI / 180;
                else if ( sico.DELTA == 1 ) pdelta[i] = sico.UNDEF;

                i += 1;
            }

            for ( k=0; k<nvect_all; k++ ) { // preparing segment files for output rasters:

                v[k] = 0;
                Segment_put(&seg_v[k], &v[k], x, y);
            }
        }}
        for ( k=0; k<nvect_all; k++ ) Segment_flush(&seg_v[k]);


    #else


// Initializing and preparing working and output variables (if GRASS is not used)

        if ( sico.TOPO == 1 ) {

            px = finascxy( elevname, 1, sico ); // internal x coordinate of pixel
            py = finascxy( elevname, 2, sico ); // internal y coordinate of pixel
        }
        else if ( sico.TOPO == 2 ) {
            px = finascxy( aoiname, 1, sico );
            py = finascxy( aoiname, 2, sico );
        }

        pelev = finascval( elevname, px, py, sico ); // elevation
        if ( sico.RELM == 1 ) phrelease = finascval( hreleasename, px, py, sico ); // (solid) release height
        else phrelease = (double*) calloc( sico.IMAX, sizeof(double));
        if ( sico.MODEL == 2 && sico.RELM2 == 1 ) phrelease2 = finascval( hreleasename2, px, py, sico ); // fluid release height
        else if ( sico.MODEL == 2 ) phrelease2 = (double*) calloc( sico.IMAX, sizeof(double));
        if ( sico.ENTR == 1 ) phentrmax = finascval( hentrmaxname, px, py, sico ); // maximum height of (solid) entrainment
        else phentrmax = (double*) calloc( sico.IMAX, sizeof(double));
        if ( sico.MODEL == 2 && sico.ENTR2 == 1 ) phentrmax2 = finascval( hentrmaxname2, px, py, sico ); // max. height of fluid entrainment
        else if ( sico.MODEL == 2 ) phentrmax2 = (double*) calloc( sico.IMAX, sizeof(double));
        if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) pcentr = finascval( centrname, px, py, sico ); // entrainment coefficient
        if ( sico.DELTA == 1 ) pdelta = finascval( deltaname, px, py, sico ); // basal friction angle

        for ( i=0; i<sico.IMAX; i++ ) { // finalizing input arrays:

            pelev0[i] = pelev[i];
            if ( sico.RELM != 1 || phrelease[i] == sico.UNDEF ) phrelease[i] = 0;
            if ( sico.MODEL == 2 && ( sico.RELM2 != 1 || phrelease2[i] == sico.UNDEF )) phrelease2[i] = 0;
            if ( sico.ENTR != 1 || phentrmax[i] == sico.UNDEF ) phentrmax[i] = 9999;
            if ( sico.MODEL == 2 && ( sico.ENTR2 != 1 || phentrmax2[i] == sico.UNDEF )) phentrmax2[i] = 9999;
            if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 && pcentr[i] != sico.UNDEF ) pcentr[i] = pow( 10, pcentr[i] );
            else if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) pcentr[i] = sico.UNDEF;
            if ( sico.DELTA == 1 && pdelta[i] != sico.UNDEF ) pdelta[i] = pdelta[i] * sico.PI / 180;
            else if ( sico.DELTA == 1 ) pdelta[i] = sico.UNDEF;
        }


    #endif


    for ( i=0; i<sico.IMAX; i++ ) { // updating flow height maxima and volumes:

        if ( sico.RELM == 1 ) {
            if ( phrelease[i] > hflow_max0 ) hflow_max0 = phrelease[i]; // updating maximum (solid) release height
            vol_flow0 += phrelease[i] * pow ( sico.CSZ, 2 ); // updating (solid) release volume
        }
        if ( sico.MODEL == 2 && sico.RELM2 == 1 ) {
            if ( phrelease2[i] > hflow_max02 ) hflow_max02 = phrelease2[i]; // updating maximum fluid release height
            vol_flow02 += phrelease2[i] * pow ( sico.CSZ, 2 ); // updating fluid release volume
        }
    }

    if ( vol_flow0 == 0 ) csolid = 0; else csolid = 1; // control for presence of solid material
    if ( vol_flow02 == 0 ) cfluid = 0; else cfluid = 1; // control for presence of fluid material


// Defining computational domains

    // *** cdomain[i] = 0: 1st row edge pixels
    // *** cdomain[i] = 1: all other pixels

    // *** cdomain2[i] = 1: 1st, 2nd or 3rd row edge pixels
    // *** cdomain2[i] = 0: all other pixels

    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood
        ctrlv = 1; ctrlvv = 1; ctrlvvv = 1; // resetting controls
        for ( j=1; j<9; j++ ) { // loop over all neighbour pixels:

            if ( in[j] < 0 || in[j] >= sico.IMAX ) ctrlv = 0; // 1st row edge pixels

            inn = fin ( in[j], px, py, pelev, sico ); // neighbourhood of neighbour pixel
            for ( jj=1; jj<9; jj++ ) { // loop over neighbourhood
                if ( inn[jj] < 0 || inn[jj] >= sico.IMAX ) ctrlvv = 0; // 2nd row edge pixels

                innn = fin ( inn[jj], px, py, pelev, sico ); // neighbourhood of neighbour pixel
                for ( jjj=1; jjj<9; jjj++ ) // loop over neighbourhood
                    if ( innn[jjj] < 0 || innn[jjj] >= sico.IMAX ) ctrlvvv = 0; // 3rd row edge pixels

                free( innn ); // releasing memory
            }
            free( inn ); // releasing memory
        }

        if ( ctrlv == 0 ) { cdomain[i] = 0; cdomain2[i] = 1; }
        else if ( ctrlvv == 0 || ctrlvvv == 0 ) { cdomain[i] = 1; cdomain2[i] = 1; }
        else { cdomain[i] = 1; cdomain2[i] = 0; }

        free( in ); // releasing memory
    }


// Computing slopes and topography-following pixel sizes

    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

        if ( cdomain[i] != 0 ) { // if pixel is no 1st row edge pixel:

            in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

            betax[i] = fbeta( pelev[in[5]], pelev[in[2]], sico ); // slopes
            betay[i] = fbeta( pelev[in[4]], pelev[in[1]], sico );
            betaxy[i] = fbetaxy( betax[i], betay[i] );

            free( in ); // releasing memory
        }
        else {
            betax[i] = 0;
            betay[i] = 0;
            betaxy[i] = 0;
        }

        dx[i] = sico.CSZ / cos( betax[i] ); // topography-following pixel spacing
        dy[i] = sico.CSZ / cos( betay[i] );
    }


// Converting heights into depths

    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels of reduced coordinate system:

        if ( sico.MODEL == 1 ) aw[i][0] = fconvin( phrelease, phrelease, hflow_max0, 1,
            px, py, pelev, betax, betay, betaxy, i, sico ); // release depth

        else if ( sico.MODEL == 2 ) {
            aw[i][0] = fconvin( phrelease, phrelease2, hflow_max0+hflow_max02, 1,
                px, py, pelev, betax, betay, betaxy, i, sico ); // solid release depth
            aw[i][3] = fconvin( phrelease, phrelease2, hflow_max0+hflow_max02, 2,
                px, py, pelev, betax, betay, betaxy, i, sico ); // fluid release depth
        }
    }


// Writing input parameters to vectors and initializing volumes

    hflow_max = 0; // initializing maximum (solid) release depth
    if ( sico.MODEL == 2 ) hflow_max2 = 0; // initializing maximum fluid release depth
    hflow_maxmax = 0; // initializing absolute maximum flow depth
    vol_flow = 0; // initializing (solid) release volume
    vol_entr = 0; // initializing (solid) entrained or deposited volume
    if ( sico.MODEL == 2 ) {
        vol_flow2 = 0; // initializing fluid release volume
        vol_entr2 = 0; // initializing fluid entrained or deposited volume
    }

    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels of reduced coordinate system:

        dentr[i] = 0; // depth of (solid) entrainment

        cstopped[i] = 0; // control for stopping

        if ( sico.CORRHEIGHT == 0 ) carea = pow ( sico.CSZ, 2 );
        else carea = pow ( sico.CSZ , 2 ) * pow( 1 - pow( sin( betax[i] ) , 2 ) * pow ( sin( betay[i] ) , 2 ) , 0.5 )
            / ( cos( betax[i] ) * cos( betay[i] ) ); // topography-following area of pixel

        if ( sico.MODEL == 1 ) { // for one-phase model:

            for ( k=1; k<8; k++ ) aw[i][k] = 0;
            if ( sico.RELM == 1 ) aw[i][8] = aw[i][0];
            else aw[i][8] = 0;
            for ( k=9; k<13; k++ ) aw[i][k] = 0;
        }
        else if ( sico.MODEL == 2 ) { // for two-phase model:

            dentr2[i] = 0; // depth of fluid entrainment

            for ( k=1; k<3; k++ ) aw[i][k] = 0;
            for ( k=4; k<10; k++ ) aw[i][k] = 0;
            aw[i][10] = aw[i][0] + aw[i][3];
            for ( k=11; k<18; k++ ) aw[i][k] = 0;
            if ( sico.RELM == 1 ) aw[i][18] = aw[i][0];
            else aw[i][18] = 0;
            aw[i][19] = 0;
            if ( sico.RELM2 == 1 ) aw[i][20] = aw[i][3];
            else aw[i][20] = 0;
            aw[i][21] = 0;
            aw[i][22] = aw[i][0] + aw[i][3];
            for ( k=23; k<29; k++ ) aw[i][k] = 0;

            if ( aw[i][3] > hflow_max2 ) hflow_max2 = aw[i][3]; // updating maximum fluid release depth
            if ( hflow_max + hflow_max2 > hflow_maxmax ) hflow_maxmax = hflow_max + hflow_max2; // updating absolute maximum flow depth
            vol_flow2 += aw[i][3] * carea; // updating fluid release volume
        }

        if ( aw[i][0] > hflow_max ) hflow_max = aw[i][0]; // updating maximum (solid) release depth
        vol_flow += aw[i][0] * carea; // updating (solid) release volume
        if ( sico.MODEL == 1 && hflow_max > hflow_maxmax ) hflow_maxmax = hflow_max; // updating absolute maximum flow depth

        if ( hydrograph == 1 ) { // coordinates and elevation of centres of hydrographs:

            for ( hydj = 0; hydj < ( hydnin + hydnout ); hydj++ ) {
                if ( px[i] == hydx[hydj] && py[i] == hydy[hydj] ) {

                    hydi[hydj] = i;
                    hydelev[hydj] = pelev[i];
                }
            }
        }
    }

    vol_flow_bef = vol_flow; // (solid) flow volume at start
    vol_entr_bef = vol_entr; // (solid) entrained or deposited volume at start
    vol_noflux_bef = 0; // (solid) volume without fluxes at start
    if ( sico.MODEL == 2 ) { // for two-phase model:
        vol_flow2_bef = vol_flow2; // fluid flow volume at start
        vol_entr2_bef = vol_entr2; // fluid entrained or deposited volume at start
        vol_noflux2_bef = 0; // fluid volume without fluxes at start
    }


// Defining hydrograph profiles

    if ( hydrograph == 1 ) { // if at least one input hydrograph is provided:

        hydp = alloc_imatrix( (int)(2*hydlmax/sico.CSZ+2), hydnin+hydnout ); // allocating memory to hydrograph profiles

        for ( hydj = 0; hydj < ( hydnin + hydnout ); hydj++ ) { // loop over all hydrographs:

            if ( hydalpha[hydj] < 0 ) hydalpha[hydj] = falpha ( betax[hydi[hydj]], betay[hydi[hydj]], sico ) + sico.PI / 2;
                // cross-slope direction
            if ( hydalpha[hydj] > 2 * sico.PI ) hydalpha[hydj] -= 2 * sico.PI;
            hydp0 = fhydp ( hydx[hydj], hydy[hydj], hydl[hydj], px, py, hydalpha[hydj], hydlmax, sico );
            for ( i=0; i<(int)(2*hydlmax/sico.CSZ+2); i++ ) hydp[i][hydj] = hydp0[i]; // hydrograph profile
            free( hydp0 );

            hydout_xmin_metric = ( float ) py[hydp[1][hydj]] * sico.CSZ + sico.BDWEST; // external coordinates of profile terminal points
            hydout_xmax_metric = ( float ) py[hydp[hydp[0][hydj]][hydj]] * sico.CSZ + sico.BDWEST;
            hydout_ymin_metric = sico.BDNORTH - ( float ) px[hydp[1][hydj]] * sico.CSZ;
            hydout_ymax_metric = sico.BDNORTH - ( float ) px[hydp[hydp[0][hydj]][hydj]] * sico.CSZ;

            if ( hydj < hydnin ) fprintf(f_hydout, "I%i\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n", hydj+1,
                hydout_xmin_metric, hydx_metric[hydj], hydout_xmax_metric, hydout_ymin_metric, hydy_metric[hydj], hydout_ymax_metric);
            else fprintf(f_hydout, "O%i\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n", hydj-hydnin+1,
                hydout_xmin_metric, hydx_metric[hydj], hydout_xmax_metric, hydout_ymin_metric, hydy_metric[hydj], hydout_ymax_metric);
                // writing hydrograph profile terminal points and centre to text file
        }
    }


// Initializing control variables

    vcelr = 0; // initial value of flow velocity plus wave speed
    tlength = 0.01; // initial length of time step (s)
    tsum = 0; // initial time (s)
    tint = 0; // initial time since last output (s)
    cflmax = 0; // initial maximum CFL value
    nsum = 1; // initial total number of time steps
    nout = 1; // initial number of output time steps
    ccontinue = 1; // control whether simulation should be continued (1 or 0)
    csuccess = 1; // control for success of simulation (1 or 0)
    coverflow = 0; // control for overflow i.e. part of the flow leaving the study area (1 or 0)
    ctrl_hydout = 1; // control for hydrograph output
    if ( sico.DIFFCTRL == 1 ) { for ( i=0; i<sico.IMAX; i++ ) { for ( j=1; j<9; j++ ) { cready[i][j] = 0; }}}
        // degree of fill of pixel with regard to flux in a given direction (ratio)


// Summary of release and initial state of the flow

    if ( sico.MODEL == 1 ) { // for one-phase model:

        if ( hflow_max0 >= 1000 ) prec_hflow = 0; // precision for summary of maximum flow heights and depths
        else if ( hflow_max0 >= 100 ) prec_hflow = 1;
        else if ( hflow_max0 >= 10 ) prec_hflow = 2;
        else if ( hflow_max0 >= 1 ) prec_hflow = 3;
        else if ( hflow_max0 >= 0.1 ) prec_hflow = 4;
        else if ( hflow_max0 == 0 ) prec_hflow = 2;
        else prec_hflow = 5;

        if ( vol_flow0 >= 100000000 ) prec_vol = 0; // precision for summary of flow volumes
        else if ( vol_flow0 >= 1000000 ) prec_vol = 1;
        else if ( vol_flow0 >= 100000 ) prec_vol = 2;
        else if ( vol_flow0 >= 10000 ) prec_vol = 3;
        else if ( vol_flow0 >= 1000 ) prec_vol = 4;
        else if ( vol_flow0 == 0 ) prec_vol = 0;
        else prec_vol = 5;

        prec_ekin = prec_vol-3; if ( prec_ekin < 0 ) prec_ekin = 0; // precision for summary of flow kinetic energies

        printf("   nout\tnsum\tcfl\ttlength\ttsum\tdmax\tvmax\trmax\tvolume\tekin\n");
        printf("   R\t-----\t-----\t-----\t-----\t*%.*f\t-----\t-----\t*%.*f\t-----\n",
            prec_hflow, hflow_max0, prec_vol, vol_flow0/1000 ); // displaying release parameters
        printf("   0\t0\t-----\t-----\t0.0\t%.*f\t0.00\t0.00\t%.*f\t-----\n",
            prec_hflow, hflow_max, prec_vol, vol_flow/1000 ); // displaying initial parameters
        fflush(stdout); // forcing immediate display

        fprintf(f_summary, "nout\tnsum\tcfl\ttlength\ttsum\tdmax\tvmax\trmax\tvolume\tekin\n");
        fprintf(f_summary, "R\t-----\t-----\t-----\t-----\t*%.*f\t-----\t-----\t*%.*f\t-----\n",
            prec_hflow, hflow_max0, prec_vol, vol_flow0 ); // writing release parameters to summary file
        fprintf(f_summary, "0\t0\t-----\t-----\t0.0\t%.*f\t0.00\t0.00\t%.*f\t\n", prec_hflow, hflow_max, prec_vol, vol_flow );
            // writing initial parameters to summary file
    }
    else if ( sico.MODEL == 2 ) { // for two-phase model:

        if ( hflow_max0 >= 1000 || hflow_max02 >= 1000 ) prec_hflow = 0;  // precision for summary of maximum flow heights and depths
        else if ( hflow_max0 >= 100 || hflow_max02 >= 100 ) prec_hflow = 1;
        else if ( hflow_max0 >= 10 || hflow_max02 >= 10 ) prec_hflow = 2;
        else if ( hflow_max0 >= 1 || hflow_max02 >= 1 ) prec_hflow = 3;
        else if ( hflow_max0 >= 0.1 || hflow_max02 >= 0.1 ) prec_hflow = 4;
        else if ( hflow_max0 == 0 && hflow_max02 == 0 ) prec_hflow = 2;
        else prec_hflow = 5;

        if ( vol_flow0 >= 100000000 || vol_flow02 >= 10000000 ) prec_vol = 0; // precision for summary of flow volumes
        else if ( vol_flow0 >= 1000000 || vol_flow02 >= 1000000 ) prec_vol = 1;
        else if ( vol_flow0 >= 100000 || vol_flow02 >= 100000 ) prec_vol = 2;
        else if ( vol_flow0 >= 10000 || vol_flow02 >= 10000 ) prec_vol = 3;
        else if ( vol_flow0 >= 1000 || vol_flow02 >= 1000 ) prec_vol = 4;
        else if ( vol_flow0 == 0 && vol_flow02 == 0 ) prec_vol = 0;
        else prec_vol = 5;

        prec_ekin = prec_vol-3; if ( prec_ekin < 0 ) prec_ekin = 0; // precision for summary of flow kinetic energies

        printf("   nout\tnsum\tcfl\ttlength\ttsum\tdmaxs\tvmaxs\tdmaxf\tvmaxf\tvolumes\tvolumef\tekin\n");
        printf("   R\t-----\t-----\t-----\t-----\t*%.*f\t-----\t*%.*f\t-----\t*%.*f\t*%.*f\t-----\n",
            prec_hflow, hflow_max0, prec_hflow, hflow_max02, prec_vol, vol_flow0/1000, prec_vol, vol_flow02/1000);
            // displaying release parameters
        printf("   0\t0\t-----\t-----\t0.0\t%.*f\t0.00\t%.*f\t0.00\t%.*f\t%.*f\t-----\n", prec_hflow, hflow_max, prec_hflow, hflow_max2,
            prec_vol, vol_flow/1000, prec_vol, vol_flow2/1000); // displaying initial parameters
        fflush(stdout); // forcing immediate display

        fprintf(f_summary, "nout\tnsum\tcfl\ttlength\ttsum\tdmaxs\tvmaxs\tdmaxf\tvmaxf\tvolumes\tvolumef\tekin\n");
        fprintf(f_summary, "R\t-----\t-----\t-----\t-----\t*%.*f\t-----\t*%.*f\t-----\t*%.*f\t*%.*f\t-----\n",
            prec_hflow, hflow_max0, prec_hflow, hflow_max02, prec_vol, vol_flow0, prec_vol, vol_flow02 );
            // writing release parameters to summary file
        fprintf(f_summary, "0\t0\t-----\t-----\t0.0\t%.*f\t0.00\t%.*f\t0.00\t%.*f\t%.*f\t-----\n",
            prec_hflow, hflow_max, prec_hflow, hflow_max2, prec_vol, vol_flow, prec_vol, vol_flow2 );
            // writing initial parameters to summary file
    }


// *** Start of loop over time steps until break criterion is met: -------------------------------------------------

    while ( tsum <= tmax && ccontinue == 1 ) {
        if ( tint >= tout ) tint -= tout; // resetting time interval for output, if required


// *** Start of loop over two steps, each moving the system half of a pixel (NOC scheme) ---------------------------

        for ( p=0; p<2; p++ ) {


        // Updating slopes, if necessary

            if ( sico.ENTRAINMENT > 0 || sico.STOPPING > 0 ) { // if entrainment/deposition or stopping are activated:

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( cdomain[i] != 0 ) { // if pixel is no edge pixel:

                        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                        betax[i] = fbeta( pelev[in[5]], pelev[in[2]], sico );
                        betay[i] = fbeta( pelev[in[4]], pelev[in[1]], sico );
                        betaxy[i] = fbetaxy( betax[i], betay[i] );

                        free( in ); // releasing memory
                    }
                }
            }


// ----- START -- Updating flow depths and velocities according to input hydrographs -------------------------------

            if ( hydrograph == 1 ) {

                for ( hydj = 0; hydj < hydnin; hydj++ ) { // loop over all input hydrographs:

                    for ( hydk=0; hydk<=hydtmax[hydj]; hydk++ ) { // identifying relevant line of input hydrograph:
                        if ( hydhyd[hydk][0][hydj] <= tsum ) hydt=hydk;
                        else break;
                    }

                    if ( sico.MODEL == 1 ) hhyd0 = hydhyd[hydt][1][hydj]; // total flow height at centre of hydrograph
                    else if ( sico.MODEL == 2 ) hhyd0 = hydhyd[hydt][1][hydj] + hydhyd[hydt][3][hydj];

                    if ( hhyd0 >= sico.HFLOWMIN ) {

                        for ( hydk=0; hydk<= hydp[0][hydj]; hydk++ ) { // loop over all pixels of hydrograph profile:

                            hhyd = hhyd0 - ( pelev[hydp[hydk][hydj]] - hydelev[hydj] ); // total flow height at pixel

                            if ( hhyd >= sico.HFLOWMIN ) {

                                aw[hydp[hydk][hydj]][0] = hydhyd[hydt][1][hydj] * hhyd / hhyd0 * cos( betaxy[hydp[hydk][hydj]] );
                                    // (solid) flow depth
                                aw[hydp[hydk][hydj]][1] = hydhyd[hydt][1][hydj] * hhyd / hhyd0 * hydhyd[hydt][2][hydj]
                                    * cos( hydalpha[hydj] - sico.PI / 2 )  * cos( betaxy[hydp[hydk][hydj]] ); // (solid) x momentum
                                aw[hydp[hydk][hydj]][2] = hydhyd[hydt][1][hydj] * hhyd / hhyd0 * hydhyd[hydt][2][hydj]
                                    * sin( hydalpha[hydj] - sico.PI / 2 )  * cos( betaxy[hydp[hydk][hydj]] ); // (solid) y momentum
                            }
                            else {
                                aw[hydp[hydk][hydj]][0] = 0;
                                aw[hydp[hydk][hydj]][1] = 0;
                                aw[hydp[hydk][hydj]][2] = 0;
                            }
                            if ( hhyd >= sico.HFLOWMIN && sico.MODEL == 2 ) {

                                aw[hydp[hydk][hydj]][3] = hydhyd[hydt][3][hydj] * hhyd / hhyd0  * cos( betaxy[hydp[hydk][hydj]] );
                                    // fluid flow depth
                                aw[hydp[hydk][hydj]][4] = hydhyd[hydt][3][hydj] * hhyd / hhyd0 * hydhyd[hydt][4][hydj]
                                    * cos( hydalpha[hydj] - sico.PI / 2 ) * cos( betaxy[hydp[hydk][hydj]] ); // fluid x momentum
                                aw[hydp[hydk][hydj]][5] = hydhyd[hydt][3][hydj] * hhyd / hhyd0 * hydhyd[hydt][4][hydj]
                                    * sin( hydalpha[hydj] - sico.PI / 2 ) * cos( betaxy[hydp[hydk][hydj]] ); // fluid y momentum
                            }
                            else if ( sico.MODEL == 2 ) {
                                aw[hydp[hydk][hydj]][3] = 0;
                                aw[hydp[hydk][hydj]][4] = 0;
                                aw[hydp[hydk][hydj]][5] = 0;
                            }
                        }
                    }

// ----- STOP --- Updating flow depths and velocities according to input hydrographs -------------------------------


// ----- START -- Writing hydrograph infos to files ----------------------------------------------------------------

                    if ( sico.MULT == 0 && ctrl_hydout == 1 ) { // for first time step after output:

                        hydh = aw[hydi[hydj]][0] / cos( betaxy[hydi[hydj]] ); // (solid) flow height
                        if ( hydh >= sico.HFLOWMIN )
                            hydv = pow( pow( aw[hydi[hydj]][1], 2 ) + pow( aw[hydi[hydj]][2], 2 ), 0.5 ) / aw[hydi[hydj]][0];
                        else hydv= 0; // (solid) flow velocity

                        if ( sico.MODEL == 1 ) {

                            if ( hydh >= sico.HFLOWMIN ) hydr = aw[hydi[hydj]][3] / aw[hydi[hydj]][0];
                            else hydr= 0; // flow random kinetic energy
                            hyde = aw[hydi[hydj]][4]; // entrained height
                            hydh2 = 0; hydv2 = 0; hyde2 = 0;
                         }
                        else if ( sico.MODEL == 2 ) {

                            hydh2 = aw[hydi[hydj]][3] / cos( betaxy[hydi[hydj]] ); // fluid flow height
                            if ( hydh2 >= sico.HFLOWMIN )
                                hydv2 = pow( pow( aw[hydi[hydj]][4], 2 ) + pow( aw[hydi[hydj]][5], 2 ), 0.5 ) / aw[hydi[hydj]][3];
                            else hydv2= 0; // fluid flow velocity
                            hyde = aw[hydi[hydj]][6]; // entrained solid height
                            hyde2 = aw[hydi[hydj]][6]; // entrained fluid height
                            hydr = 0;
                        }

                        hydq = 0; // resetting (solid) discharge
                        if ( sico.MODEL == 2 ) hydq2 = 0; // resetting fluid discharge

                        if ( hydalpha[hydj] < sico.PI / 2 ) { hydfx = 1; hydfy = 1; } // negative/positive signs depending on direction:
                        else if ( hydalpha[hydj] < sico.PI ) { hydfx = -1; hydfy = 1; }
                        else if ( hydalpha[hydj] < 3 * sico.PI / 2 ) { hydfx = -1; hydfy = -1; }
                        else { hydfx = 1; hydfy = -1; }

                        if ( hydalpha[hydj] == 0 || hydalpha[hydj] == sico.PI / 2 || hydalpha[hydj] == sico.PI
                            || hydalpha[hydj] == 3 * sico.PI / 2 ) hydfalpha = 1;
                        else if ( fabs( 1 / sin( hydalpha[hydj] )) < fabs( 1 / cos( hydalpha[hydj] )))
                            hydfalpha = fabs( 1 / sin( hydalpha[hydj] ));
                        else hydfalpha = fabs( 1 / cos( hydalpha[hydj] )); // correction factor for profile direction

                        for ( hydk = 1; hydk <= hydp[0][hydj]; hydk++ ) { // loop over all pixels of profile:

                            alpha = falpha( betax[hydp[hydk][hydj]], betay[hydp[hydk][hydj]], sico ); // aspect
                            hydbeta = atan ( tan( betaxy[hydp[hydk][hydj]] ) * cos ( alpha - hydalpha[hydj] + sico.PI / 2 )); // corrected slope

                            hydfcorr = sico.CSZ * hydfalpha / cos( hydbeta ); // reference length for discharge
                            hydm0 = pow( pow(aw[hydp[hydk][hydj]][1], 2 ) + pow( aw[hydp[hydk][hydj]][2], 2), 0.5 );

                            if ( aw[hydp[hydk][hydj]][0] >= sico.HFLOWMIN && hydm0 > 0 ) {

                                hydmx = aw[hydp[hydk][hydj]][1];
                                hydmy = aw[hydp[hydk][hydj]][2];

                                if ( hydmy > 0 ) hydmalpha = acos( hydmx / hydm0 );
                                else hydmalpha = 2 * sico.PI - acos( hydmx / hydm0 );
                                hydnalpha = hydalpha[hydj] - sico.PI / 2; if ( hydnalpha < 0 ) hydnalpha += 2 * sico.PI;
                                hydm = hydm0 * cos ( hydnalpha - hydmalpha ); // momentum in direction perpendicular to profile
                                hydq += hydm * hydfcorr; // updating (solid) discharge
                            }

                            if ( sico.MODEL == 2 ) {

                                hydm0 = pow( pow(aw[hydp[hydk][hydj]][4], 2 ) + pow( aw[hydp[hydk][hydj]][5], 2), 0.5 );

                                if ( aw[hydp[hydk][hydj]][3] >= sico.HFLOWMIN && hydm0 > 0 ) {

                                    hydmx = aw[hydp[hydk][hydj]][4];
                                    hydmy = aw[hydp[hydk][hydj]][5];

                                    if ( hydmy > 0 ) hydmalpha = acos( hydmx / hydm0 );
                                    else hydmalpha = 2 * sico.PI - acos( hydmx / hydm0 );
                                    hydnalpha = hydalpha[hydj] - sico.PI / 2; if ( hydnalpha < 0 ) hydnalpha += 2 * sico.PI;
                                    hydm = hydm0 * cos ( hydnalpha - hydmalpha ); // momentum in direction perpendicular to profile
                                    hydq2 += hydm * hydfcorr; // updating fluid discharge
                                }
                            }
                            else hydq2 = 0;
                        }

                        fprintf(f_hydinfo[hydj], "%.1f\t%.2f\t%.2f\t%.2f\t%.2f\t%.0f\t%.2f\t%.2f\t%.2f\t%.0f\n",
                            tsum + 2 * tlength, hydh, hydv, hydr, hyde, hydq, hydh2, hydv2, hyde2, hydq2);
                            // writing hydrograph info to file

                        ctrl_hydout = 0; // resetting control for hydrograph output
                    }
               }
            }

// ----- STOP --- Writing hydrograph infos to files ----------------------------------------------------------------


// *** Start of loop over various time step lengths until CFL criterion is met -------------------------------------

            ccfl = 0; // resetting control for fulfilment of CFL criterion
            while ( ccfl == 0 ) {


// F4 -- START -- Suppression of oscillations ----------------------------------------------------------------------

                // *** cflux[i] = 0: fluxes are not computed in order to avoid oscillations of horizontal flow surfaces
                // *** cflux[i] = 1: fluxes are computed

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( sico.NOOSC == 1 ) { // if function is enabled:

                        cflux[i] = 0; // setting initial value of flux control to negative
                        if ( cdomain[i] != 0 ) { // if pixel is no 1st row edge pixel, testing whether fluxes should be computed or not:

                            in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                            for ( j=0; j<9; j++ ) { // loop over neighbourhood:

                                if ( sico.CORRHEIGHT == 0 ) { fflux = 1; ffluxn = 1; } // conversion factors between flow depths and heights
                                else { fflux = 1 / cos( betaxy[i] ); ffluxn = 1 / cos( betaxy[in[j]] ); }

                                if ( sico.MODEL == 1 ) { // one-phase model:

                                    if ( fabs(( pelev[in[j]] + ffluxn * aw[in[j]][0] )
                                        - ( pelev[i] + fflux * aw[i][0] )) >= sico.HFLOWMIN
                                        && ( ffluxn * aw[in[j]][0] >= sico.HFLOWMIN
                                        || pelev[in[j]] < pelev[i] )) cflux[i] = 1;
                                        // setting flux control to positive if inflow or outflow is possible

                                }
                                else if ( sico.MODEL == 2 ) { // two-phase model:

                                    if ( fabs(( pelev[in[j]] + ffluxn * ( aw[in[j]][0] + aw[in[j]][3] ))
                                        - ( pelev[i] + fflux * ( aw[i][0] + aw[i][3] ))) >= sico.HFLOWMIN
                                        && ( ffluxn * ( aw[in[j]][0] + aw[in[j]][3] ) >= sico.HFLOWMIN
                                        || pelev[in[j]] < pelev[i] )) cflux[i] = 1;
                                        // setting flux control to positive if inflow or outflow is possible
                                }
                            }

                            free( in ); // releasing memory
                        }
                    }
                    else cflux[i] = 1; // if function is disabled, setting flux control to positive
                }


// F4 -- STOP --- Suppression of oscillations ----------------------------------------------------------------------


// ----- START -- Updating computational domains -------------------------------------------------------------------

                // *** cdomain[i] = 0: 1st row edge pixel
                // *** cdomain[i] = 1: no-flux pixel (suppression of oscillations)
                // *** cdomain[i] = 3: included pixels, flow depth < minimum at & around pixel
                // *** cdomain[i] = 4: included pixels, flow depth >= minimum around pixel (flow boundary)
                // *** cdomain[i] = 5: included pixels, flow depth >= minimum (flow)

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                    if ( cdomain[i] != 0 ) { // for all non-edge pixels:

                        ctrlv = 0; ctrlr = 1; // resetting controls
                        for ( j=1; j<9; j++ ) { // loop over neighbourhood:

                            if ( cflux[i] == 1 && ( cflux[in[j]] == 1
                                && ( aw[in[j]][0] >= sico.HFLOWMIN || ( sico.MODEL == 2 && aw[in[j]][3] >= sico.HFLOWMIN )))) ctrlv = 1;
                                // setting control for pixel to positive if not at or surrounded by no-flux area

                            if ( cstopped[i] == 1 || cstopped[in[j]] == 1 ) ctrlr = 0;
                                // setting control for pixel to negative if at or adjacent to stopped area
                        }
                        if ( ctrlv == 1 && ctrlr == 1 ) cdomain[i] = 3; // updating computational domain
                    }

                    if ( cdomain[i] == 3 ) { // if pixel is part of computational domain:

                        ctrlr = 0; // resetting control

                        for ( j=1; j<9; j++ ) { // loop over neighbourhood:
                            if ( sico.MODEL == 1 && aw[in[j]][0] >= sico.HFLOWMIN  ) ctrlr = 1;
                            else if ( sico.MODEL == 2
                                && ( aw[in[j]][0] >= sico.HFLOWMIN || aw[in[j]][3] >= sico.HFLOWMIN )) ctrlr = 1;
                                // if neighbour pixel is flow pixel, setting control to positive
                        }
                        if ( ctrlr == 1 ) cdomain[i] = 4; // flow depth >= minimum around pixel

                        if ( sico.MODEL == 1 && aw[i][0] > sico.HFLOWMIN  ) cdomain[i] = 5;
                        else if ( sico.MODEL == 2
                            && ( aw[i][0] >= sico.HFLOWMIN || aw[i][3] >= sico.HFLOWMIN )) cdomain[i] = 5; // flow depth >= minimum
                    }
                    free( in ); // releasing memory
                }

// ----- STOP --- Updating computational domains -------------------------------------------------------------------


// ----- START -- Preparing flow boundary --------------------------------------------------------------------------

                // *** Setting all flow heights at and around the flow boundary to the minimum flow depth instead of zero
                // *** Purpose: avoidance of unstable numerics

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                    if ( cdomain[i] == 0 ) { for ( k=1; k<nvect_min; k++ ) aw[i][k] = 0; } // setting edge values to zero

                    if ( cdomain[i] > 1 && aw[i][0] < sico.HFLOWMIN && csolid == 1 ) {
                        // if (solid) flow depth is smaller than minimum:

                        ctrlr = 0; ctrlv = 1; // resetting controls
                        for ( j=0; j<9; j++ ) { // loop over all neighbour pixles:

                            if ( cdomain[in[j]] > 3 ) ctrlr = 1; // including pixels at and adjacent to flow boundaries
                            if ( cdomain[in[j]] == 1 ) ctrlv = 0; // excluding no-flux pixels
                        }
                        if ( ctrlr == 1 && ctrlv == 1 ) { // if both controls are positive:

                            if ( aw[i][0] < sico.HFLOWMIN ) aw[i][0] = sico.HFLOWMIN; // setting (solid) flow depth to minimum
                            for ( k=1; k<3; k++ ) aw[i][k] = 0; // setting (solid) momenta to zero
                        }
                    }
                    if ( sico.MODEL == 2 && cdomain[i] > 1 && aw[i][3] < sico.HFLOWMIN && cfluid == 1 ) {
                        // if fluid flow depth is smaller than minimum:

                        ctrlr = 0; ctrlv = 1; // resetting controls
                        for ( j=0; j<9; j++ ) { // loop over all neighbour pixels:

                            if ( cdomain[in[j]] > 3 ) ctrlr = 1; // including pixels at and adjacent to flow boundaries
                            if ( cdomain[in[j]] == 1 ) ctrlv = 0; // excluding no-flux pixels
                        }
                        if ( ctrlr == 1 && ctrlv == 1 ) { // if both controls are positive:

                            if ( aw[i][3] < sico.HFLOWMIN ) aw[i][3] = sico.HFLOWMIN; // setting fluid flow depth to minimum
                            for ( k=4; k<6; k++ ) aw[i][k] = 0; // setting fluid momenta to zero
                        }
                    }
                    free( in ); // releasing memory
                }

// ----- STOP --- Preparing flow boundary --------------------------------------------------------------------------


// ----- START -- Flow propagation with NOC scheme -----------------------------------------------------------------

                // *** High-resolution scheme, system is moved half a pixel size each time step
                // *** Technically, the system is shifted one pixel each second time step
                // *** At the end, the new state variables are written to a temporary vector (awt[i][k])
                // *** The values from the temporary vector are written to the permanent vector (aw[i][k])
                // ***  as soon as it has been verified that the time step length is sufficiently short
                // ***  to fulfil the CFL criterion

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( cdomain[i] > 3 ) { // if pixel is in or adjacent to flow area:

                        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood
                        if ( sico.DELTA == 1 ) {
                            if ( pdelta[i] != sico.UNDEF ) sflow.DELTA = pdelta[i]; // basal friction angle
                            else sflow.DELTA = sflow.DELTA0;
                        }
                        else sflow.DELTA = sflow.DELTA0;
                        sflow.MY = tan( sflow.DELTA ); // basal friction coefficient


                    // Fluxes, source terms and gradients (original coordinate system)

                        gz = fgz( betaxy[i], sico ); // components of gravity
                        gx = fgd( betax[i], betay[i], betax[i], 1, sico );
                        gy = fgd( betax[i], betay[i], betay[i], 2, sico );

                        if ( aw[i][0] >= sico.HFLOWMIN ) vflow=pow(pow(aw[i][1]/aw[i][0],2)+pow(aw[i][2]/aw[i][0],2),0.5);
                        else vflow=0; // (solid) flow velocity

                        if ( aw[in[2]][0] >= sico.HFLOWMIN ) vflowxp = aw[in[2]][1]/aw[in[2]][0]; else vflowxp = 0;
                        if ( aw[in[5]][0] >= sico.HFLOWMIN ) vflowxm = aw[in[5]][1]/aw[in[5]][0]; else vflowxm = 0;
                        if ( aw[in[1]][0] >= sico.HFLOWMIN ) vflowyp = aw[in[1]][2]/aw[in[1]][0]; else vflowyp = 0;
                        if ( aw[in[4]][0] >= sico.HFLOWMIN ) vflowym = aw[in[4]][2]/aw[in[4]][0]; else vflowym = 0;
                            // velocities of neighbour pixels

                        dvflowx=vflowxp-vflowxm; // gradient of (solid) flow velocity in x direction
                        dvflowy=vflowyp-vflowym; // gradient of (solid) flow velocity in y direction
                        if ( sico.MODEL == 1 ) dvflow = dvflowx + dvflowy; // gradient of flow velocity
                        else if ( sico.MODEL == 2 ) gk = fk2 ( dvflowx, dvflowy, sflow ); // earth pressure coefficients

                        if ( sico.CORRHEIGHT == 0 ) corrfact = 1; // correction factor for flow height to depth conversion
                        else corrfact = 1 / cos( betaxy[i] );

                        if ( sico.MODEL == 1 ) { // for one-phase model:

                            potentr = 0;

                            if ( aw[i][0] >= sico.HFLOWMIN ) vflowx = aw[i][1] / aw[i][0]; else vflowx = 0;
                            if ( aw[i][0] >= sico.HFLOWMIN ) vflowy = aw[i][2] / aw[i][0]; else vflowy = 0;
                            if ( aw[i][0] >= sico.HFLOWMIN ) rflow = aw[i][3] / aw[i][0]; else rflow = 0;

                            gf = ff1 ( aw[i][0], vflowx, vflowy, dvflow, rflow, gz, sflow, sico ); // flux in x direction
                            gg = fg1 ( aw[i][0], vflowx, vflowy, dvflow, rflow, gz, sflow, sico ); // flux in y direction
                            gs = fs1 ( aw[i][0], vflowx, vflowy, vflow, rflow, gx, gy, gz, sflow, sico, sbase, potentr ); // source term
                        }
                        else if ( sico.MODEL == 2 ) { // for two-phase model:

                            whflowx = (( aw[in[2]][0] + aw[in[2]][3] ) - ( aw[in[5]][0] + aw[in[5]][3] )) / ( 2 * dx[i] );
                                // x gradient of total flow depth
                            whflowy = (( aw[in[1]][0] + aw[in[1]][3] ) - ( aw[in[4]][0] + aw[in[4]][3] )) / ( 2 * dy[i] );
                                // y gradient of total flow depth

                            if ( aw[i][0] >= sico.HFLOWMIN ) vflowx = aw[i][1] / aw[i][0]; else vflowx = 0;
                                // solid and fluid x and y velocities
                            if ( aw[i][3] >= sico.HFLOWMIN ) vflowx2 = aw[i][4] / aw[i][3]; else vflowx2 = 0;
                            if ( aw[i][0] >= sico.HFLOWMIN ) vflowy = aw[i][2] / aw[i][0]; else vflowy = 0;
                            if ( aw[i][3] >= sico.HFLOWMIN ) vflowy2 = aw[i][5] / aw[i][3]; else vflowy2 = 0;

                            welev[0] = pelev[i];
                            for ( l=1; l<9; l++ ) { // solid and fluid velocities, solid ratios and elevation of neighbour pixels:
                                if ( aw[in[l]][0] >= sico.HFLOWMIN ) wvflowx[l] = aw[in[l]][1] / aw[in[l]][0]; else wvflowx[l] = 0;
                                if ( aw[in[l]][0] >= sico.HFLOWMIN ) wvflowy[l] = aw[in[l]][2] / aw[in[l]][0]; else wvflowy[l] = 0;
                                if ( aw[in[l]][3] >= sico.HFLOWMIN ) wvflowx2[l] = aw[in[l]][4] / aw[in[l]][3]; else wvflowx2[l] = 0;
                                if ( aw[in[l]][3] >= sico.HFLOWMIN ) wvflowy2[l] = aw[in[l]][5] / aw[in[l]][3]; else wvflowy2[l] = 0;
                                if ( aw[in[l]][0] + aw[in[l]][3] >= sico.HFLOWMIN )
                                    walpha[l] = aw[in[l]][0] / ( aw[in[l]][0] + aw[in[l]][3] );
                                else walpha[l] = 0;
                                welev[l] = pelev[in[l]];
                            }

                            potentr = 0; potentr2 = 0;

                            gf = ff2 ( aw[i][0], aw[i][3], vflowx, vflowx2, vflowy, vflowy2,
                                gk[0], gz, sflow, sico ); // flux in x direction
                            gg = fg2 ( aw[i][0], aw[i][3], vflowx, vflowx2, vflowy, vflowy2,
                                gk[1], gz, sflow, sico ); // flux in y direction
                            gs = fs2 ( aw[i][0], aw[i][3], vflowx, vflowx2, vflowy, vflowy2, whflowx, whflowy, wvflowx, wvflowy,
                                wvflowx2, wvflowy2, walpha, welev, gx, gy, gz, sflow, dx, dy, i, sico, betax, betay, betaxy,
                                potentr, potentr2, sbase ); // source term
                        }
                        for ( k=0; k<nvect_min; k++ ) { af[i][k] = gf[k]; ag[i][k] = gg[k]; as[i][k] = gs[k]; }
                            // assigning function returns to vectors
                        if ( sico.MODEL == 2 ) free(gk); free(gf); free(gg); free(gs); // releasing memory

                        for (k=0; k<nvect_min; k++) { // slopes of the vector components:

                            asigma_x[i][k] = fsigma ( aw[i][k], aw[in[5]][k], aw[in[2]][k], 1, dx, dy, i );
                            asigma_y[i][k] = fsigma ( aw[i][k], aw[in[4]][k], aw[in[1]][k], 2, dx, dy, i );
                        }

                        free( in ); // releasing memory
                    }
                    else { // setting all components of the vectors to zero for irrelevant pixels:
                        for ( k=0; k<nvect_min; k++ ) { af[i][k]=0; ag[i][k]=0; as[i][k]=0; asigma_x[i][k]=0; asigma_y[i][k]=0; }
                    }
                }


            // Computing slopes of the fluxes

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:
                    if ( cdomain[i] > 3 ) { // if pixel is in or adjacent to flow area:

                        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                        for ( k=0; k<nvect_min; k++ ) { // slopes of the the components of the fluxes:
                            asigma_f[i][k] = fsigma ( af[i][k], af[in[5]][k], af[in[2]][k], 1, dx, dy, i );
                            asigma_g[i][k] = fsigma ( ag[i][k], ag[in[4]][k], ag[in[1]][k], 2, dx, dy, i );
                        }
                        free( in ); // releasing memory

                    }
                    else { // setting all components of the vectors to zero for irrelevant pixels:
                        for ( k=0; k<nvect_min; k++) { asigma_f[i][k]=0; asigma_g[i][k]=0; }
                    }
                }


            // Computing value of vector at quarter of pixel after half time step

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:
                    if ( cdomain[i] > 3 ) {  // if pixel is in or adjacent to flow area:

                        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                        fj[0][0]=1; fj[0][1]=1; fj[1][0]=1; fj[1][1]=-1; fj[2][0]=-1; fj[2][1]=1; fj[3][0]=-1; fj[3][1]=-1;
                            // factors for negative terms

                        for ( j=0; j<6; j++ ) { // loop over adjacent pixels:
                            for ( k=0; k<nvect_min; k++ ) { // loop over all components of the vector:

                                dw_dt=-asigma_f[in[j]][k]-asigma_g[in[j]][k]+as[in[j]][k]; // gradient of vector
                                if ( j < 4 ) wintc[i][j][k]=aw[in[j]][k]+0.5*tlength*dw_dt+fj[j][0]*0.25*dx[i]*asigma_x[in[j]][k]
                                   +fj[j][1]*0.25*dy[i]*asigma_y[in[j]][k]; // value of vector at quarter of pixel after half time step
                            }
                        }
                        free( in ); // releasing memory
                    }
                    else { // setting all components of the vectors to zero for irrelevant pixels:
                        for ( j=0; j<6; j++ ) { // loop over neighbours:
                            for ( k=0; k<nvect_min; k++ ) { // loop over all components of the vector:

                                dw_dt=0; // setting gradient of vector to zero
                                if ( j < 4 ) wintc[i][j][k]=0; // setting value of vector at quarter of pixel after half time step to zero
                            }
                        }
                    }
                }

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:
                    if ( cdomain[i] > 3 ) { // if pixel is in or adjacent to flow area:

                        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood
                        if ( sico.DELTA == 1 ) {
                            if ( pdelta[i] != sico.UNDEF ) sflow.DELTA = pdelta[i]; // basal friction angle
                            else sflow.DELTA = sflow.DELTA0;
                        }
                        else sflow.DELTA = sflow.DELTA0;
                        sflow.MY = tan( sflow.DELTA ); // basal friction coefficient


                    // Computing value of vector after half time step

                        fj[0][0]=1; fj[0][1]=1; fj[1][0]=1; fj[1][1]=-1; fj[2][0]=-1; fj[2][1]=1; fj[3][0]=-1; fj[3][1]=-1;
                            // factors for negative terms

                        for ( j=0; j<6; j++ ) { // loop over adjacent pixels:
                            for ( k=0; k<nvect_min; k++ ) { // loop over all components of the vector:

                                if ( j < 4 ) winta[j][k]=aw[in[j]][k]+fj[j][0]*0.25*dx[i]*asigma_x[in[j]][k]+fj[j][1]
                                    *0.25*dy[i]*asigma_y[in[j]][k]; // value at quarter of pixel
                                dw_dt=-asigma_f[in[j]][k]-asigma_g[in[j]][k]+as[in[j]][k]; // gradient of vector
                                wintb[j][k]=aw[in[j]][k]+0.5*tlength*dw_dt;
                                // value of vector after half time step (Taylor extrapolation in time)
                            }
                        }

                        for ( j=0; j<4; j++ ) { // loop over adjacent pixels:

                            if ( sico.MODEL == 1 && wintc[i][j][0] >= sico.HFLOWMIN ) hctrl = 1;
                            else if ( sico.MODEL == 2 && ( wintc[i][j][0] >= sico.HFLOWMIN || wintc[i][j][3] >= sico.HFLOWMIN ))
                                hctrl = 1;
                            else hctrl = 0; // control for flow depth at quarter of pixel after half time step


                        // Fluxes, source terms and gradients (shifted coordinate system)

                            if ( hctrl == 1 ) { // if flow depth is equal to or larger than minimum:

                                gz = fgz( betaxy[i], sico ); // components of gravity
                                gx = fgd( betax[i], betay[i], betax[i], 1, sico );
                                gy = fgd( betax[i], betay[i], betay[i], 2, sico );
                            }

                            if ( sico.MODEL == 1 && wintb[j][0] >= sico.HFLOWMIN ) hctrl = 1;
                            else if ( sico.MODEL == 2 && ( wintb[j][0] >= sico.HFLOWMIN || wintb[j][3] >= sico.HFLOWMIN )) hctrl = 1;
                            else hctrl = 0;  // control for flow depth after half time step

                            if ( hctrl == 0 ) for ( k=0; k<nvect_min; k++ ) { f[j][k]=0; g[j][k]=0; }
                                // setting vectors to zero if flow depth is zero
                            else {

                                if ( wintb[j][0] >= sico.HFLOWMIN )
                                    vflow=pow(pow(wintb[j][1]/wintb[j][0],2)+pow(wintb[j][2]/wintb[j][0],2),0.5);
                                else vflow=0; // (solid) flow velocity

                                if ( wintb[2][0] >= sico.HFLOWMIN && wintb[5][0] >= sico.HFLOWMIN )
                                    dvflowx=(wintb[2][1]/wintb[2][0])-(wintb[5][1]/wintb[5][0]); else dvflowx=0;
                                    // gradient of (solid) flow velocity in x direction

                                if ( wintb[1][0] >= sico.HFLOWMIN && wintb[4][0] >= sico.HFLOWMIN )
                                    dvflowy=(wintb[1][2]/wintb[1][0])-(wintb[4][2]/wintb[4][0]); else dvflowy=0;
                                    // gradient of (solid) flow velovity in y direction

                                if ( sico.MODEL == 1 ) dvflow=dvflowx+dvflowy; // gradient of flow velocity

                                if ( sico.MODEL == 1 ) {

                                    gf = ff1 ( wintb[j][0], wintb[j][1]/wintb[j][0], wintb[j][2]/wintb[j][0], dvflow,
                                        wintb[j][3]/wintb[j][0], gz, sflow, sico ); // flux in x direction
                                    gg = fg1 ( wintb[j][0], wintb[j][1]/wintb[j][0], wintb[j][2]/wintb[j][0], dvflow,
                                        wintb[j][3]/wintb[j][0], gz, sflow, sico ); // flux in y direction
                                }
                                else if ( sico.MODEL == 2 ) {

                                    if (wintb[j][0] >= sico.HFLOWMIN) vflowx = wintb[j][1]/wintb[j][0]; else vflowx = 0;
                                    if (wintb[j][3] >= sico.HFLOWMIN) vflowx2 = wintb[j][4]/wintb[j][3]; else vflowx2 = 0;
                                    if (wintb[j][0] >= sico.HFLOWMIN) vflowy = wintb[j][2]/wintb[j][0]; else vflowy = 0;
                                    if (wintb[j][3] >= sico.HFLOWMIN) vflowy2 = wintb[j][5]/wintb[j][3]; else vflowy2 = 0;
                                        // solid and fluid x and y velocities

                                    gk = fk2 ( dvflowx, dvflowy, sflow ); // earth pressure coefficients

                                    gf = ff2 ( wintb[j][0], wintb[j][3], vflowx, vflowx2, vflowy, vflowy2,
                                        gk[0], gz, sflow, sico ); // flux in x direction
                                    gg = fg2 ( wintb[j][0], wintb[j][3], vflowx, vflowx2, vflowy, vflowy2,
                                        gk[1], gz, sflow, sico ); // flux in y direction

                                }

                                for ( k=0; k<nvect_min; k++ ) { f[j][k] = gf[k]; g[j][k] = gg[k]; }
                                if ( sico.MODEL == 2 ) free(gk); free(gf); free(gg);
                            }

                            if ( sico.MODEL == 1 && wintc[i][j][0] >= sico.HFLOWMIN ) hctrl = 1;
                            else if ( sico.MODEL == 2 && ( wintc[i][j][0] >= sico.HFLOWMIN || wintc[i][j][3] >= sico.HFLOWMIN ))
                                hctrl = 1;
                            else hctrl = 0; // control for flow depth at quarter of pixel after half time step

                            if ( hctrl == 0 ) { for ( k=0; k<nvect_min; k++ ) s[j][k]=0; }
                                // setting vectors to zero if flow depth is zero
                            else {

                                if ( wintc[i][j][0] >= sico.HFLOWMIN )
                                    vflow=pow(pow(wintc[i][j][1]/wintc[i][j][0],2)+pow(wintc[i][j][2]/wintc[i][j][0],2),0.5);
                                else vflow = 0; // (solid) flow velocity

                                if ( sico.CORRHEIGHT == 0 ) corrfact = 1; // correction factor for flow height to depth conversion
                                else corrfact = 1 / cos( betaxy[i] );

                                if ( sico.MODEL == 1 ) {

                                    potentr = 0;

                                    gs = fs1 ( wintc[i][j][0], wintc[i][j][1]/wintc[i][j][0], wintc[i][j][2]/wintc[i][j][0], vflow,
                                        wintc[i][j][3]/wintc[i][j][0], gx, gy, gz, sflow, sico, sbase, potentr ); // source term
                                }
                                else if ( sico.MODEL == 2 ) {

                                    whflowx = (( wintc[in[2]][j][0] + wintc[in[2]][j][3] ) - ( wintc[in[5]][j][0] + wintc[in[5]][j][3] ))
                                        / ( 2 * dx[i] );// x gradient of total flow depth
                                    whflowy = (( wintc[in[1]][j][0] + wintc[in[1]][j][3] ) - ( wintc[in[4]][j][0] + wintc[in[4]][j][3] ))
                                        / ( 2 * dy[i] ); // y gradient of total flow depth

                                    if ( wintc[i][j][0] >= sico.HFLOWMIN ) vflowx = wintc[i][j][1]/wintc[i][j][0]; else vflowx = 0;
                                    if ( wintc[i][j][3] >= sico.HFLOWMIN ) vflowx2 = wintc[i][j][4]/wintc[i][j][3]; else vflowx2 = 0;
                                    if ( wintc[i][j][0] >= sico.HFLOWMIN ) vflowy = wintc[i][j][2]/wintc[i][j][0]; else vflowy = 0;
                                    if ( wintc[i][j][3] >= sico.HFLOWMIN ) vflowy2 = wintc[i][j][5]/wintc[i][j][3]; else vflowy2 = 0;
                                        // solid and fluid x and y velocities

                                    welev[0] = pelev[i];
                                    for ( l=1; l<9; l++ ) {
                                        // solid and fluid velocities, solid ratios and elevation of neighbouring pixels:

                                        if ( wintc[in[l]][j][0] >= sico.HFLOWMIN ) wvflowx[l] = wintc[in[l]][j][1] / wintc[in[l]][j][0];
                                        else wvflowx[l] = 0;
                                        if ( wintc[in[l]][j][0] >= sico.HFLOWMIN ) wvflowy[l] = wintc[in[l]][j][2] / wintc[in[l]][j][0];
                                        else wvflowy[l] = 0;
                                        if ( wintc[in[l]][j][3] >= sico.HFLOWMIN ) wvflowx2[l] = wintc[in[l]][j][4] / wintc[in[l]][j][3];
                                        else wvflowx2[l] = 0;
                                        if ( wintc[in[l]][j][3] >= sico.HFLOWMIN ) wvflowy2[l] = wintc[in[l]][j][5] / wintc[in[l]][j][3];
                                        else wvflowy2[l] = 0;
                                        if ( wintc[in[l]][j][0] + wintc[in[l]][j][3] >= sico.HFLOWMIN ) walpha[l] = wintc[in[l]][j][0]
                                            / ( wintc[in[l]][j][0] + wintc[in[l]][j][3] ); else walpha[l] = 0;
                                        welev[l] = pelev[in[l]];
                                    }

                                    potentr = 0; potentr2 = 0;

                                    gs = fs2 ( wintc[i][j][0], wintc[i][j][3], vflowx, vflowx2, vflowy, vflowy2,
                                        whflowx, whflowy, wvflowx, wvflowy, wvflowx2, wvflowy2, walpha, welev,
                                        gx, gy, gz, sflow, dx, dy, i, sico, betax, betay, betaxy, potentr, potentr2, sbase ); // source term
                                }
                                for ( k=0; k<nvect_min; k++ ) s[j][k] = gs[k];
                                free(gs);
                            }
                        }


                    // Value of vector at top right corner of the pixel

                        for ( k=0; k<nvect_min; k++ ) { // loop over all components of the vector:

                            wintd[i][k]=0.25*(winta[0][k]+winta[1][k]+winta[2][k]+winta[3][k])
                                -tlength/dx[i]*(0.5*(f[2][k]+f[3][k])-0.5*(f[0][k]+f[1][k]))
                                -tlength/dy[i]*(0.5*(g[1][k]+g[3][k])-0.5*(g[0][k]+g[2][k]))
                                +0.25*tlength*(s[0][k]+s[1][k]+s[2][k]+s[3][k]); // value of vector at top right corner of the pixel
                        }
                        free( in );
                    }
                    else for ( k=0; k<nvect_min; k++ ) wintd[i][k]=aw[i][k];
                        // setting all components of the vectors to zero for irrelevant pixels:
                }


            // Moving vector if second sub-timestep and writing values to temporary vector

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( sico.TOPO == 1 ) { // pixel neighbourhood (only direction 6 is relevant):
                        if ( i%sico.N == 0 || (i+1)%sico.N == 0 || i < sico.N || i > sico.IMAX - sico.N ) in6 = -1;
                        else in6 = i - sico.N - 1;
                    }
                    else {
                        j = i; while ( ( px[j] != px[i]-1 || py[j] != py[i]-1 ) && j >= 0 && px[j] > px[i]-2 ) j -= 1; in6 = j;
                        if ( px[j] != px[i]-1 || py[j] != py[i]-1 ) in6 = -1;
                    }

                    ctrlv = 1; if ( in6 < 0 ) ctrlv = 0;
                    if ( ctrlv == 1 ) {
                        ctrlr = 0;
                        if ( sico.MODEL == 1 ) {
                            if ( wintd[i][0] >= sico.HFLOWMIN ) ctrlr = 1;
                            if ( wintd[in6][0] >= sico.HFLOWMIN ) ctrlr = 1;
                        }
                        else if ( sico.MODEL == 2 ) {
                            if ( wintd[i][0] >= sico.HFLOWMIN || wintd[i][3] >= sico.HFLOWMIN ) ctrlr = 1;
                            if ( wintd[in6][0] >= sico.HFLOWMIN || wintd[in6][3] >= sico.HFLOWMIN ) ctrlr = 1;
                        }
                    }

                    if ( ctrlv == 1 && ctrlr == 1 && cdomain[i] > 3 ) { // if pixel is valid and relevant:

                        for ( k=0; k<nvect_min; k++ ) { // loop over all components of the vector:
                            if ( p == 1 && ( cdomain[in6] > 3 || cflux[in6] == 0 )) awt[i][k]=wintd[in6][k];
                                // if second sub-timestep and relevant neighbour pixel: moving vector one cell
                            else if ( p != 1 ) awt[i][k]=wintd[i][k];
                                // if first sub-timestep and relevant neighbour pixel: not moving vector
                            else if (( sico.MODEL == 1 && k < 4 ) || ( sico.MODEL == 2 && k < 6 )) awt[i][k] = 0;
                                // else, setting temporary vector (except for entrainment and eposition) to zero
                            else awt[i][k] = aw[i][k];
                        }
                    }
                    else if ( sico.MODEL == 1 && cflux[i] != 0 && cstopped[i] != 1 ) { for ( k=0; k<4; k++ ) awt[i][k] = 0; }
                    else if ( sico.MODEL == 2 && cflux[i] != 0 && cstopped[i] != 1 ) { for ( k=0; k<6; k++ ) awt[i][k] = 0; }
                    else { for ( k=0; k<nvect_min; k++ ) awt[i][k] = aw[i][k]; }
                        // setting temporary vectors (except for entrainment and deposition) for all other (except no-flux) pixels to zero

                    for ( k=0; k<nvect_min; k++ ) { if ( isnan( awt[i][k] ) != 0 ) awt[i][k] = 0; } // setting no data values to zero
                }

// ----- STOP --- Flow propagation with NOC scheme -----------------------------------------------------------------


// F2 -- START -- Diffusion control --------------------------------------------------------------------------------

                // *** cedge [i][j] (cedge2 [i][j]): edge pixel with regard to solid (fluid) in direction j - 1 = yes, 2 = no
                // *** cready[i][j] (cready2[i][j]): degree of solid (fluid) fill of neighbour pixels in direction j - ratio
                // *** cedge0[i] (cedge02[i]): number of pixels which may provide solid (fluid) inflow
                // *** cneighbours[i] (cneighbours2[i]): number of neighbour pixels with non-zero flow depth


            // Resetting flow boundary

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:
                    if ( cdomain[i] != 5 && cstopped[i] != 1 && cflux[i] != 0 ) { for ( k=0; k<3; k++ ) aw[i][k] = 0; }
                    if ( sico.MODEL == 2 && cdomain[i] != 5 && cstopped[i] != 1 && cflux[i] != 0 ) { for ( k=3; k<6; k++ ) aw[i][k] = 0; }
                    // setting permanent vectors to zero outside of domain
                }

                if ( sico.DIFFCTRL == 1 ) { // if function is enabled:

                    for ( i=0; i<sico.IMAX; i++ ) { cedge0[i] = 0; cedge02[i] = 0; }


                // Updating degree of fill of neighbour pixels

                    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                        if ( cdomain[i] == 5 ) { // if pixel was flow pixel at the beginning of the time step:

                            in = fin ( i, px, py, pelev, sico ); // neighbourhood

                            if ( aw[i][0] >= sico.HFLOWMIN ) {
                                // identifying directions for which pixel is edge pixel (with regard to solid):

                                for ( j=1; j<9; j++ ) {
                                    if ( aw[in[j]][0] < sico.HFLOWMIN  ) cedge[i][j] = 1;
                                    else cedge[i][j] = 0;
                                }
                            }
                            if ( sico.MODEL == 2 && aw[i][3] >= sico.HFLOWMIN ) {
                                // identifying directions for which pixel is edge pixel with regard to fluid:

                                for ( j=1; j<9; j++ ) {
                                    if ( aw[in[j]][3] < sico.HFLOWMIN ) cedge2[i][j] = 1;
                                    else cedge2[i][j] = 0;
                                }
                            }
                            if ( nsum == 1 ) { // for first time step:

                                if ( cedge[i][2] == 1 && aw[i][0] >= sico.HFLOWMIN ) cready[i][2] = 2; // degree of fill (for solid)
                                if ( cedge[i][5] == 1 && aw[i][0] >= sico.HFLOWMIN ) cready[i][5] = 2;
                                if ( cedge[i][1] == 1 && aw[i][0] >= sico.HFLOWMIN ) cready[i][1] = 2;
                                if ( cedge[i][4] == 1 && aw[i][0] >= sico.HFLOWMIN ) cready[i][4] = 2;

                                if ( sico.MODEL == 2 && cedge2[i][2] == 1 && aw[i][3] >= sico.HFLOWMIN ) // degree of fill for fluid
                                    cready2[i][2] = 2;
                                if ( sico.MODEL == 2 && cedge2[i][5] == 1 && aw[i][3] >= sico.HFLOWMIN )
                                    cready2[i][5] = 2;
                                if ( sico.MODEL == 2 && cedge2[i][1] == 1 && aw[i][3] >= sico.HFLOWMIN )
                                    cready2[i][1] = 2;
                                if ( sico.MODEL == 2 && cedge2[i][4] == 1 && aw[i][3] >= sico.HFLOWMIN )
                                    cready2[i][4] = 2;
                            }
                            else { // for all other time steps:

                                if ( cedge[i][2] == 1 && aw[i][0] >= sico.HFLOWMIN )
                                    cready[i][2] += aw[i][1] / aw[i][0] * tlength / sico.CSZ; // degree of fill (for solid)
                                if ( cedge[i][5] == 1 && aw[i][0] >= sico.HFLOWMIN )
                                    cready[i][5] -= aw[i][1] / aw[i][0] * tlength / sico.CSZ;
                                if ( cedge[i][1] == 1 && aw[i][0] >= sico.HFLOWMIN )
                                    cready[i][1] += aw[i][2] / aw[i][0] * tlength / sico.CSZ;
                                if ( cedge[i][4] == 1 && aw[i][0] >= sico.HFLOWMIN )
                                    cready[i][4] -= aw[i][2] / aw[i][0] * tlength / sico.CSZ;

                                if ( sico.MODEL == 2 && cedge2[i][2] == 1 && aw[i][3] >= sico.HFLOWMIN ) // degree of fill for fluid
                                    cready2[i][2] += aw[i][4] / aw[i][3] * tlength / sico.CSZ;
                                if ( sico.MODEL == 2 && cedge2[i][5] == 1 && aw[i][3] >= sico.HFLOWMIN )
                                    cready2[i][5] -= aw[i][4] / aw[i][3] * tlength / sico.CSZ;
                                if ( sico.MODEL == 2 && cedge2[i][1] == 1 && aw[i][3] >= sico.HFLOWMIN )
                                    cready2[i][1] += aw[i][5] / aw[i][3] * tlength / sico.CSZ;
                                if ( sico.MODEL == 2 && cedge2[i][4] == 1 && aw[i][3] >= sico.HFLOWMIN )
                                    cready2[i][4] -= aw[i][5] / aw[i][3] * tlength / sico.CSZ;
                            }
                        }
                    }


                // Number of pixels which may provide inflow to neighbour pixels (cedge0, cedge02)

                    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels of reduced coordinate system:

                        if ( cdomain[i] == 5 ) {

                            in = fin ( i, px, py, pelev, sico );

                            for ( j=1; j<3; j++ ) {

                                if (( cedge [i][j] == 0 || cready [i][j] > 1 )
                                    && aw[i][0] >= sico.HFLOWMIN ) cedge0 [in[j]] += 1;
                                if ( sico.MODEL == 2 && ( cedge2[i][j] == 0 || cready2[i][j] > 1 )
                                    && aw[i][3] >= sico.HFLOWMIN ) cedge02[in[j]] += 1;
                            }
                            for ( j=4; j<6; j++ ) {

                                if (( cedge [i][j] == 0 || cready [i][j] > 1 )
                                    && aw[i][0] >= sico.HFLOWMIN ) cedge0 [in[j]] += 1;
                                if ( sico.MODEL == 2 && ( cedge2[i][j] == 0 || cready2[i][j] > 1 )
                                    && aw[i][3] >= sico.HFLOWMIN ) cedge02[in[j]] += 1;
                            }
                            if (( cedge[i][3] == 0 || cready[i][1] > 1 || cready[i][2] > 1 )
                                && aw[i][0] >= sico.HFLOWMIN ) cedge0 [in[3]] += 1;
                            if (( cedge[i][6] == 0 || cready[i][4] > 1 || cready[i][5] > 1 )
                                && aw[i][0] >= sico.HFLOWMIN ) cedge0 [in[6]] += 1;
                            if (( cedge[i][7] == 0 || cready[i][2] > 1 || cready[i][4] > 1 )
                                && aw[i][0] >= sico.HFLOWMIN ) cedge0 [in[7]] += 1;
                            if (( cedge[i][8] == 0 || cready[i][1] > 1 || cready[i][5] > 1 )
                                && aw[i][0] >= sico.HFLOWMIN ) cedge0 [in[8]] += 1;

                            if ( sico.MODEL == 2 && ( cedge2[i][3] == 0 || cready2[i][1] > 1
                                || cready2[i][2] > 1 ) && aw[i][3] >= sico.HFLOWMIN ) cedge02[in[3]] += 1;
                            if ( sico.MODEL == 2 && ( cedge2[i][6] == 0 || cready2[i][4] > 1
                                || cready2[i][5] > 1 ) && aw[i][3] >= sico.HFLOWMIN ) cedge02[in[6]] += 1;
                            if ( sico.MODEL == 2 && ( cedge2[i][7] == 0 || cready2[i][2] > 1
                                || cready2[i][4] > 1 ) && aw[i][3] >= sico.HFLOWMIN ) cedge02[in[7]] += 1;
                            if ( sico.MODEL == 2 && ( cedge2[i][8] == 0 || cready2[i][1] > 1
                                || cready2[i][5] > 1 ) && aw[i][3] >= sico.HFLOWMIN ) cedge02[in[8]] += 1;

                            free (in);
                        }
                    }


                // Number of neighbour pixels with non-zero flow depth (cneighbours, cneighbours2)

                    for ( i=0; i<sico.IMAX; i++ ) {

                        if ( cdomain[i] == 5 ) {

                            in = fin ( i, px, py, pelev, sico );

                            cneighbours[i] = 0; cneighbours2[i] = 0;
                            for ( j=1; j<9; j++ ) { if ( awt[in[j]][0] >= sico.HFLOWMIN ) cneighbours [i] += 1; }
                            for ( j=1; j<9; j++ ) { if ( sico.MODEL == 2 && awt[in[j]][3] >= sico.HFLOWMIN ) cneighbours2[i] += 1; }

                            free (in);
                        }
                    }


                // Reallocating flow depth and momentum from outside pixels to edge pixels

                    for ( i=0; i<sico.IMAX; i++ ) {

                        if ( cdomain[i] == 3 ) {

                            in = fin ( i, px, py, pelev, sico );

                            for ( j=1; j<9; j++ ) {

                                if ( cedge [i][j] == 1 && cedge0[in[j]] == 0 ) {

                                    for ( k=0; k<3; k++ ) {
                                        if ( cneighbours[in[j]] > 0 )
                                            awt[i][k] += awt[in[j]][k] / cneighbours[in[j]];
                                    }
                                }
                                if ( sico.MODEL == 2 && cedge2[i][j] == 1 && cedge02[in[j]] == 0 ) {

                                    for ( k=3; k<6; k++ ) {
                                        if ( cneighbours2[in[j]] > 0 )
                                            awt[i][k] += awt[in[j]][k] / cneighbours2[in[j]];
                                    }
                                }
                            }
                            free (in);
                        }
                    }


                // Setting flow depth and momentum at outside pixels to zero:

                    for ( i=0; i<sico.IMAX; i++ ) {

                        if ( cdomain[i] >= 3 ) {

                            in = fin ( i, px, py, pelev, sico ); // identifying pixel neighbourhood

                            for ( j=1; j<9; j++ ) {

                                if ( cedge [i][j] == 1 && cedge0 [in[j]] == 0 ) {
                                    for ( k=0; k<3; k++ ) awt[in[j]][k] = 0;
                                }
                                if ( sico.MODEL == 2 && cedge2[i][j] == 1 && cedge02[in[j]] == 0 ) {
                                    for ( k=3; k<6; k++ ) awt[in[j]][k] = 0;
                                }
                            }
                            free (in);
                        }
                    }
                }

// F2 -- STOP --- Diffusion control --------------------------------------------------------------------------------


// ----- START -- Evaluating time step length and validity of time step --------------------------------------------

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( cdomain[i] >= 3 ) {  // if fluxes are possible at pixel:

                        in = fin ( i, px, py, pelev, sico ); // pixel neighbourhood

                        if ( awt[in[2]][0] >= sico.HFLOWMIN ) vflowxp = awt[in[2]][1] / awt[in[2]][0]; else vflowxp = 0;
                        if ( awt[in[5]][0] >= sico.HFLOWMIN ) vflowxm = awt[in[5]][1] / awt[in[5]][0]; else vflowxm = 0;
                        if ( awt[in[1]][0] >= sico.HFLOWMIN ) vflowyp = awt[in[1]][2] / awt[in[1]][0]; else vflowyp = 0;
                        if ( awt[in[4]][0] >= sico.HFLOWMIN ) vflowym = awt[in[4]][2] / awt[in[4]][0]; else vflowym = 0;
                            // (solid) flow velocities of adjacent pixels

                        dvflowx = vflowxp - vflowxm; // gradient of (solid) flow velocity in x direction
                        dvflowy = vflowyp - vflowym; // gradient of (solid) flow velocity in y direction

                        if ( sico.MODEL == 1 ) { // one-phase model:

                            dvflow = dvflowx + dvflowy; // gradient of flow velocity
                            vcelr0 = fcvelr1( awt[i][0], awt[i][1]/awt[i][0], awt[i][2]/awt[i][0], dvflow, sflow, betax, betay, i, sico );
                                // velocity plus wave speed
                            if ( vcelr0 > vcelr ) vcelr = vcelr0; // updating maximum velocity plus wave speed
                        }
                        else if ( sico.MODEL == 2 ) { // two-phase model:

                            if ( awt[i][0] > sico.HFLOWMIN ) vflowx = awt[i][1] / awt[i][0]; else vflowx = 0;
                            if ( awt[i][0] > sico.HFLOWMIN ) vflowy = awt[i][2] / awt[i][0]; else vflowy = 0;
                            if ( awt[i][3] > sico.HFLOWMIN ) vflowx2 = awt[i][4] / awt[i][3]; else vflowx2 = 0;
                            if ( awt[i][3] > sico.HFLOWMIN ) vflowy2 = awt[i][5] / awt[i][3]; else vflowy2 = 0;
                                // fluid flow velocities of adjacent pixels

                            gk = fk2 ( dvflowx, dvflowy, sflow ); // earth pressure coefficients
                            vcelr0 = fcvelr2( awt[i][0], awt[i][3], vflowx, vflowy, vflowx2, vflowy2, sflow,
                                gk[0], gk[1], betax, betay, i, sico ); // velocity plus wave speed
                            if ( vcelr0 > vcelr ) vcelr = vcelr0; // updating maximum velocity plus wave speed

                            free( gk ); // releasing memory
                        }
                        free( in ); // releasing memory
                    }
                }

                cfl = vcelr * tlength / sico.CSZ; // updating CFL value

                if ( cfl <= sico.CFL[0] || vcelr == 0 ) { // if time step length meets CFL criterion:

                    ccfl = 1; // setting control for fulfilment of CFL criterion to positive
                    if ( cfl > cflmax ) cflmax = cfl; // updating maximum CFL value, if necessary

                    tsum += tlength; // updating time since release
                    tint += tlength; // updating time since last output

                    if ( tlength <= 0.00001 ) { // if time step length indicates numerical failure:
                        ccontinue = 0; // setting control for continuation to negative
                        csuccess = 0;  // setting control for simulation success to negative
                    }

                    if ( vcelr > 0 ) tlength = sico.CFL[0] * 0.90 * sico.CSZ / vcelr; else tlength = 0.01;
                    // updating length of time step according to CFL criterion
                    vcelr = 0; // resetting flow velocity plus wave speed
                }

                else { // if time step is too long too meet CFL criterion, reducing time step length:

                    tlength = 0.90 * tlength * sico.CFL[0] / cfl; // defining new time step length
                    vcelr = 0; // resetting flow velocity plus wave speed
                }

// ----- STOP --- Evaluating time step length and validity of time step --------------------------------------------


                #ifdef WITHGRASS


                    printf("   %i\t%i\t%.3f\t%.1f  \t%.1f\t...\r", nout, nsum, cflmax, 100*tlength, tsum ); // displaying progress
                    fflush(stdout); // forcing immediate display


                #endif


// *** End of loop over time step lengths --------------------------------------------------------------------------

            }

            for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                for ( k=0; k<nvect_min; k++ ) aw[i][k] = awt[i][k]; // applying values of temporary vectors
            }


// F5 -- START -- Entrainment --------------------------------------------------------------------------------------

            for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) {
                    if ( pcentr[i] != sico.UNDEF ) sbase.CENTR = pcentr[i]; // entrainment coefficient
                    else sbase.CENTR = sbase.CENTR0;
                }
                else sbase.CENTR = sbase.CENTR0;

                if ( sico.MODEL == 1 && sico.ENTRAINMENT > 0 && aw[i][0] >= sico.HFLOWMIN ) {
                    // if entrainment is enabled with one-phase model:

                    if ( sico.CORRHEIGHT == 0 ) corrfact = 1; // correction factor for flow height to depth conversion
                    else corrfact = cos( betaxy[i] );

                    potentr = ( phentrmax[i] - ( pelev0[i] - pelev[i] )) * corrfact;
                    if ( potentr < 0 ) potentr = 0; // potential entrainment rate

                    qentr = tlength * sbase.CENTR * pow( pow( aw[i][1], 2 ) + pow( aw[i][2], 2 ), 0.5 ); // entrainment rate

                    if ( qentr * 1.00 > potentr ) qentr = potentr;
                        // constraining solid entrainment rate by remaining entrainable solid depth

                    aw[i][0] = aw[i][0] + qentr * 1.00; // solid flow depth
                    aw[i][1] = aw[i][1] + qentr * 1.00 * aw[i][1] / aw[i][0]; // solid x momentum
                    aw[i][2] = aw[i][2] + qentr * 1.00 * aw[i][1] / aw[i][0]; // solid y momentum
                    aw[i][4] = aw[i][4] - qentr * 1.00; // change of basal topography
                }

                else if ( sico.MODEL == 2 && sico.ENTRAINMENT > 0
                    && ( aw[i][0] >= sico.HFLOWMIN || aw[i][3] >= sico.HFLOWMIN )) {
                    // if entrainment is enabled with two-phase model:

                    if ( sico.CORRHEIGHT == 0 ) corrfact = 1; // correction factor for flow height to depth conversion
                    else corrfact = cos( betaxy[i] );

                    if ( phentrmax[i] + phentrmax2[i] > 0 ) rpotentr = phentrmax[i] / ( phentrmax[i] + phentrmax2[i] );
                    else rpotentr = 0; // solid ratio

                    potentr = ( phentrmax[i] - rpotentr * ( pelev0[i] - pelev[i] )) * corrfact;
                        // potential solid entrainment rate
                    potentr2 = ( phentrmax2[i] - ( 1 - rpotentr ) * ( pelev0[i] - pelev[i] )) * corrfact;
                        // potential fluid entrainment rate
                    if ( potentr < 0 ) potentr = 0; if ( potentr2 < 0 ) potentr2 = 0;

                    qentr = tlength * sbase.CENTR * rpotentr * pow( pow( aw[i][1] * sflow.RHOS + aw[i][4] * sflow.RHOF, 2 )
                        + pow( aw[i][2] * sflow.RHOS + aw[i][5] * sflow.RHOF, 2 ), 0.5 );
                    qentr2 = tlength * sbase.CENTR * ( 1 - rpotentr ) * pow( pow( aw[i][1] * sflow.RHOS + aw[i][4] * sflow.RHOF, 2 )
                        + pow( aw[i][2] * sflow.RHOS + aw[i][5] * sflow.RHOF, 2 ), 0.5 );

                    vbulkx = ( aw[i][1] * sflow.RHOS + aw[i][4] * sflow.RHOF ) / ( aw[i][0] * sflow.RHOS + aw[i][3] * sflow.RHOF );
                    vbulky = ( aw[i][2] * sflow.RHOS + aw[i][5] * sflow.RHOF ) / ( aw[i][0] * sflow.RHOS + aw[i][3] * sflow.RHOF );

                    if ( qentr * 1.00 > potentr ) qentr = potentr;
                        // constraining solid entrainment rate by remaining entrainable solid depth
                    if ( qentr2 * 1.00 > potentr2 ) qentr2 = potentr2;
                        // constraining fluid entrainment rate by remaining entrainable fluid depth

                    aw[i][0] = aw[i][0] + qentr * 1.00; // solid flow depth
                    aw[i][1] = aw[i][1] + qentr * 1.00 * vbulkx; // solid x momentum
                    aw[i][2] = aw[i][2] + qentr * 1.00 * vbulky; // solid y momentum
                    aw[i][3] = aw[i][3] + qentr2 * 1.00; // fluid flow depth
                    aw[i][4] = aw[i][4] + qentr2 * 1.00 * vbulkx; // fluid x momentum
                    aw[i][5] = aw[i][5] + qentr2 * 1.00 * vbulky; // fluid y momentum
                    aw[i][6] = aw[i][6] - qentr * 1.00; // change of basal topography (solid)
                    aw[i][7] = aw[i][7] - qentr2 * 1.00; // change of basal topography (fluid)
                }
            }

// F5 -- STOP --- Entrainment --------------------------------------------------------------------------------------


// F6 -- START -- Stopping of flow ---------------------------------------------------------------------------------

            // *** Stopping criterion by energy balance
            // *** Factor of mobility fom: time required for stopping / time step length
            // *** 1 = Global stopping: fom is computed for bulk mass, material is deposited and simulation is terminated
            // ***     when stopping occurs
            // *** 2 = Local stopping with immediate deposition: fom is computed for 3 x 3 neighbourhood,
            // ***     stopped material is deposited immediately and simulation continues
            // *** 3 = Local stopping with delayed deposition: fom is computed for 3 x 3 neighbourhood, stopped material
            // ***     is frozen (momenta = zero) and deposited only at the end of the simulation

            if ( tsum > 10 && sico.STOPPING > 0 ) { // if function is enabled:

                if ( sico.STOPPING == 1 ) { // for global stopping:

                    mom = 0; momx = 0; momy = 0; mass = 0; betavg = 0; nalpha = 0;
                        // resetting variables for approach 1 (global stopping)

                    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                        if ( cdomain[i] > 1 && ( aw[i][0] >= sico.HFLOWMIN || ( sico.MODEL == 2 && aw[i][3] >= sico.HFLOWMIN ))) {
                            // if pixel is flow pixel:

                            if ( sico.MODEL == 1 ) { // for one-phase model:
                                momx += aw[i][1] * sflow.RHO; // updating global x momentum
                                momy += aw[i][2] * sflow.RHO; // updating global y momentum
                            }
                            else if ( sico.MODEL == 2 ) { // for two-phase model:
                                momx += aw[i][1] * sflow.RHOS + aw[i][4] * sflow.RHOF; // updating global x momentum
                                momy += aw[i][2] * sflow.RHOS + aw[i][5] * sflow.RHOF; // updating global y momentum
                            }
                            nalpha += 1; // number of pixels
                        }
                    }
                    if ( nalpha > 0 ) { momx = momx / nalpha; momy = momy / nalpha; } // average x and y momenta
                    alphavg = falphav( momx, momy, sico ); // global direction of movement

                    for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                        if ( cdomain[i] > 1 && ( aw[i][0] >= sico.HFLOWMIN || ( sico.MODEL == 2 && aw[i][3] >= sico.HFLOWMIN ))) {
                            // if pixel is flow pixel:

                            alpha = falpha( betax[i], betay[i], sico ); // aspect
                            betavg += tan( betaxy[i] * cos( alpha - alphavg )); // updating movement-following slope

                            if ( sico.MODEL == 1 ) { // for one-phase flow:

                                alphav = falphav( aw[i][1] * sflow.RHO, aw[i][2] * sflow.RHO, sico ); // direction of movement
                                mom += pow( pow (aw[i][1], 2 ) + pow( aw[i][2], 2 ), 0.5 ) * cos( alphav - alphavg ) * sflow.RHO;
                                    // updating topography-following momentum
                                mass += aw[i][0] * sflow.RHO; // updating mass
                            }

                            else if ( sico.MODEL == 2 ) { // for two-phase flow:

                                alphav = falphav( aw[i][1] * sflow.RHOS + aw[i][4] * sflow.RHOF,
                                    aw[i][2] * sflow.RHOS + aw[i][5] * sflow.RHOF, sico ); // direction of movement

                                mom += pow( pow (aw[i][1], 2 ) + pow( aw[i][2], 2 ), 0.5 ) * cos( alphav - alphavg ) * sflow.RHOS
                                    + pow( pow (aw[i][4], 2 ) + pow( aw[i][5], 2 ), 0.5 ) * cos( alphav - alphavg ) * sflow.RHOF;
                                    // updating topography-following momentum
                                mass += aw[i][0] * sflow.RHOS + aw[i][3] * sflow.RHOF; // updating mass
                            }

                            nalpha += 1; // number of pixels
                        }
                    }
                    if ( nalpha > 0 ) { mom = mom / nalpha; mass = mass / nalpha; betavg = atan( betavg / nalpha ); }
                        // averages
                    vflow = mom / mass; // topography-following bulk flow velocity
                }

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( cdomain[i] > 1 && ( aw[i][0] >= sico.HFLOWMIN || ( sico.MODEL == 2 && aw[i][3] >= sico.HFLOWMIN ))) {
                        // if pixel is flow pixel:

                        in = fin ( i, px, py, pelev, sico ); // neighbourhood
                        if ( sico.DELTA == 1 ) {
                            if ( pdelta[i] != sico.UNDEF ) sflow.DELTA = pdelta[i]; // basal friction angle
                            else sflow.DELTA = sflow.DELTA0;
                        }
                        else sflow.DELTA = sflow.DELTA0;
                        sflow.MY = tan( sflow.DELTA ); // basal friction coefficient

                        if ( sico.STOPPING == 1 ) imax = 1; else imax = 9;
                            // preparing loop over neighbourhood only for approaches 2 or 3 (local stopping)

                        if ( sico.STOPPING != 1 ) { // for local stopping:

                            mom = 0; momx = 0; momy = 0; mass = 0; betavg = 0; nalpha = 0;
                                // resetting variables for approach 1 (global stopping)

                            for ( j=0; j<imax; j++ ) { // loop over neighbourhood:

                                if ( cdomain[in[j]] > 1 && ( aw[in[j]][0] >= sico.HFLOWMIN
                                    || ( sico.MODEL == 2 && aw[in[j]][3] >= sico.HFLOWMIN ))) {
                                    // if pixel is flow pixel:

                                    if ( sico.MODEL == 1 ) { // for one-phase flow:

                                        momx += aw[in[j]][1] * sflow.RHO; // updating global x momentum
                                        momy += aw[in[j]][2] * sflow.RHO; // updating global y momentum
                                    }
                                    else if ( sico.MODEL == 2 ) { // for two-phase flow:

                                        momx += aw[in[j]][1] * sflow.RHOS + aw[in[j]][4] * sflow.RHOF; // updating global x momentum
                                        momy += aw[in[j]][2] * sflow.RHOS + aw[in[j]][5] * sflow.RHOF; // updating global y momentum
                                    }
                                    nalpha += 1; // number of pixels
                                }
                            }
                            if ( nalpha > 0 ) { momx = momx / nalpha; momy = momy / nalpha; } // average x and y momenta
                            alphavg = falphav( momx, momy, sico ); // global direction of movement

                            for ( j=0; j<imax; j++ ) { // loop over neighbourhood:

                                if ( cdomain[in[j]] > 1 && ( aw[in[j]][0] >= sico.HFLOWMIN
                                    || ( sico.MODEL == 2 && aw[in[j]][3] >= sico.HFLOWMIN ))) {
                                    // if pixel is flow pixel:

                                    alpha = falpha( betax[in[j]], betay[in[j]], sico ); // aspect
                                    betavg += tan( betaxy[in[j]] * cos( alpha - alphavg )); // updating movement-following slope

                                    if ( sico.MODEL == 1 ) { // for one-phase flow:

                                        alphav = falphav( aw[in[j]][1] * sflow.RHO, aw[in[j]][2] * sflow.RHO, sico );
                                            // direction of movement

                                        mom += pow( pow (aw[in[j]][1], 2 )
                                            + pow( aw[in[j]][2], 2 ), 0.5 ) * cos( alphav - alphavg ) * sflow.RHO;
                                            // updating topography-following momentum

                                        mass += aw[in[j]][0] * sflow.RHO; // updating mass
                                    }

                                    else if ( sico.MODEL == 2 ) { // for two-phase flow:

                                        alphav = falphav( aw[in[j]][1] * sflow.RHOS + aw[in[j]][4] * sflow.RHOF,
                                            aw[in[j]][2] * sflow.RHOS + aw[in[j]][5] * sflow.RHOF, sico ); // direction of movement

                                        mom += pow( pow (aw[in[j]][1], 2 )
                                            + pow( aw[in[j]][2], 2 ), 0.5 ) * cos( alphav - alphavg ) * sflow.RHOS
                                            + pow( pow (aw[in[j]][4], 2 ) + pow( aw[in[j]][5], 2 ), 0.5 )
                                            * cos( alphav - alphavg ) * sflow.RHOF;

                                            // updating topography-following momentum
                                        mass += aw[in[j]][0] * sflow.RHOS + aw[in[j]][3] * sflow.RHOF; // updating mass
                                    }

                                    nalpha += 1; // number of pixels
                                }
                            }
                            if ( nalpha > 0 ) { mom = mom / nalpha; mass = mass / nalpha; betavg = atan( betavg / nalpha ); }
                                // averages
                            vflow = mom / mass; // topography-following bulk flow velocity
                        }

                        if ( sico.STOPPING != 1 ) { // for approaches 2 or 3 (local stopping):

                            fom[i] = ( pow( 2, 0.5 ) - 1 ) * vflow
                                / ( sico.GRAVITY * cos( betavg ) * ( sflow.MY - tan( betavg )) * tlength ); // factor of mobility
                        }

                        free ( in ); // releasing memory
                    }
                }

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                    if ( cdomain[i] > 1 && ( aw[i][0] >= sico.HFLOWMIN || ( sico.MODEL == 2 && aw[i][3] >= sico.HFLOWMIN ))) {
                        // if pixel is flow pixel:

                        in = fin ( i, px, py, pelev, sico ); // neighbourhood
                        if ( sico.DELTA == 1 ) {
                            if ( pdelta[i] != sico.UNDEF ) sflow.DELTA = pdelta[i]; // basal friction angle
                            else sflow.DELTA = sflow.DELTA0;
                        }
                        else sflow.DELTA = sflow.DELTA0;
                        sflow.MY = tan( sflow.DELTA ); // basal friction coefficient

                        if ( sico.STOPPING == 1 ) { // for global stopping:

                            fom[i] = ( pow( 2, 0.5 ) - 1 ) * vflow
                                / ( sico.GRAVITY * cos( betavg ) * ( sflow.MY - tan( betavg )) * tlength ); // factor of mobility
                        }
                        if ( sico.STOPPING != 2 && fom[i] <= 1 ) { // if factor of mobility is 1 or lower for approach 1 or 3:

                            for ( j=0; j<imax; j++ ) {

                                cstopped[in[j]] = 1; // updating control for stopping
                                aw[in[j]][1] = 0; aw[in[j]][2] = 0; // setting (solid) momenta to zero
                                if ( sico.MODEL == 2 ) aw[in[j]][4] = 0; aw[in[j]][5] = 0; // setting fluid momenta to zero
                            }
                        }
                        else if ( sico.STOPPING == 2 && fom[i] <= 1 ) {
                            // if factor of mobility is 1 or lower for approach 2:

                            for ( j=0; j<imax; j++ ) {

                                if ( sico.MODEL == 1 ) { // for one-phase model:
                                    aw[in[j]][4] += aw[in[j]][0]; // updating deposited depth
                                    for ( k=0; k<4; k++ ) aw[i][k] = 0; // setting flow depths and momenta to zero
                                }
                                else if ( sico.MODEL == 2 ) { // for two-phase model:
                                    aw[in[j]][6] += aw[in[j]][0]; // updating deposited solid depth
                                    aw[in[j]][7] += aw[in[j]][3]; // updating deposited fluid depth
                                    for ( k=0; k<6; k++ ) aw[i][k] = 0; // setting flow depths and momenta to zero
                                }
                            }
                        }
                        free ( in ); // releasing memory
                    }
                }
            }

// F6 -- STOP --- Stopping of flow ---------------------------------------------------------------------------------


        // Updating vectors of composite, maximum and cumulative values, and basal topography

            for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                if ( sico.CORRHEIGHT == 0 ) corrfact = 1; // correction factor for depth to height conversion
                else corrfact = 1 / cos( betaxy[i] );

                if ( sico.MODEL == 1 ) { // for one-phase model:

                    pelev[i] = pelev[i] + aw[i][4] * corrfact; // correcting elevation for entrainment

                    if ( aw[i][0] >= sico.HFLOWMIN ) aw[i][5] = pow( pow( aw[i][1]/aw[i][0], 2 ) + pow( aw[i][2]/aw[i][0], 2 ) , 0.5 );
                        // updating flow velocity
                    else aw[i][5] = 0;
                    aw[i][6] = 0.5 * sflow.RHO * aw[i][0] * pow( aw[i][5], 2 ); // updating flow kinetic energy
                    aw[i][7] = ( 0.5 * sflow.RHO * pow( aw[i][5], 2 ) + sico.GRAVITY * sflow.RHO * aw[i][0] );
                        // updating flow pressure

                    if ( aw[i][0] > aw[i][8] ) aw[i][8] = aw[i][0]; //updating maximum flow depth at pixel
                    if ( aw[i][5] > aw[i][9] && aw[i][0] >= sico.IMPTHR[0] ) aw[i][9] = aw[i][5];
                        // updating maximum flow velocity at pixel
                    if ( aw[i][0] >= sico.HFLOWMIN && aw[i][3] / aw[i][0] > aw[i][10] ) aw[i][10] = aw[i][3] / aw[i][0];
                        // updating maximum random kinetic energy at pixel
                    if ( aw[i][6] > aw[i][11] ) aw[i][11] = aw[i][6]; //updating maximum flow kinetic energy at pixel
                    if ( aw[i][7] > aw[i][12] ) aw[i][12] = aw[i][7]; // updating maximum flow pressure at pixel
                }
                else if ( sico.MODEL == 2 ) { // for two-phase model:

                    pelev[i] = pelev0[i] + ( aw[i][6] + aw[i][7] ) * corrfact; // correcting elevation for entrainment or deposition

                    if ( aw[i][0] >= sico.HFLOWMIN ) aw[i][8] = pow( pow( aw[i][1]/aw[i][0], 2 ) + pow( aw[i][2]/aw[i][0], 2 ) , 0.5 );
                    else aw[i][8] = 0;
                    if ( aw[i][3] >= sico.HFLOWMIN ) aw[i][9] = pow( pow( aw[i][4]/aw[i][3], 2 ) + pow( aw[i][5]/aw[i][3], 2 ) , 0.5 );
                    else aw[i][9] = 0; // updating flow velocities
                    aw[i][10] = aw[i][0] + aw[i][3]; // updating total flow depth
                    aw[i][11] = 0.5 * sflow.RHOS * aw[i][0] * pow( aw[i][8], 2 );
                        // updating solid flow kinetic energy
                    aw[i][12] = 0.5 * sflow.RHOF * aw[i][3] * pow( aw[i][9], 2 );
                        // updating fluid flow kinetic energy
                    aw[i][13] = aw[i][11] + aw[i][12];
                        // updating total flow kinetic energy
                    if ( aw[i][0] >= sico.HFLOWMIN ) aw[i][14] = aw[i][11] / aw[i][0] + sico.GRAVITY * sflow.RHOS * aw[i][0];
                        // updating solid flow pressure
                    else aw[i][14] = 0;
                    if ( aw[i][3] >= sico.HFLOWMIN ) aw[i][15] = aw[i][12] / aw[i][3] + sico.GRAVITY * sflow.RHOF * aw[i][3];
                        // updating fluid flow pressure
                    else aw[i][15] = 0;
                    if ( aw[i][10] >= sico.HFLOWMIN ) aw[i][16] = aw[i][13] / aw[i][10]
                        + ( sico.GRAVITY * sflow.RHOS * aw[i][0] + sico.GRAVITY * sflow.RHOF * aw[i][3] ); // updating total flow pressure
                    else aw[i][16] = 0;
                    aw[i][17] = aw[i][6] + aw[i][7]; // updating total entrained or deposited depth

                    if ( aw[i][0] > aw[i][18] ) aw[i][18] = aw[i][0]; //updating maximum solid flow depth at pixel
                    if ( aw[i][8] > aw[i][19] && aw[i][0] >= sico.IMPTHR[0] ) aw[i][19] = aw[i][8];
                        // updating maximum solid flow velocity at pixel
                    if ( aw[i][3] > aw[i][20] ) aw[i][20] = aw[i][3]; //updating maximum fluid flow depth at pixel
                    if ( aw[i][9] > aw[i][21] && aw[i][3] >= sico.IMPTHR[0] ) aw[i][21] = aw[i][9];
                        // updating maximum fluid flow velocity at pixel
                    if ( aw[i][10] > aw[i][22] ) aw[i][22] = aw[i][10]; // updating maximum total flow depth at pixel
                    if ( aw[i][11] > aw[i][23] ) aw[i][23] = aw[i][11]; // updating maximum solid flow kinetic energy at pixel
                    if ( aw[i][14] > aw[i][24] ) aw[i][24] = aw[i][14]; // updating maximum solid flow pressure at pixel
                    if ( aw[i][12] > aw[i][25] ) aw[i][25] = aw[i][12]; // updating maximum fluid flow kinetic energy at pixel
                    if ( aw[i][15] > aw[i][26] ) aw[i][26] = aw[i][15]; // updating maximum fluid flow pressure at pixel
                    if ( aw[i][13] > aw[i][27] ) aw[i][27] = aw[i][13]; // updating maximum total flow kinetic energy at pixel
                    if ( aw[i][16] > aw[i][28] ) aw[i][28] = aw[i][16]; // updating maximum total flow pressure at pixel
                }
            }


// *** End of loop over two steps, each moving the system half of a pixel (NOC scheme) -----------------------------

        }


// ----- START -- Updating maximum values of flow parameters and volumes -------------------------------------------

        hflow_max = 0; // resetting maximum (solid) flow depth
        vflow_max = 0; // resetting maximum (solid) flow velocity
        vol_flow = 0; // resetting (solid) flow volume
        vol_edge = 0; // resetting (solid) lost flow volume
        vol_entr = 0; // resetting (solid) entrained volume
        vol_noflux = 0; // resetting (solid) volume without fluxes
        ekin_flow = 0; // resetting total kinetic energy of flow

        vol_npixels = 0; vol_npixels2 = 0; // resetting number of pixels for volume conservation

        if ( sico.MODEL == 1 ) rflow_max = 0; // for one-phase model, resetting maximum random kinetic energy
        else if ( sico.MODEL == 2 ) { // for two-phase model:

            hflow_max2 = 0; // resetting maximum fluid flow depth
            vflow_max2 = 0; // resetting maximum fluid velocity
            vol_flow2 = 0; // resetting fluid flow volume
            vol_edge2 = 0; // resetting fluid lost flow volume
            vol_entr2 = 0; // resetting fluid entrained volume
            vol_noflux2 = 0; // resetting fluid volume without fluxes
        }

        for ( i=0; i<sico.IMAX; i++ ) { // updating maxima, volumes and kinetic energy:

            if ( cdomain[i] == 0 ) { for ( k=0; k<3; k++ ) aw[i][k] = 0; }
            if ( sico.MODEL == 2 && cdomain[i] == 0 ) { for ( k=3; k<6; k++ ) aw[i][k] = 0; }
                // setting state variables for edge pixels to zero

            if ( sico.CORRHEIGHT == 0 ) carea = pow ( sico.CSZ, 2 );
            else carea = pow ( sico.CSZ, 2 ) * pow( 1 - pow( sin( betax[i] ) , 2 ) * pow ( sin( betay[i] ) , 2 ) , 0.5 )
                / ( cos( betax[i] ) * cos( betay[i] ) ); // topography-following area of pixel

            if ( aw[i][0] > hflow_max ) hflow_max = aw[i][0]; // maximum (solid) flow depth

            if ( cstopped[i] == 0 && cflux[i] == 1 && aw[i][0] >= sico.HFLOWMIN ) vol_flow += aw[i][0] * carea; // (solid) flow volume
            if ( cdomain2[i] == 1 && aw[i][0] >= sico.HFLOWMIN ) vol_edge += aw[i][0] * carea;
                // (solid) flow volume leaving area of interest

            if (( cflux[i] == 0 || cstopped[i] == 1 ) && aw[i][0] >= sico.HFLOWMIN )
                vol_noflux += aw[i][0] * carea; // (solid) volume without fluxes

            if ( sico.MODEL == 1 ) { // for one-phase model:

                if ( hflow_max > hflow_maxmax ) hflow_maxmax = hflow_max; // absolute maximum flow depth

                if ( aw[i][0] >= sico.IMPTHR[0] && aw[i][5] > vflow_max ) vflow_max = aw[i][5]; // maximum flow velocity
                if ( aw[i][0] >= sico.IMPTHR[0] && aw[i][3] / aw[i][0] > rflow_max ) rflow_max = aw[i][3] / aw[i][0];
                    // maximum random kinetic energy

                if ( cdomain[i] != 0 && ( sico.ENTRAINMENT > 0 || sico.STOPPING > 0 )) vol_entr += aw[i][4] * carea;
                    // entrained or deposited volume

                ekin_flow += aw[i][6] * carea; // kinetic energy of flow
            }

            else if ( sico.MODEL == 2 ) { // for two-phase model:

                if ( aw[i][3] > hflow_max2 ) hflow_max2 = aw[i][3];
                if ( hflow_max + hflow_max2 > hflow_maxmax ) hflow_maxmax = hflow_max + hflow_max2; // absolute maximum flow depth

                if ( cdomain[i] != 0 && aw[i][0] >= sico.IMPTHR[0] && aw[i][8] > vflow_max ) vflow_max = aw[i][8];
                if ( cdomain[i] != 0 && aw[i][3] >= sico.IMPTHR[0] && aw[i][9] > vflow_max2 ) vflow_max2 = aw[i][9];
                    // maximum solid and fluid velocities

                if ( cstopped[i] == 0 && cflux[i] == 1 && aw[i][3] >= sico.HFLOWMIN ) vol_flow2 += aw[i][3] * carea;
                    // fluid flow volume
                if ( cdomain2[i] == 1 && aw[i][3] >= sico.HFLOWMIN ) vol_edge2 += aw[i][3] * carea;
                    // fluid  flow volume leaving area of interest

                if (( cflux[i] == 0 || cstopped[i] == 1 ) && aw[i][3] >= sico.HFLOWMIN )
                    vol_noflux2 += aw[i][3] * carea; // fluid volume without fluxes

                if ( cdomain[i] != 0 && ( sico.ENTRAINMENT > 0 || sico.STOPPING > 0 )) vol_entr += aw[i][6] * carea;
                if ( cdomain[i] != 0 && ( sico.ENTRAINMENT > 0 || sico.STOPPING > 0 )) vol_entr2 += aw[i][7] * carea;
                    // entrained or deposited volumes

                ekin_flow += aw[i][13] * carea; // total flow kinetic energy
            }

            if ( aw[i][0] >= sico.IMPTHR[0] && cstopped[i] == 0 && cflux[i] == 1 ) vol_npixels += 1;
            if ( sico.MODEL == 2 && aw[i][3] >= sico.IMPTHR[0] && cstopped[i] == 0 && cflux[i] == 1 ) vol_npixels2 += 1;
                // number of pixels for volume conservation

        }

// ----- STOP --- Updating maximum values of flow parameters and volumes -------------------------------------------


    // Indicating stopping of the flow (the entire flow has stopped moving or left the area of interest, termination of the simulation)

        if (( sico.MODEL == 1 && vol_flow == 0 ) || ( sico.MODEL == 2 && vol_flow == 0 && vol_flow2 == 0 )) {

            printf("\n                       \n                       ------------------ FLOW STOPPED ------------------\n\n");
            fflush(stdout); // forcing immediate display
            fprintf(f_summary, "\n                       \n                       ------------------ FLOW STOPPED ------------------\n\n");
            ccontinue = 0; // setting control for continuation to negative
        }


    // Indicating numerical failure (termination of the simulation)

        if ( ccontinue == 0 && csuccess == 0 ) { // stopping simulation if time step length indicates numerical failure:

            printf("\n                       \n                       --------------- NUMERICAL FAILURE ----------------\n\n");
            fflush(stdout); // forcing immediate display
            fprintf(f_summary, "\n                       \n                       --------------- NUMERICAL FAILURE ----------------\n\n");
        }


    // Indicating overflow (part of the flow leaving the area of interest, simulation continues)

        if (( vol_edge > 0 || ( sico.MODEL == 2 && vol_edge2 > 0 )) && coverflow == 0 ) {
            printf("\n                       \n                       -------------------- OVERFLOW --------------------\n\n");
            fflush(stdout); // forcing immediate display
            fprintf(f_summary, "\n                       \n                       -------------------- OVERFLOW --------------------\n\n");
            coverflow = 1; // setting control for overflow to positive
        }


// F3 -- START -- Correction of flow depth and momentum for conservation of volume ---------------------------------

        //***** Conservation of volume is only applied as long as no overflow has occurred

        if ( sico.CONSVOL == 1 && coverflow == 0 ) { // if correction of volume is enabled:

            vol_diff = ( vol_flow - vol_flow_bef ) + ( vol_noflux - vol_noflux_bef ) + (vol_entr - vol_entr_bef );
            vol_diff2 = ( vol_flow2 - vol_flow2_bef ) + ( vol_noflux2 - vol_noflux2_bef ) + (vol_entr2 - vol_entr2_bef );
                // difference in flow volumes, compared to previous time step

            vol_flowold = vol_flow; vol_flow2old = vol_flow2; // storing flow volumes in new variables
            vol_flow = 0; vol_flow2 = 0; // resetting solid and fluid flow volumes

            for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                if ( sico.CORRHEIGHT == 0 ) carea = pow ( sico.CSZ, 2 );
                else carea = pow ( sico.CSZ, 2 ) * pow( 1 - pow( sin( betax[i] ) , 2 ) * pow ( sin( betay[i] ) , 2 ) , 0.5 )
                    / ( cos( betax[i] ) * cos( betay[i] ) ); // topography-following area of pixel

                if ( aw[i][0] >= sico.IMPTHR[0] && cstopped[i] == 0 && cflux[i] == 1 ) {

                    for ( k=0; k<3; k++ ) aw[i][k] -= vol_diff / ( vol_npixels * carea );
                        // correcting (solid components of) state variables
                }
                if ( aw[i][0] >= sico.HFLOWMIN && cstopped[i] == 0 && cflux[i] == 1 )vol_flow += aw[i][0] * carea;
                    // updating solid flow volume
                if ( sico.MODEL == 2 && aw[i][3] >= sico.IMPTHR[0] && cstopped[i] == 0 && cflux[i] == 1 ) {
                    for ( k=3; k<6; k++ ) aw[i][k] -= vol_diff2 / ( vol_npixels2 * carea );
                        // correcting fluid components of state variables
                }
                if ( sico.MODEL == 2 && aw[i][3] >= sico.HFLOWMIN && cstopped[i] == 0 && cflux[i] == 1 ) vol_flow2 += aw[i][3] * carea;
                    // updating fluid flow volume
            }

            vol_flow_bef = vol_flow; // (solid) flow volume at start of next time step
            vol_entr_bef = vol_entr; // (solid) entrained or deposited volume at start of next time step
            vol_noflux_bef = vol_noflux; // (solid) volume without fluxes at start of next time step
            if ( sico.MODEL == 2 ) { // for two-phase model:
                vol_flow2_bef = vol_flow2; // fluid flow volume at start of next time step
                vol_entr2_bef = vol_entr2; // fluid entrained or deposited volume at start of next time step
                vol_noflux2_bef = vol_noflux2; // fluid volume without fluxes at start of next time step
            }
        }

// F3 -- STOP -- Correction of flow depth and momentum for conservation of volume ----------------------------------


    // Correcting deposited depth and basal topography for stopping, if necessary

        if (( sico.STOPPING == 1 || sico.STOPPING == 3 ) && (( tint >= tout && (( tsum + tlength ) > tmax )) || ccontinue == 0 )) {
            // if time step is last one and stopping approaches 1 or 3 are applied:

            for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                if ( sico.CORRHEIGHT == 0 ) corrfact = 1; // correction factor for depth to height conversion
                else corrfact = 1 / cos( betaxy[i] );

                if ( sico.MODEL == 1 ) { // for one-phase model:

                    aw[i][4] += cstopped[i] * aw[i][0]; // updating entrained or deposited depth
                    pelev[i] += cstopped[i] * aw[i][0] * corrfact; // correcting elevation for stopping
                    aw[i][0] -= cstopped[i] * aw[i][0]; // updating flow depth
                }
                else if ( sico.MODEL == 2 ) { // for two-phase model:

                    aw[i][6] += cstopped[i] * aw[i][0]; // updating entrained or deposited solid depth
                    aw[i][7] += cstopped[i] * aw[i][3]; // updating entrained or deposited fluid depth
                    aw[i][17] = aw[i][6] + aw[i][7]; // updating total entrained or deposited depth
                    pelev[i] += ( cstopped[i] * aw[i][0] + cstopped[i] * aw[i][3] ) * corrfact;
                        // correcting elevation for stopping
                    aw[i][0] -= cstopped[i] * aw[i][0]; // updating solid flow depth
                    aw[i][3] -= cstopped[i] * aw[i][3]; // updating fluid flow depth
                }
            }
        }

        if ( tint >= tout || ccontinue == 0 ) { // if defined interval or last time step is reached:


// ----- START -- Writing hydrograph infos to files ----------------------------------------------------------------

            if ( tint >= tout && sico.MULT == 0 && hydrograph == 1 ) {

                ctrl_hydout = 1; // control for hydrograph output

                for ( hydj = hydnin; hydj < hydnin + hydnout; hydj++ ) {  // loop over all output hydrographs:

                    hydh = aw[hydi[hydj]][0]  / cos( betaxy[hydi[hydj]] ); // (solid) flow height
                    if ( hydh >= sico.HFLOWMIN )
                        hydv = pow( pow( aw[hydi[hydj]][1], 2 ) + pow( aw[hydi[hydj]][2], 2 ), 0.5 ) / aw[hydi[hydj]][0];
                    else hydv= 0; // (solid) flow velocity

                    if ( sico.MODEL == 1 ) {

                        if ( hydh >= sico.HFLOWMIN ) hydr = aw[hydi[hydj]][3] / aw[hydi[hydj]][0];
                        else hydr= 0; // flow random kinetic energy
                        hyde = aw[hydi[hydj]][4]; // entrained height
                        hydh2 = 0; hydv2 = 0; hyde2 = 0;
                    }
                    else if ( sico.MODEL == 2 ) {

                        hydh2 = aw[hydi[hydj]][3] / cos( betaxy[hydi[hydj]] ); // fluid flow height
                        if ( hydh2 >= sico.HFLOWMIN )
                            hydv2 = pow( pow( aw[hydi[hydj]][4], 2 ) + pow( aw[hydi[hydj]][5], 2 ), 0.5 ) / aw[hydi[hydj]][3];
                        else hydv2= 0; // fluid flow velocity
                        hyde = aw[hydi[hydj]][6]; // entrained solid height
                        hyde2 = aw[hydi[hydj]][6]; // entrained fluid height
                        hydr = 0;
                    }

                    hydq = 0; // resetting (solid) discharge
                    if ( sico.MODEL == 2 ) hydq2 = 0; // resetting fluid discharge

                    if ( hydalpha[hydj] < sico.PI / 2 ) { hydfx = 1; hydfy = 1; } // negative/positive signs depending on direction:
                    else if ( hydalpha[hydj] < sico.PI ) { hydfx = -1; hydfy = 1; }
                    else if ( hydalpha[hydj] < 3 * sico.PI / 2 ) { hydfx = -1; hydfy = -1; }
                    else { hydfx = 1; hydfy = -1; }

                    if ( hydalpha[hydj] == 0 || hydalpha[hydj] == sico.PI / 2 || hydalpha[hydj] == sico.PI
                        || hydalpha[hydj] == 3 * sico.PI / 2 ) hydfalpha = 1;
                    else if ( fabs( 1 / sin( hydalpha[hydj] )) < fabs( 1 / cos( hydalpha[hydj] )))
                        hydfalpha = fabs( 1 / sin( hydalpha[hydj] ));
                    else hydfalpha = fabs( 1 / cos( hydalpha[hydj] )); // correction factor for profile direction

                    for ( hydk = 1; hydk <= hydp[0][hydj]; hydk++ ) { // loop over all pixels of profile:

                        alpha = falpha( betax[hydp[hydk][hydj]], betay[hydp[hydk][hydj]], sico ); // aspect
                        hydbeta = atan ( tan( betaxy[hydp[hydk][hydj]] ) * cos ( alpha - hydalpha[hydj] + sico.PI / 2 )); // corrected slope

                        hydfcorr = sico.CSZ * hydfalpha / cos( hydbeta ); // reference length for discharge

                        hydm0 = pow( pow(aw[hydp[hydk][hydj]][1], 2 ) + pow( aw[hydp[hydk][hydj]][2], 2), 0.5 );

                        if ( aw[hydp[hydk][hydj]][0] >= sico.HFLOWMIN && hydm0 > 0 ) {

                            hydmx = aw[hydp[hydk][hydj]][1];
                            hydmy = aw[hydp[hydk][hydj]][2];

                            if ( hydmy > 0 ) hydmalpha = acos( hydmx / hydm0 ); else hydmalpha = 2 * sico.PI - acos( hydmx / hydm0 );
                            hydnalpha = hydalpha[hydj] - sico.PI / 2; if ( hydnalpha < 0 ) hydnalpha += 2 * sico.PI;
                            hydm = hydm0 * cos( hydnalpha - hydmalpha ); // momentum in direction perpendicular to profile
                            hydq += hydm * hydfcorr; // updating (solid) discharge
                        }

                        if ( sico.MODEL == 2 ) {

                            hydm0 = pow( pow(aw[hydp[hydk][hydj]][4], 2 ) + pow( aw[hydp[hydk][hydj]][5], 2), 0.5 );

                            if ( aw[hydp[hydk][hydj]][3] >= sico.HFLOWMIN && hydm0 > 0 ) {

                                hydmx = aw[hydp[hydk][hydj]][4];
                                hydmy = aw[hydp[hydk][hydj]][5];

                                if ( hydmy > 0 ) hydmalpha = acos( hydmx / hydm0 ); else hydmalpha = 2 * sico.PI - acos( hydmx / hydm0 );
                                hydnalpha = hydalpha[hydj] - sico.PI / 2; if ( hydnalpha < 0 ) hydnalpha += 2 * sico.PI;
                                hydm = hydm0 * cos ( hydnalpha - hydmalpha ); // momentum in direction perpendicular to profile
                                hydq2 += hydm * hydfcorr; // updating fluid discharge
                            }
                        }
                        else hydq2 = 0;
                    }

                    fprintf(f_hydinfo[hydj], "%.1f\t%.2f\t%.2f\t%.2f\t%.2f\t%.0f\t%.2f\t%.2f\t%.2f\t%.0f\n",
                        tsum + 2 * tlength, hydh, hydv, hydr, hyde, hydq, hydh2, hydv2, hyde2, hydq2); // writing hydrograph info to file
                }
            }

// ----- STOP --- Writing hydrograph infos to files ----------------------------------------------------------------


// ----- START -- Writing output files and raster maps -------------------------------------------------------------


            #ifdef WITHGRASS


            // Writing output (if GRASS is used)

                for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:


                // Converting depths into heights

                    if ( cdomain[i] != 0 ) {

                        if ( sico.MODEL == 1 ) {

                            hflowi = fconvout( i, aw, hflow_max, 1, 1,
                                px, py, pelev, betax, betay, betaxy, sico );
                            hentri = fconvout( i, aw, hflow_max, 1, 2,
                                px, py, pelev, betax, betay, betaxy, sico );
                        }
                        else if ( sico.MODEL == 2 ) {

                            hflowi = fconvout( i, aw, hflow_max+hflow_max2, 1, 1,
                                px, py, pelev, betax, betay, betaxy, sico );
                            hflowi2 = fconvout( i, aw, hflow_max+hflow_max2, 2, 1,
                                px, py, pelev, betax, betay, betaxy, sico );

                            hentri = fconvout( i, aw, hflow_max+hflow_max2, 1, 2,
                                px, py, pelev, betax, betay, betaxy, sico );
                            hentri2 = fconvout( i, aw, hflow_max+hflow_max2, 2, 2,
                                px, py, pelev, betax, betay, betaxy, sico );
                        }
                    }
                    else {
                        hflowi = 0; hentri = 0;
                        if ( sico.MODEL == 2 ) {
                            hflowi2 = 0; hentri2 = 0;
                        }
                    }


                // Writing output parameters to segment files

                    v[0] = hflowi; // (solid) flow height
                    if ( aw[i][0] >= sico.HFLOWMIN ) v[1] = aw[i][1] / aw[i][0]; else v[1] = 0; // (solid) velocity in x direction
                    if ( aw[i][0] >= sico.HFLOWMIN ) v[2] = aw[i][2] / aw[i][0]; else v[2] = 0; // (solid) velocity in y direction

                    if ( sico.MODEL == 1 ) { // for one-phase model:

                        if ( aw[i][0] >= sico.HFLOWMIN ) v[3] = aw[i][3] / aw[i][0]; else v[3] = 0; // random kinetic energy
                        v[4] = hentri; // entrained height
                        v[5] = aw[i][5]; // velocity
                        v[6] = aw[i][6]; // flow kinetic energy
                        v[7] = aw[i][7]; // flow pressure
                    }
                    else if ( sico.MODEL == 2 ) { // for two-phase model:

                        v[3] = hflowi2; // fluid flow height
                        if ( aw[i][3] >= sico.HFLOWMIN ) v[4] = aw[i][4] / aw[i][3]; else v[4] = 0; // fluid velocity in x direction
                        if ( aw[i][3] >= sico.HFLOWMIN ) v[5] = aw[i][5] / aw[i][3]; else v[5] = 0; // fluid velocity in y direction
                        v[6] = hentri; // entrained or deposited solid height
                        v[7] = hentri2; // entrained or deposited fluid height
                        v[8] = aw[i][8]; // solid velocity
                        v[9] = aw[i][9]; // fluid velocity
                        v[10] = hflowi + hflowi2; // total flow height
                        v[11] = aw[i][11]; // solid flow kinetic energy
                        v[12] = aw[i][12]; // fluid flow kinetic energy
                        v[13] = aw[i][13]; // total flow kinetic energy
                        v[14] = aw[i][14]; // solid flow pressure
                        v[15] = aw[i][15]; // fluid flow pressure
                        v[16] = aw[i][16]; // total flow pressure
                        v[17] = hentri + hentri2; // total entrained or deposited height
                    }

                    for ( k=0; k<nvect_red; k++ ) Segment_put(&seg_v[k], &v[k], px[i], py[i]); // writing values to segment files
                }
                for ( k=0; k<nvect_red; k++ ) Segment_flush(&seg_v[k]); // forcing to immediately write values to segment file


            #endif


        // Summary of state of the flow at output time step

            if ( sico.MODEL == 1 ) { // for one-phase model:

                printf("   %i\t%i\t%.3f\t%.1f\t%.1f\t%.*f\t%.2f\t%.2f\t%.*f\t%.*f\n",
                    nout, nsum, cflmax, 100*tlength, tsum, prec_hflow, hflow_max, vflow_max, rflow_max,
                    prec_vol, vol_flow/1000, prec_ekin, ekin_flow/1000000); // displaying parameters
                fflush(stdout); // forcing immediate display
                fprintf(f_summary, "%i\t%i\t%.3f\t%.1f\t%.1f\t%.*f\t%.2f\t%.2f\t%.*f\t%.*f\n",
                    nout, nsum, cflmax, 100*tlength, tsum, prec_hflow, hflow_max, vflow_max, rflow_max,
                    prec_vol, vol_flow, prec_ekin, ekin_flow); // writing parameters to summary file
            }
            else if ( sico.MODEL == 2 ) { // for two-phase model:

                printf("   %i\t%i\t%.3f\t%.1f\t%.1f\t%.*f\t%.2f\t%.*f\t%.2f\t%.*f\t%.*f\t%.*f\n",
                    nout, nsum, cflmax, 100*tlength, tsum, prec_hflow, hflow_max, vflow_max, prec_hflow, hflow_max2, vflow_max2,
                    prec_vol, vol_flow/1000, prec_vol, vol_flow2/1000, prec_ekin, ekin_flow/1000000); // displaying parameters
                fflush(stdout); // forcing immediate display
                fprintf(f_summary, "%i\t%i\t%.3f\t%.1f\t%.1f\t%.*f\t%.2f\t%.*f\t%.2f\t%.*f\t%.*f\t%.*f\n",
                    nout, nsum, cflmax, 100*tlength, tsum, prec_hflow, hflow_max, vflow_max, prec_hflow, hflow_max2, vflow_max2,
                    prec_vol, vol_flow, prec_vol, vol_flow2, prec_ekin, ekin_flow); // writing parameters to summary file
            }

            sprintf(path, "%s%snout%d.txt", outfiles, prefix, xint); // writing number of time steps and control for success to file:
            f_nout=fopen(path, "w");
            fprintf(f_nout, "%i\n%i\n%.0f\n%0f\n", nout, csuccess, vol_entr, vol_entr2 );
            fclose(f_nout);


            if ( sico.MULT == 0 ) { // for single model run:


            // Writing output raster maps

                if ( nout < 10 ) sprintf( madd, "000"); // fill string (for maintaining correct order in list of maps)
                else if ( nout < 100 ) sprintf( madd, "00");
                else if ( nout < 1000 ) sprintf( madd, "0");
                else sprintf( madd, "");

                for ( k=0; k<nvect_red; k++ ) { // preparing, writing and closing output raster maps for time steps
                    sprintf( mv, "%s%s%s%i", prefix, mv0[k], madd, nout );


                    #ifdef WITHGRASS


                        foutrast ( mv, seg_v[k], sico ); // if GRASS is used, writing GRASS raster maps


                    #endif


                    if ( sico.MODEL == 1 ) foutasc ( aw, px, py, outmaps, mv, hflow_max, pelev, betax, betay, betaxy,
                        k, sico ); // writing ascii raster maps (one-phase model)
                    else if ( sico.MODEL == 2 ) foutasc ( aw, px, py, outmaps, mv, hflow_max+hflow_max2, pelev, betax, betay, betaxy,
                        k, sico ); // writing ascii raster maps (two-phase model)
                }


            // Writing file for display of flow vectors as arrows

                if ( nout == 1 && sico.MODEL == 1 ) foutdircoord ( f_directions, 1, px, py, sico );
                else if ( nout == 1 && sico.MODEL == 2 ) {
                    foutdircoord ( f_directions, 2, px, py, sico );
                    foutdircoord ( f_directions2, 3, px, py, sico );
                } // writing coordinates to file

                if ( sico.MODEL == 1 ) for ( k=0; k<nvect_min; k++ ) foutdir ( f_directions, aw, k, 1, px, py, sico );
                else if ( sico.MODEL == 2 ) {
                    for ( k=0; k<3; k++ ) foutdir ( f_directions, aw, k, 2, px, py, sico );
                    for ( k=3; k<6; k++ ) foutdir ( f_directions2, aw, k, 3, px, py, sico );
                    // writing parameters to file
                }
            }


        // Writing output raster maps for last time step

            if ( ( tsum + tlength ) > tmax || ccontinue == 0 ) { // if time step is last one:

                for ( k=0; k<nvect_red; k++ ) { // preparing, writing and closing output raster maps for last time step:
                    if ( ( sico.MODEL == 1 && ( k==0 || k == 4 ))
                        || ( sico.MODEL == 2 && ( k==0 || k == 3 || k==6 || k == 7 || k == 10 || k == 17 ))) {
                        if ( sico.MULT == 0 ) sprintf( mv, "%s%s_fin", prefix, mv0[k] );
                        else sprintf( mv, "%s%s_fin%d", prefix, mv0[k], xint ); // names of output raster maps


                        #ifdef WITHGRASS


                            foutrast ( mv, seg_v[k], sico ); // if GRASS is used, writing GRASS raster maps


                        #endif


                        if ( sico.MODEL == 1 ) foutasc ( aw, px, py, outmaps, mv, hflow_max, pelev, betax, betay, betaxy,
                            k, sico ); // writing Ascii raster maps (one-phase model)
                        else if ( sico.MODEL == 2 ) foutasc ( aw, px, py, outmaps, mv, hflow_max+hflow_max2, pelev, betax, betay, betaxy,
                            k, sico ); // writing Ascii raster maps (two-phase model)
                    }
                }
            }

// ----- STOP --- Writing output files and raster maps -------------------------------------------------------------


            cflmax = 0; // resetting maximum cfl value
            nout += 1; // updating number of output time steps
        }

        nsum += 1; // updating total number of time steps


// *** End of loop over time steps ---------------------------------------------------------------------------------

    }


// Writing output rasters of cumulative and maximum values


    #ifdef WITHGRASS


    // if GRASS is used, writing data to segment files:

        for ( k=nvect_red; k<nvect_all; k++ ) { // loop over all components of the vector:

            for ( i=0; i<sico.IMAX; i++ ) { // loop over all pixels:

                if ( cdomain[i] != 0 ) { // if pixel is not a 1st row edge pixel:

                    if ( sico.MODEL == 1 && k == 8 ) pout = fconvout( i, aw, hflow_maxmax, 1, 3,
                        px, py, pelev, betax, betay, betaxy, sico );
                    else if ( sico.MODEL == 2 && k == 18 ) pout = fconvout( i, aw, hflow_maxmax, 1, 3,
                        px, py, pelev, betax, betay, betaxy, sico );
                    else if ( sico.MODEL == 2 && k == 20 ) pout = fconvout( i, aw, hflow_maxmax, 2, 3,
                        px, py, pelev, betax, betay, betaxy, sico );
                    else if ( sico.MODEL == 2 && k == 22 ) pout = fconvout( i, aw, hflow_maxmax, 3, 3,
                        px, py, pelev, betax, betay, betaxy, sico );
                    else pout = aw[i][k]; // converting depths into heights
                }
                else pout = aw[i][k]; // for 1st row edge pixels, applying depths as heights

                Segment_put(&seg_v[k], &pout, px[i], py[i]); // writing values to segment file
            }
        }


    #endif


    for ( k=nvect_red; k<nvect_all; k++ ) { // loop over all elements of vector:

        if ( sico.MULT == 0 ) sprintf( mv, "%s%s", prefix, mv0[k] );
        else sprintf( mv, "%s%s%d", prefix, mv0[k], xint ); // name of output raster map


        #ifdef WITHGRASS


            Segment_flush(&seg_v[k] ); // forcing to write values to segment file immediately
            foutrast ( mv, seg_v[k], sico ); // writing GRASS raster maps


        #endif


        foutasc ( aw, px, py, outmaps, mv, hflow_maxmax, pelev, betax, betay, betaxy, k, sico ); // writing Ascii raster maps
    }


    #ifdef WITHGRASS


    // Releasing segment data (if GRASS is used)

        if ( sico.TOPO == 2 ) Segment_release(&seg_aoi);
        Segment_release(&seg_elev);
        if ( sico.RELM == 1 ) Segment_release(&seg_hrelease);
        if ( sico.MODEL == 2 && sico.RELM2 == 1 ) Segment_release(&seg_hrelease2);
        if ( sico.ENTR == 1 ) Segment_release(&seg_hentrmax);
        if ( sico.MODEL == 2 && sico.ENTR2 == 1 ) Segment_release(&seg_hentrmax2);
        if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) Segment_release(&seg_centr);
        if ( sico.DELTA == 1 ) Segment_release(&seg_delta);
        for ( k=0; k<nvect_all; k++ ) Segment_release(&seg_v[k]);
        free ( seg_v );


    #endif


// Closing output files

    fclose ( f_summary );
    if ( hydrograph == 1 ) {

        fclose ( f_hydout );
        for ( i = 0; i < hydnin + hydnout; i++ ) {
            if ( sico.MULT == 0 ) fclose( f_hydinfo[i] );
        }
    }

    if ( sico.MULT == 0 ) fclose ( f_directions );
    if ( sico.MULT == 0 && sico.MODEL == 2 ) fclose ( f_directions2 );


// Releasing memory


    #ifdef WITHGRASS


        free(v);


    #endif


    free(flowpar); free(phrelease); free(phentrmax); free(dentr); free(cflux); free(cdomain); free(cdomain2); free(cstopped);
    if ( sico.MODEL == 2 ) { free(phrelease2); free(phentrmax2); free(dentr2); }
    if ( sico.STOPPING != 0 ) free(fom);
    free(aw[0]); free(awt[0]); free(af[0]); free(ag[0]); free(as[0]);
    free(asigma_x[0]); free(asigma_y[0]); free(asigma_f[0]); free(asigma_g[0]);
    free(winta[0]); free(wintb[0]); free(wintd[0]); free(f[0]); free(g[0]); free(s[0]);
    free(aw); free(awt); free(af); free(ag); free(as); free(asigma_x); free(asigma_y); free(asigma_f); free(asigma_g);
    free(winta); free(wintb); free(wintd); free(f); free(g); free(s); free(mv0);
    free_dmatrix3(wintc, sico.IMAX, 4);
    if ( hydrograph == 1 ) {
        free_dmatrix3(hydhyd, hydtmaxx+1, 5);
        free(hydp[0]), free(hydp);
        free(hydelev); free(hydalpha); free(hydx_metric); free(hydy_metric); free(hydl); free(hydtmax); free(hydi); free(hydx); free(hydy);
        for ( hydj=0; hydj<hydnin; hydj++ ) free(hydreleasename[hydj]);
    }
    free(pelev); free(pelev0); free(px); free(py); free(dx); free(dy); free(betax); free(betay); free(betaxy);
    free(madd); free(prefix); free(outmaps); free(outfiles); free(elevname);
    if ( sico.RELM == 1 ) free(hreleasename); if ( sico.ENTR == 1 ) free(hentrmaxname); free(sico.MAINMAPSET);
    if ( sico.ENTRAINMENT == 1 && sico.CENTR == 1 ) free(centrname);
    if ( sico.DELTA == 1 ) free(deltaname); if ( sico.TOPO == 2 ) free(aoiname);
    if ( sico.MODEL == 2 ) { if ( sico.RELM2 == 1 ) free(hreleasename2); if ( sico.ENTR2 == 1 ) free(hentrmaxname2); }
    if ( sico.DIFFCTRL == 1 ) { free(cedge[0]); free(cready[0]); free(cedge); free(cready); free(cedge0); free(cneighbours); }
    if ( sico.DIFFCTRL == 1 && sico.MODEL == 2 ) {
        free(cedge2[0]); free(cready2[0]); free(cedge2); free(cready2); free(cedge02); free(cneighbours2);
    }


// Computing and displaying time needed for model execution

    time_stop = clock(); // time at end of model execution
    time_elapsed = ( ( float ) ( time_stop - time_start ) ) / CLOCKS_PER_SEC; // time needed for model execution
    printf("Model execution completed in %.2f seconds.\n", time_elapsed);
    fflush(stdout); // forcing immediate display

    return 0;
}

